@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.4 2006/11/07 19:48:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.8 2007/10/10 18:59:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -48,17 +48,22 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/c
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
 #include "utils/main.h"		/* for Path and CellLibPath */
+#include "utils/stack.h"
 
     /* Exports */
 bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
 bool CalmaDoLower = TRUE;	 /* If TRUE, allow lowercase labels. */
 bool CalmaFlattenArrays = FALSE; /* If TRUE, output arrays as individual uses */
+
+    /* Experimental stuff---not thoroughly tested (as of Sept. 2007)! */
 bool CalmaContactArrays = FALSE; /* If TRUE, output contacts as subcell arrays */
+bool CalmaMergeTiles = FALSE;	 /* If TRUE, merge tiles into polygons in output. */
 
     /* Forward declarations */
 extern int calmaWriteInitFunc();
 extern int calmaWriteMarkFunc();
 extern int calmaWritePaintFunc();
+extern int calmaMergePaintFunc();
 extern int calmaWriteUseFunc();
 extern void calmaWriteContacts();
 extern void calmaDelContacts();
@@ -70,6 +75,10 @@ extern void calmaOutDate();
 extern void calmaOutStringRecord();
 extern void calmaOut8();
 extern void calmaOutR8();
+extern void calmaProcessBoundary();
+extern void calmaMergeBoundaries();
+extern void calmaRemoveColinear();
+extern void calmaRemoveDegenerate();
 
 /* Structure used by calmaWritePaintFunc() */
 
@@ -78,6 +87,38 @@ typedef struct {
    Rect *area;		/* Clipping area, in GDS coordinates	*/
 } calmaOutputStruct;
 
+/*--------------------------------------------------------------*/
+/* Structures used by the tile merging algorithm 		*/
+/*--------------------------------------------------------------*/
+
+#define GDS_PENDING	0
+#define GDS_UNPROCESSED CLIENTDEFAULT
+#define GDS_PROCESSED	1
+
+#define PUSHTILE(tp) \
+    if ((tp)->ti_client == (ClientData) GDS_UNPROCESSED) { \
+	(tp)->ti_client = (ClientData) GDS_PENDING; \
+	STACKPUSH((ClientData) (tp), SegStack); \
+    }
+
+#define LB_EXTERNAL	0	/* Polygon external edge	*/
+#define LB_INTERNAL	1	/* Polygon internal edge	*/
+#define LB_INIT		2	/* Data not yet valid		*/
+
+typedef struct LB1 {
+    char lb_type;		/* Boundary Type (external or internal) */
+    Point lb_start;		/* Start point */
+    struct LB1 *lb_next;	/* Next point record */
+} LinkedBoundary;
+
+typedef struct BT1 {
+    LinkedBoundary *bt_first;   /* Polygon list */
+    int bt_points;		/* Number of points in this list */
+    struct BT1 *bt_next;	/* Next polygon record */
+} BoundaryTop;
+    
+/*--------------------------------------------------------------*/
+
 /* Number assigned to each cell */
 int calmaCellNum;
 
@@ -585,10 +626,10 @@ calmaOutFunc(def, f, cliprect)
 	calmaPaintLayerNumber = layer->cl_calmanum;
 	calmaPaintLayerType = layer->cl_calmatype;
 
-
-	(void) DBSrPaintArea((Tile *) NULL, CIFPlanes[type],
-	    cliprect, &CIFSolidBits, calmaWritePaintFunc,
-	    (ClientData) &cos);
+	DBSrPaintArea((Tile *) NULL, CIFPlanes[type],
+		cliprect, &CIFSolidBits, (CalmaMergeTiles) ?
+		calmaMergePaintFunc : calmaWritePaintFunc,
+		(ClientData) &cos);
     }
 
     /* Output labels */
@@ -1170,6 +1211,772 @@ calmaDelContacts()
 	}
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * calmaAddSegment ---
+ *
+ * Process a new polygon edge, inserting it into a polygon record as
+ * required.  If the edge is between a GDS layer and TT_SPACE, then
+ * we insert a point record.  If the edge is between two tiles of the
+ * same layer, then we insert a tile record.
+ *
+ * Results:
+ *	Return 1 if an internal segment was generated, 0 if an external
+ *	segment was generate.  On error, return -1 (failure to find a
+ *	connecting point; this shouldn't happen).
+ *
+ *	Returns the current segment in the original pointer position (1st
+ *	argument).  If segments are added in counterclockwise order, then
+ *	this should be most efficient.
+ *
+ * Side effects:
+ *	May allocate memory. 
+ * ---------------------------------------------------------------------------
+ */
+
+int
+calmaAddSegment(lbptr, poly_edge, p1x, p1y, p2x, p2y)
+    LinkedBoundary **lbptr;
+    bool poly_edge;
+    int p1x, p1y, p2x, p2y;
+{
+    LinkedBoundary *newseg, *curseg, *stopseg;
+    bool startmatch = FALSE;
+    bool endmatch = FALSE;
+
+    stopseg = NULL;
+    for (curseg = *lbptr; curseg != stopseg; curseg = curseg->lb_next)
+    {
+	stopseg = *lbptr;
+	if (curseg->lb_type == LB_INIT)
+	{
+	    if ((p1x == curseg->lb_start.p_x) && (p1y == curseg->lb_start.p_y))
+		startmatch = TRUE;
+
+	    if ((p2x == curseg->lb_next->lb_start.p_x) &&
+			(p2y == curseg->lb_next->lb_start.p_y))
+		endmatch = TRUE;
+
+	    if (startmatch && endmatch)
+	    {
+		/* Segment completes this edge */
+		curseg->lb_type = (poly_edge) ? LB_EXTERNAL : LB_INTERNAL;
+		*lbptr = curseg;
+		return (int)curseg->lb_type;
+	    }
+	    else if (startmatch || endmatch)
+	    {
+		/* Insert a new segment after curseg */
+		newseg = (LinkedBoundary *)mallocMagic(sizeof(LinkedBoundary));
+		newseg->lb_next = curseg->lb_next;
+		curseg->lb_next = newseg;
+
+		if (startmatch)
+		{
+		    newseg->lb_type = curseg->lb_type;
+		    curseg->lb_type = (poly_edge) ? LB_EXTERNAL : LB_INTERNAL;
+		    newseg->lb_start.p_x = p2x;
+		    newseg->lb_start.p_y = p2y;
+		}
+		else
+		{
+		    newseg->lb_type = (poly_edge) ? LB_EXTERNAL : LB_INTERNAL;
+		    newseg->lb_start.p_x = p1x;
+		    newseg->lb_start.p_y = p1y;
+		}
+		curseg = newseg;
+		*lbptr = curseg;
+		return (int)curseg->lb_type;
+	    }
+	}
+    }
+    return -1;		/* This shouldn't happen, but isn't fatal. */
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * calmaRemoveDegenerate ---
+ *
+ *    This routine takes lists of polygons and removes any degenerate
+ *    segments (those that backtrack on themselves) from each one.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Deallocates memory for any segments that are removed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaRemoveDegenerate(blist)
+    BoundaryTop *blist;
+{
+    bool segfound;
+    LinkedBoundary *stopseg, *curseg, *lastseg;
+    BoundaryTop *bounds;
+
+    for (bounds = blist; bounds != NULL; bounds = bounds->bt_next)
+    {
+	segfound = TRUE;
+	while (segfound)
+	{
+	    segfound = FALSE;
+	    stopseg = NULL;
+	    for (lastseg = bounds->bt_first; lastseg != stopseg;)
+	    {
+		stopseg = bounds->bt_first;
+		curseg = lastseg->lb_next;
+
+		if (GEO_SAMEPOINT(curseg->lb_start,
+			curseg->lb_next->lb_next->lb_start))
+		{
+		    segfound = TRUE;
+		    lastseg->lb_next = curseg->lb_next->lb_next;
+
+		    freeMagic(curseg->lb_next);
+		    freeMagic(curseg);
+	    
+		    /* Make sure record doesn't point to a free'd segment */
+		    bounds->bt_first = lastseg;
+		    bounds->bt_points -= 2;
+		    break;
+		}
+		else
+		    lastseg = lastseg->lb_next;
+	    }
+	}
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * calmaRemoveColinear ---
+ *
+ *    This routine takes lists of polygons and removes any redundant
+ *    (colinear) points.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Deallocates memory for any segments that are removed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaRemoveColinear(blist)
+    BoundaryTop *blist;
+{
+    LinkedBoundary *stopseg, *curseg, *lastseg;
+    BoundaryTop *bounds;
+
+    for (bounds = blist; bounds != NULL; bounds = bounds->bt_next)
+    {
+	stopseg = NULL;
+	for (lastseg = bounds->bt_first; lastseg != stopseg;)
+	{
+	    stopseg = bounds->bt_first;
+	    curseg = lastseg->lb_next;
+
+	    if (((lastseg->lb_start.p_x == curseg->lb_start.p_x) &&
+		(lastseg->lb_start.p_x == curseg->lb_next->lb_start.p_x)) ||
+		((lastseg->lb_start.p_y == curseg->lb_start.p_y) &&
+		(lastseg->lb_start.p_y == curseg->lb_next->lb_start.p_y)))
+	    {
+		lastseg->lb_next = curseg->lb_next;
+	    
+		/* Make sure record doesn't point to a free'd segment */
+		if (curseg == bounds->bt_first) bounds->bt_first = lastseg;
+
+		freeMagic(curseg);
+		bounds->bt_points--;
+	    }
+	    else if ((lastseg->lb_start.p_x != curseg->lb_start.p_x) &&
+		(lastseg->lb_start.p_y != curseg->lb_start.p_y) &&
+		(curseg->lb_start.p_x != curseg->lb_next->lb_start.p_x) &&
+		(curseg->lb_start.p_y != curseg->lb_next->lb_start.p_y))
+	    {
+		/* Check colinearity of non-Manhattan edges */
+		int delx1, dely1, delx2, dely2, gcf;
+
+		delx1 = curseg->lb_start.p_x - lastseg->lb_start.p_x;
+		dely1 = curseg->lb_start.p_y - lastseg->lb_start.p_y;
+		delx2 = curseg->lb_next->lb_start.p_x - curseg->lb_start.p_x;
+		dely2 = curseg->lb_next->lb_start.p_y - curseg->lb_start.p_y;
+
+		if ((delx1 != delx2) || (dely1 != dely2))
+		{
+		    gcf = FindGCF(delx1, dely1);
+		    if (gcf > 1)
+		    {
+			delx1 /= gcf;
+			dely1 /= gcf;
+		    }
+		}
+		if ((delx1 != delx2) || (dely1 != dely2))
+		{
+		    gcf = FindGCF(delx2, dely2);
+		    if (gcf > 1)
+		    {
+			delx2 /= gcf;
+			dely2 /= gcf;
+		    }
+		}
+		if ((delx1 == delx2) && (dely1 == dely2))
+	 	{
+		    lastseg->lb_next = curseg->lb_next;
+		    if (curseg == bounds->bt_first) bounds->bt_first = lastseg;
+		    freeMagic(curseg);
+		    bounds->bt_points--;
+		}
+		else
+		    lastseg = lastseg->lb_next;
+	    }
+	    else
+		lastseg = lastseg->lb_next;
+	}
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * calmaMergeSegments ---
+ *
+ *    Once a tile has been disassembled into segments, and it is not a simple
+ *    rectangle (which would have been handled already), then merge it into
+ *    the list of boundaries.
+ *
+ *    Note that this algorithm is O(N^2) and has lots of room for improvement!
+ *    Still, each segment is never checked against more than 200 points,
+ *    because when a boundary reaches this number (the maximum for GDS
+ *    boundary records), the record will tend to be skipped (it should
+ *    probably be output here. . .) 
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Output, memory allocation and deallocation
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaMergeSegments(edge, blist, num_points)
+    LinkedBoundary *edge;    
+    BoundaryTop **blist;
+    int num_points;
+{
+    LinkedBoundary *stopseg, *curseg, *lastseg;
+    LinkedBoundary *compstop, *compseg, *complast;
+    BoundaryTop *bounds, *newbounds;
+
+    if (*blist == NULL) goto make_new_bound;
+
+    /* Check each internal edge for an antiparallel match with	*/
+    /* an internal edge in the boundary lists.			*/
+
+    stopseg = NULL;
+    for (lastseg = edge; lastseg != stopseg; lastseg = lastseg->lb_next)
+    {
+	stopseg = edge;
+	curseg = lastseg->lb_next;
+	if (curseg->lb_type == LB_EXTERNAL) continue;
+
+	for (bounds = *blist; bounds != NULL; bounds = bounds->bt_next)
+	{
+	    /* Avoid overflow on GDS boundary point limit.  Note	*/
+	    /* that a merge will remove 2 points, but GDS requires	*/
+	    /* that we add the 1st point to the end of the list.	*/
+
+	    if (bounds->bt_points + num_points > 201) continue;
+
+	    compstop = NULL;
+	    for (complast = bounds->bt_first; complast != compstop;
+			complast = complast->lb_next)
+	    {
+	        compstop = bounds->bt_first;
+		compseg = complast->lb_next;
+		if (compseg->lb_type == LB_EXTERNAL) continue;
+
+		/* Edges match antiparallel only. Rect points are *not*	*/
+		/* canonical. r_ll and p1 are both 1st points traveling	*/
+		/* in a counterclockwise direction along the perimeter.	*/
+
+		if (GEO_SAMEPOINT(compseg->lb_start, curseg->lb_next->lb_start) &&
+		   GEO_SAMEPOINT(compseg->lb_next->lb_start, curseg->lb_start))
+		{
+		    lastseg->lb_next = compseg->lb_next;
+		    complast->lb_next = curseg->lb_next;
+
+		    freeMagic(compseg);
+		    freeMagic(curseg);
+	
+		    /* Make sure the record doesn't point to the free'd segment */
+		    if (compseg == bounds->bt_first) bounds->bt_first = complast;
+		    bounds->bt_points += num_points - 2;
+		    return;
+		}
+	    }
+	}
+    }
+
+    /* If still no connecting edge was found, or if we overflowed the GDS max	*/
+    /* number of records for a boundary, then start a new entry.		*/ 
+
+make_new_bound:
+
+    newbounds = (BoundaryTop *)mallocMagic(sizeof(BoundaryTop));
+    newbounds->bt_first = edge;
+    newbounds->bt_next = *blist;
+    newbounds->bt_points = num_points;
+    *blist = newbounds;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * Process a LinkedBoundary list into a polygon and generate GDS output.
+ * Free the linked list when done.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Output, memory deallocation.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaProcessBoundary(blist, cos)
+    BoundaryTop *blist;
+    calmaOutputStruct *cos;
+{
+    FILE *f = cos->f;
+    LinkedBoundary *listtop, *lbref, *lbstop, *lbfree;
+    BoundaryTop *bounds;
+    int sval;
+    int chkcount;	/* diagnostic */
+
+    for (bounds = blist; bounds != NULL; bounds = bounds->bt_next)
+    {
+	/* Boundary */
+	calmaOutRH(4, CALMA_BOUNDARY, CALMA_NODATA, f);
+
+	/* Layer */
+	calmaOutRH(6, CALMA_LAYER, CALMA_I2, f);
+	calmaOutI2(calmaPaintLayerNumber, f);
+
+	/* Data type */
+	calmaOutRH(6, CALMA_DATATYPE, CALMA_I2, f);
+	calmaOutI2(calmaPaintLayerType, f);
+
+	/* Record length = ((#points + 1) * 2 values * 4 bytes) + 4 bytes header */
+	calmaOutRH(4 + (bounds->bt_points + 1) * 8, CALMA_XY, CALMA_I4, f);
+
+	/* Coordinates (repeat 1st point) */
+
+	listtop = bounds->bt_first;
+	lbstop = NULL;
+	chkcount = 0;
+	for (lbref = listtop; lbref != lbstop; lbref = lbref->lb_next)
+	{
+	    lbstop = listtop;
+	    calmaOutI4(lbref->lb_start.p_x * calmaPaintScale, f);
+	    calmaOutI4(lbref->lb_start.p_y * calmaPaintScale, f);
+	    chkcount++;
+	}
+	calmaOutI4(listtop->lb_start.p_x * calmaPaintScale, f);
+	calmaOutI4(listtop->lb_start.p_y * calmaPaintScale, f);
+
+	if (chkcount != bounds->bt_points)
+	    TxError("Points recorded=%d;  Points written=%d\n",
+			bounds->bt_points, chkcount);
+
+	/* End of element */
+	calmaOutRH(4, CALMA_ENDEL, CALMA_NODATA, f);
+
+#ifdef DEBUG
+	/* Diagnostic: report the contents of the list */
+	TxPrintf("Polygon path (%d points):\n", bounds->bt_points);
+
+	listtop = bounds->bt_first;
+	lbstop = NULL;
+
+	for (lbref = listtop; lbref != lbstop; lbref = lbref->lb_next)
+	{
+	    if (lbref != listtop)
+		TxPrintf("->");
+	    else 
+		lbstop = listtop;
+
+	    switch(lbref->lb_type)
+	    {
+		case LB_EXTERNAL:
+		    TxPrintf("(%d %d)", lbref->lb_start.p_x, lbref->lb_start.p_y);
+		    break;
+		case LB_INTERNAL:
+		    TxPrintf("[[%d %d]]", lbref->lb_start.p_x, lbref->lb_start.p_y);
+		    break;
+		case LB_INIT:
+		    TxPrintf("XXXXX");
+		    break;
+	    }
+	}
+	TxPrintf("\n\n");
+#endif
+
+	/* Free the LinkedBoundary list */
+
+	lbref = listtop;
+	while (lbref->lb_next != listtop)
+	{
+	    freeMagic(lbref);
+	    lbref = lbref->lb_next;
+	}
+	freeMagic(lbref);
+    }
+
+    /* Free the BoundaryTop list */
+
+    for (bounds = blist; bounds != NULL; bounds = bounds->bt_next)
+	freeMagic(bounds);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaMergePaintFunc --
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Writes to the disk file.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+calmaMergePaintFunc(tile, cos)
+    Tile *tile;			/* Tile to be written out. */
+    calmaOutputStruct *cos;	/* Information needed by algorithm */
+{
+    FILE *f = cos->f;
+    Rect *clipArea = cos->area;
+    Tile *t, *tp;
+    TileType ttype;
+    int i, llx, lly, urx, ury, intedges, num_points, split_type;
+    bool is_ext;
+    static Stack *SegStack = (Stack *)NULL;
+
+    static LinkedBoundary *edge;
+    LinkedBoundary *lb;
+    BoundaryTop *bounds = NULL;
+
+    /* Quick check for tiles that have already been processed */
+    if (tile->ti_client == (ClientData)GDS_PROCESSED) return 0;
+
+    if (SegStack == (Stack *)NULL)
+	SegStack = StackNew(64);
+
+    PUSHTILE(tile);
+    while (!StackEmpty(SegStack))
+    {
+	t = (Tile *) STACKPOP(SegStack);
+	if (t->ti_client != (ClientData)GDS_PENDING) continue;
+	t->ti_client = (ClientData)GDS_PROCESSED;
+
+#ifdef NONMANHATTAN
+	split_type = -1;
+	if (IsSplit(t))
+	{
+	    /* If we use SplitSide, then we need to set it when	the	*/
+	    /* tile is pushed.  Since these are one-or-zero mask layers	*/
+	    /* I assume it is okay to just check which side is TT_SPACE	*/
+
+	    /* split_type = (SplitSide(t) << 1) | SplitDirection(t); */
+	    split_type = SplitDirection(t);
+	    if (TiGetLeftType(t) == TT_SPACE) split_type |= 2;
+	    num_points = 2;
+	    if (edge != NULL)
+	    {
+		/* Remove one point from the edge record for rectangles */
+		/* and relink the last entry back to the new head.	*/
+
+		lb = edge;
+		while (lb->lb_next != edge) lb = lb->lb_next;
+		lb->lb_next = edge->lb_next;
+	 	freeMagic(edge);
+		edge = edge->lb_next;
+	    }
+	}
+	else
+#endif
+	    num_points = 3;
+
+	/* Create a new linked boundary structure with 4 unknown edges.	*/
+	/* This structure is reused when we encounter isolated tiles,	*/
+	/* so we avoid unnecessary overhead in the case of, for		*/
+	/* example, large contact cut arrays.				*/
+
+	if (edge == NULL)
+	{
+	    edge = (LinkedBoundary *)mallocMagic(sizeof(LinkedBoundary));
+	    lb = edge;
+
+	    for (i = 0; i < num_points; i++)
+	    {
+		lb->lb_type = LB_INIT;
+		lb->lb_next = (LinkedBoundary *)mallocMagic(sizeof(LinkedBoundary));
+		lb = lb->lb_next;
+	    }
+	    lb->lb_type = LB_INIT;
+	    lb->lb_next = edge;
+	}
+
+	lb = edge;
+	llx = LEFT(t);
+	lly = BOTTOM(t);
+	urx = RIGHT(t);
+	ury = TOP(t);
+	intedges = 0;
+
+	/* Initialize the "edge" record with the corner points of the	*/
+	/* tile.							*/
+
+#ifdef NONMANHATTAN
+	if (IsSplit(t))
+	{
+	    switch (split_type)
+	    {
+		case 0x0:
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    break;
+		case 0x1:
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    break;
+		case 0x2:
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    break;
+		case 0x3:
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = llx;
+		    lb->lb_start.p_y = ury;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    lb->lb_start.p_x = urx;
+		    lb->lb_start.p_y = lly;
+		    lb->lb_type = LB_INIT;
+		    lb = lb->lb_next;
+		    break;
+	    }
+	    num_points = 1;
+	}
+	else
+	{
+#endif
+	    lb->lb_start.p_x = urx;
+	    lb->lb_start.p_y = ury;
+	    lb->lb_type = LB_INIT;
+	    lb = lb->lb_next;
+	    lb->lb_start.p_x = llx;
+	    lb->lb_start.p_y = ury;
+	    lb->lb_type = LB_INIT;
+	    lb = lb->lb_next;
+	    lb->lb_start.p_x = llx;
+	    lb->lb_start.p_y = lly;
+	    lb->lb_type = LB_INIT;
+	    lb = lb->lb_next;
+	    lb->lb_start.p_x = urx;
+	    lb->lb_start.p_y = lly;
+	    lb->lb_type = LB_INIT;
+	    lb = lb->lb_next;
+
+	    num_points = 0;
+
+#ifdef NONMANHATTAN
+	}
+	if (split_type == 0x1) goto left_search;
+#endif
+
+	/* Search the tile boundary for connected and unconnected tiles.	*/
+	/* Generate segments in a counterclockwise cycle.			*/
+
+#ifdef NONMANHATTAN
+	if (split_type == 0x2)
+	{
+	    intedges += calmaAddSegment(&lb, TRUE, RIGHT(t), TOP(t),
+			LEFT(t), BOTTOM(t));
+	    goto bottom_search;
+	}
+#endif
+
+	/* Search top */
+
+	ttype = TiGetTopType(t);
+	for (tp = RT(t); RIGHT(tp) > LEFT(t); tp = BL(tp), num_points++)
+	{
+	    is_ext = (TiGetBottomType(tp) != ttype) ? TRUE : FALSE;
+	    intedges += calmaAddSegment(&lb, is_ext,
+			MIN(RIGHT(t), RIGHT(tp)), TOP(t),
+			MAX(LEFT(t), LEFT(tp)), TOP(t));
+	    if (!is_ext) PUSHTILE(tp);
+	}
+
+#ifdef NONMANHATTAN
+	if (split_type == 0x3)
+	{
+	    intedges += calmaAddSegment(&lb, TRUE, LEFT(t), TOP(t),
+			RIGHT(t), BOTTOM(t));
+	    goto right_search;
+	}
+#endif
+
+	/* Search left */
+
+left_search:
+	ttype = TiGetLeftType(t);
+	for (tp = BL(t); BOTTOM(tp) < TOP(t); tp = RT(tp), num_points++)
+	{
+	    is_ext = (TiGetRightType(tp) != ttype) ? TRUE : FALSE;
+	    intedges += calmaAddSegment(&lb, is_ext,
+			LEFT(t), MIN(TOP(t), TOP(tp)),
+			LEFT(t), MAX(BOTTOM(t), BOTTOM(tp)));
+	    if (!is_ext) PUSHTILE(tp);
+	}
+
+#ifdef NONMANHATTAN
+	if (split_type == 0x0)
+	{
+	    intedges += calmaAddSegment(&lb, TRUE, LEFT(t), BOTTOM(t),
+			RIGHT(t), TOP(t));
+	    goto done_searches;
+	}
+#endif
+
+	/* Search bottom */
+
+bottom_search:
+	ttype = TiGetBottomType(t);
+	for (tp = LB(t); LEFT(tp) < RIGHT(t); tp = TR(tp), num_points++)
+	{
+	    is_ext = (TiGetTopType(tp) != ttype) ? TRUE : FALSE;
+	    intedges += calmaAddSegment(&lb, is_ext,
+			MAX(LEFT(t), LEFT(tp)), BOTTOM(t),
+			MIN(RIGHT(t), RIGHT(tp)), BOTTOM(t));
+	    if (!is_ext) PUSHTILE(tp);
+	}
+
+#ifdef NONMANHATTAN
+	if (split_type == 0x1)
+	{
+	    intedges += calmaAddSegment(&lb, TRUE, RIGHT(t), BOTTOM(t),
+			LEFT(t), TOP(t));
+	    goto done_searches;
+	}
+#endif
+	/* Search right */
+
+right_search:
+	ttype = TiGetRightType(t);
+	for (tp = TR(t); TOP(tp) > BOTTOM(t); tp = LB(tp), num_points++)
+	{
+	    is_ext = (TiGetLeftType(tp) != ttype) ? TRUE : FALSE;
+	    intedges += calmaAddSegment(&lb, is_ext,
+			RIGHT(t), MAX(BOTTOM(t), BOTTOM(tp)),
+			RIGHT(t), MIN(TOP(t), TOP(tp)));
+	    if (!is_ext) PUSHTILE(tp);
+	}
+
+	/* If tile is isolated, process it now and we're done */
+
+done_searches:
+	if (intedges == 0)
+	{
+	    calmaWritePaintFunc(t, cos);
+
+	    /* Although calmaWritePaintFunc is called only on isolated	*/
+	    /* tiles, we may have expanded it.  This could use a LOT of	*/
+	    /* optimizing.  1) remove colinear points in calmaAddSegment */
+	    /* when both subsegments are external paths, and 2) here,	*/
+	    /* take the shortest path to making "edge" exactly 4 points.*/	
+	    /* Note that in non-Manhattan mode, num_points may be 3.	*/
+
+	    if (num_points != 4)
+	    {
+		for (i = 0; i < num_points; i++)
+		{
+		    freeMagic(edge);
+		    edge = edge->lb_next;
+		}
+		edge = NULL;
+	    }
+	    if (!StackEmpty(SegStack))
+		TxError("ERROR:  Segment stack is supposed to be empty!\n");
+	    else
+		return 0;
+	}
+	else
+	{
+	    /* Merge boundary into existing record */
+
+	    calmaMergeSegments(edge, &bounds, num_points);
+	    edge = NULL;
+	}
+    }
+
+    /* Remove any degenerate points */
+    calmaRemoveDegenerate(bounds);
+
+    /* Remove any colinear points */
+    calmaRemoveColinear(bounds);
+
+    /* Output the boundary records */
+    calmaProcessBoundary(bounds, cos);
+
+    return 0;	/* Keep the search alive. . . */
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1458,6 +2265,7 @@ calmaOutStringRecord(type, str, f)
     int len;
     unsigned char c;
     char *table, *locstr, *origstr = NULL;
+    char *locstrprv; 	/* Added by BSI */
 
     if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
     {
@@ -1482,6 +2290,7 @@ calmaOutStringRecord(type, str, f)
     /* Output the string itself */
     while (len--)
     {
+	locstrprv = locstr;
 	c = (unsigned char) *locstr++;
 	if (c == 0) putc('\0', f);
 	else
@@ -1498,6 +2307,7 @@ calmaOutStringRecord(type, str, f)
 		    origstr = StrDup(NULL, locstr);
 
 		c = table[c];
+		locstrprv[0] = c;
 	    }
 	    if (!CalmaDoLower && islower(c))
 		(void) putc(toupper(c), f);
