@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFrdutils.c,v 1.3 2006/11/02 23:44:07 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFrdutils.c,v 1.4 2007/08/23 21:22:53 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -851,26 +851,36 @@ path_intersect(pathHead, start, respt)
 	    diagonal = TRUE;
 	}
 
-	/* Otherwise, is the point inside the triangle (path must 	*/
+	/* Is the point inside the triangle (path must 			*/
 	/* be Manhattan in this case, unless the path crosses itself,	*/
 	/* which would be extremely bad CIF to begin with).		*/
+	/* (Note that *both* tests can be true, in which case the one	*/
+	/* with the smaller absolute distance takes precedence.)	*/
 
-	else if (test_insideness(start, &path->cifp_point)) {
+	if (test_insideness(start, &path->cifp_point)) {
+	    int tmpdist = abs(newdist);		/* save this value */	
 	    if (start->cifp_x == start->cifp_next->cifp_x)
 	    {
 		newdist = path->cifp_y - start->cifp_y;
-		tmppt.p_x = start->cifp_x;
-		tmppt.p_y = path->cifp_y;
+		if (abs(newdist) < tmpdist)
+		{
+		    tmppt.p_x = start->cifp_x;
+		    tmppt.p_y = path->cifp_y;
+		    diagonal = FALSE;
+		}
 	    }
 	    else
 	    {
 		newdist = path->cifp_x - start->cifp_x;
-		tmppt.p_y = start->cifp_y;
-		tmppt.p_x = path->cifp_x;
+		if (abs(newdist) < tmpdist)
+		{
+		    tmppt.p_y = start->cifp_y;
+		    tmppt.p_x = path->cifp_x;
+		    diagonal = FALSE;
+		}
 	    }
-	    diagonal = FALSE;
 	}
-	else
+	else if (diagonal == FALSE)
 	    continue;
 
 	newdist = abs(newdist);
