@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.4 2006/11/18 00:55:29 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.7 2007/10/10 18:59:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -40,6 +40,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "textio/textio.h"
 #include "utils/malloc.h"
 #include "cif/cif.h"
+#include "drc/drc.h"	/* For WRL's DRC-CIF extensions */
 
 /* The following statics are used to keep track of things between
  * calls to CIFTechLine.
@@ -1287,7 +1288,7 @@ bloatCheck:
 		    goto errorReturn;
 		}
 	    }
-	    if ((argc < 4) || (argc == 6) || (argc == 7) || (argc > 8))
+	    if ((argc < 4) || (argc == 6) || (argc > 8))
 		goto wrongNumArgs;
 	    break;
     }
@@ -1439,6 +1440,46 @@ cifComputeRadii(layer, des)
 	grow, shrink);
      */
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * cifComputeHalo --
+ *
+ * Compute grow and shrink distances for each layer, and remember the
+ * largest.  Convert from CIF/GDS to Magic internal dimensions.
+ *
+ * Results:  None.
+ * Side effects:  Sets cs_radius value in the current cifoutput style.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+cifComputeHalo(style)
+    CIFStyle *style;
+{
+    int maxGrow, maxShrink, i;
+
+    maxGrow = maxShrink = 0;
+    for (i = 0; i < style->cs_nLayers; i++)
+    {
+	cifComputeRadii(style->cs_layers[i], style);
+	if (style->cs_layers[i]->cl_growDist > maxGrow)
+	    maxGrow = style->cs_layers[i]->cl_growDist;
+	if (style->cs_layers[i]->cl_shrinkDist > maxShrink)
+	    maxShrink = style->cs_layers[i]->cl_shrinkDist;
+    }
+    if (maxGrow > maxShrink)
+	style->cs_radius = 2*maxGrow;
+    else style->cs_radius = 2*maxShrink;
+    style->cs_radius /= style->cs_scaleFactor;
+    style->cs_radius++;
+
+    /* TxPrintf("Radius for %s CIF is %d.\n",
+     *  style->cs_name, style->cs_radius);
+     */
+}
+	
 
 /*
  * ----------------------------------------------------------------------------
@@ -1488,7 +1529,7 @@ CIFTechFinal()
 {
     CIFStyle *style = CIFCurStyle;
     CIFOp *op;
-    int i, maxGrow, maxShrink, minReduce;
+    int i, minReduce;
 
     /* Allow the case where there's no CIF at all.  This is indicated
      * by a NULL CIFCurStyle.
@@ -1659,25 +1700,8 @@ CIFTechFinal()
     /* Compute grow and shrink distances for each layer,
      * and remember the largest.
      */
-    maxGrow = maxShrink = 0;
-    for (i = 0; i < style->cs_nLayers; i++)
-    {
-	cifComputeRadii(style->cs_layers[i], style);
-	if (style->cs_layers[i]->cl_growDist > maxGrow)
-	    maxGrow = style->cs_layers[i]->cl_growDist;
-	if (style->cs_layers[i]->cl_shrinkDist > maxShrink)
-	    maxShrink = style->cs_layers[i]->cl_shrinkDist;
-    }
-    if (maxGrow > maxShrink)
-	style->cs_radius = 2*maxGrow;
-    else style->cs_radius = 2*maxShrink;
-    style->cs_radius /= style->cs_scaleFactor;
-    style->cs_radius += 1;
+    cifComputeHalo(style);
 
-    /* TxPrintf("Radius for %s CIF is %d.\n",
-     *  style->cs_name, style->cs_radius);
-     */
-	
     /* Go through the layers to see which ones depend on which
      * other ones.  The purpose of this is so that we don't
      * have to yank unnecessary layers in processing subcell
@@ -1822,6 +1846,12 @@ CIFLoadStyle(stylename)
 
     /* CIFTechFinal(); */  /* handled by TechLoad() */
     CIFTechOutputScale(DBLambda[0], DBLambda[1]);
+
+    /* If the DRC section makes reference to CIF layers, then	*/
+    /* we need to re-read the DRC section as well.		*/
+
+    if ((DRCForceReload == TRUE) && (DRCCurStyle != NULL))
+	DRCReloadCurStyle();
 }
 
 /*
@@ -2173,5 +2203,8 @@ CIFTechOutputScale(n, d)
 	    }
 	}
     }
+
+    /* Recompute value of cs_radius */
+    cifComputeHalo(ostyle);
 }
 
