@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.3 2006/11/11 00:35:01 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.6 2007/10/12 22:14:36 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -98,13 +98,14 @@ bool cmdDumpParseArgs();
 #define CALMA_ORDERING	4
 #define	CALMA_LABELS	5
 #define	CALMA_LOWER	6
-#define CALMA_READ	7
-#define CALMA_READONLY	8
-#define CALMA_RESCALE	9
-#define CALMA_WARNING	10
-#define CALMA_WRITE	11
+#define CALMA_MERGE	7
+#define CALMA_READ	8
+#define CALMA_READONLY	9
+#define CALMA_RESCALE	10
+#define CALMA_WARNING	11
+#define CALMA_WRITE	12
 #ifdef NONMANHATTAN
-#define CALMA_POLYS	12
+#define CALMA_POLYS	13
 #endif
 
 #define CALMA_WARN_HELP CIF_WARN_END	/* undefined by CIF module */
@@ -132,6 +133,7 @@ CmdCalma(w, cmd)
 	"ordering [on|off]	cause cells to be read in post-order",
 	"labels [yes|no]	cause labels to be output when writing GDS-II",
 	"lower [yes|no]		allow both upper and lower case in labels",
+	"merge [yes|no]		merge tiles into polygons in the output",
 	"read file		read Calma GDS-II format from \"file\"\n"
 	"		into edit cell",
 	"readonly [yes|no]	set cell as read-only and generate output from GDS file",
@@ -332,6 +334,28 @@ CmdCalma(w, cmd)
 	    CalmaDoLower = (option < 3) ? FALSE : TRUE;
 	    return;
 
+	case CALMA_MERGE:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaMergeTiles));
+#else
+		if (CalmaMergeTiles)
+		    TxPrintf("Merge connected tiles into polygons on output\n");
+		else
+		    TxPrintf("Output individual tiles\n");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaMergeTiles = (option < 3) ? FALSE : TRUE;
+	    return;
+
 #ifdef NONMANHATTAN
 	case CALMA_POLYS:
 	    if (cmd->tx_argc == 3)
@@ -502,6 +526,8 @@ outputCalma:
  *	cellname writeable [name] [true|false]
  * or
  *	cellname rename [name] [newname]
+ * or
+ *	cellname property [name] [property_key [property_value]]
  *
  * Results:
  *	None.
@@ -542,6 +568,7 @@ CmdCellname(w, cmd)
 	"create		create a new cell definition",
 	"delete		delete the named cell definition",
 	"flags		list option flags of the indicated cell definition",
+	"property	list or set cell definition properties",
 	"rename		rename the indicated cell",
 	"writeable	make the cell definition read-only or read-write",
 	NULL
@@ -549,7 +576,8 @@ CmdCellname(w, cmd)
     typedef enum { IDX_CHILDREN, IDX_PARENTS, IDX_EXISTS, IDX_SELF,
 		   IDX_INSTANCE, IDX_CHILDINST, IDX_CELLDEF, IDX_ALLCELLS,
 		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_CREATE,
-		   IDX_DELETE, IDX_FLAGS, IDX_RENAME, IDX_READWRITE } optionType;
+		   IDX_DELETE, IDX_FLAGS, IDX_PROPERTY, IDX_RENAME,
+		   IDX_READWRITE } optionType;
 
     if (strstr(cmd->tx_argv[0], "in"))
 	func = DBUsePrint;
@@ -563,13 +591,16 @@ CmdCellname(w, cmd)
 	    locargc--;
 	}
     }
-    if (locargc > 4 || locargc < 2) goto badusage;
+    if (locargc > 5 || locargc < 2) goto badusage;
 
     option = Lookup(cmd->tx_argv[1 + ((dolist) ? 1 : 0)], cmdCellOption);
     if (option < 0) goto badusage;
 
     if ((locargc > 3) && (option != IDX_RENAME) && (option != IDX_DELETE) &&
-		(option != IDX_READWRITE))
+		(option != IDX_READWRITE) && (option != IDX_PROPERTY))
+	goto badusage;
+
+    if ((locargc > 4) && (option != IDX_PROPERTY))
 	goto badusage;
 
     if (locargc >= 3) {
@@ -592,7 +623,7 @@ CmdCellname(w, cmd)
 	    case IDX_TOPCELLS:
 		TxError("Instances do not have a top level.  Use \"cellname\"?\n");
 		return;
-	    case IDX_IN_WINDOW: case IDX_READWRITE: case IDX_FLAGS:
+	    case IDX_IN_WINDOW: case IDX_READWRITE: case IDX_FLAGS: case IDX_PROPERTY:
 		TxError("Function unimplemented for instances.\n");
 		return;
 	    case IDX_DELETE:
@@ -636,6 +667,18 @@ CmdCellname(w, cmd)
 	case IDX_PARENTS:
 	    (*func)(cellname, PARENTS, dolist);
 	    break;
+	case IDX_PROPERTY:
+	    if (cellname == NULL)
+		cellDef = EditRootDef;
+	    else
+		cellDef = DBCellLookDef(cellname);
+
+	    if (cellDef == (CellDef *) NULL)
+		TxError("Unknown cell %s\n", cellname);
+	    else
+		CmdDoProperty(cellDef, cmd, 3 + ((dolist) ? 1 : 0));
+	    break;
+
 	case IDX_DELETE:
 	    /* Unload the cell definition and free memory */
 	    if ((locargc == 4) && !strcmp(cmd->tx_argv[3 + ((dolist) ? 1 : 0)],
@@ -3139,24 +3182,29 @@ CmdDrc(w, cmd)
 	    if (argc == 3)
 	    {
 		DRCTechHalo = cmdScaleCoord(w, argv[2], FALSE, TRUE);
-		DRCTechHalo *= DRCCurStyle->DRCScaleFactor;
+		DRCTechHalo *= DRCCurStyle->DRCScaleFactorN;
+		DRCTechHalo /= DRCCurStyle->DRCScaleFactorD;
 		if (DRCTechHalo > DRCCurStyle->DRCTechHalo)
 		    DRCTechHalo = DRCCurStyle->DRCTechHalo;
 
 		if (DRCTechHalo < DRCCurStyle->DRCTechHalo)
 		    TxPrintf("Warning: rulechecking limited to halo of %d lambda.\n",
-				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
+				DRCTechHalo * DRCCurStyle->DRCScaleFactorD
+				/ DRCCurStyle->DRCScaleFactorN);
 		else
 		    TxPrintf("DRC checks all rules (halo of %d lambda)\n",
-				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
+				DRCTechHalo * DRCCurStyle->DRCScaleFactorD
+				/ DRCCurStyle->DRCScaleFactorN);
 	    }
 	    else
 	    {
 		TxPrintf("DRC halo is %d lambda\n",
-				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
+				DRCTechHalo * DRCCurStyle->DRCScaleFactorD
+				/ DRCCurStyle->DRCScaleFactorN);
 		if (DRCTechHalo != DRCCurStyle->DRCTechHalo)
 		    TxPrintf("Maximum rule distance is %d lambda\n",
-			DRCCurStyle->DRCTechHalo / DRCCurStyle->DRCScaleFactor);
+				DRCCurStyle->DRCTechHalo * DRCCurStyle->DRCScaleFactorD
+				/ DRCCurStyle->DRCScaleFactorN);
 	    }
 	    break;
 
@@ -3220,18 +3268,22 @@ CmdDrc(w, cmd)
 	    if (argc == 3)
 	    {
 		DRCStepSize = cmdScaleCoord(w, argv[2], FALSE, TRUE);
-		DRCStepSize *= DRCCurStyle->DRCScaleFactor;
+		DRCStepSize *= DRCCurStyle->DRCScaleFactorN;
+		DRCStepSize /= DRCCurStyle->DRCScaleFactorD;
 		TxPrintf("DRC step size is now %d units)\n",
-				DRCStepSize / DRCCurStyle->DRCScaleFactor);
+				DRCStepSize * DRCCurStyle->DRCScaleFactorD
+				/ DRCCurStyle->DRCScaleFactorN);
 	    }
 	    else
 	    {
 		TxPrintf("DRC step size is %d units\n",
-			DRCStepSize / DRCCurStyle->DRCScaleFactor);
+			DRCStepSize * DRCCurStyle->DRCScaleFactorD
+			/ DRCCurStyle->DRCScaleFactorN);
 		if (DRCStepSize != (16 * DRCCurStyle->DRCTechHalo))
 		    TxPrintf("Recommended step size is %d units\n",
-				(16 * DRCCurStyle->DRCTechHalo) /
-				DRCCurStyle->DRCScaleFactor);
+				(16 * DRCCurStyle->DRCTechHalo) *
+				DRCCurStyle->DRCScaleFactorD /
+				DRCCurStyle->DRCScaleFactorN);
 	    }
 	    break;
 
