@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdFI.c,v 1.2 2006/06/16 16:16:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdFI.c,v 1.4 2007/10/09 19:00:37 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1287,11 +1287,6 @@ CmdGrid(w, cmd)
 #ifdef MAGIC_WRAPPER
 		Tcl_SetObjResult(magicinterp,
 			Tcl_NewIntObj(GrGridMultiple));
-#else
-		if (crec->dbw_multiple == 1)
-		    TxPrintf("Grid multiple drawing is off\n");
-		else
-		    TxPrintf("Grid multiple is %d\n", (int)GrGridMultiple);
 #endif
 	    }
 	    else if (StrIsInt(cmd->tx_argv[2]))
@@ -1903,8 +1898,12 @@ CmdFlatten(w, cmd)
     UndoDisable();
     flatDestUse = newuse;
     
-    scx.scx_use  = EditCellUse;
-    scx.scx_area = EditCellUse->cu_def->cd_bbox;
+    if (EditCellUse)
+	scx.scx_use  = EditCellUse;
+    else
+	scx.scx_use = (CellUse *)w->w_surfaceID;
+
+    scx.scx_area = scx.scx_use->cu_def->cd_bbox;
     scx.scx_trans = GeoIdentityTransform;
 
     DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, xMask, flatDestUse);
