@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdLQ.c,v 1.2 2006/04/14 06:51:24 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdLQ.c,v 1.3 2007/02/12 03:52:13 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1343,6 +1343,78 @@ parsepositions:
     editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdDoProperty ---
+ *
+ * This is the core of the CmdProperty function, separated out so that
+ * the function can operate on a specified CellDef in addition to operating
+ * on the currently selected EditDef.  This allows us to add the option
+ * "property" to the "cellname" command, to grab a property from a cell
+ * without having to be actively editing it.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modified cd_prop record in the current edit cell's def.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdDoProperty(def, cmd, argstart)
+    CellDef *def;
+    TxCommand *cmd;
+{
+    int printPropertiesFunc();
+    char *value;
+    bool propfound;
+    int locargc = cmd->tx_argc - argstart + 1;
+    
+    if (locargc == 1)
+    {
+	/* print all properties and their values */
+	DBPropEnum(def, printPropertiesFunc);
+    }
+    else if (locargc == 2)
+    {
+	/* print the value of the indicated property */
+	value = (char *)DBPropGet(def, cmd->tx_argv[argstart], &propfound);
+	if (propfound)
+#ifdef MAGIC_WRAPPER
+	    Tcl_SetResult(magicinterp, value, NULL);
+#else
+	    TxPrintf("%s", value);
+#endif
+	else {
+#ifdef MAGIC_WRAPPER
+	    /* If the command was "cellname list property ...", then	*/
+	    /* just return NULL if the property was not found.		*/
+	    if (strcmp(cmd->tx_argv[1], "list"))
+#endif
+		TxError("Property name %s is not defined\n");
+	}
+    }
+    else if (locargc == 3)
+    {
+	if (strlen(cmd->tx_argv[argstart + 1]) == 0)
+	    DBPropPut(def, cmd->tx_argv[argstart], NULL);
+	else
+	{
+	    value = StrDup((char **)NULL, cmd->tx_argv[argstart + 1]);
+	    DBPropPut(def, cmd->tx_argv[argstart], value);
+	}
+	def->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+    }    
+    else
+    {
+	TxError("Usage: property [name] [value]\n");
+	TxError("If value is more than one word, enclose in quotes\n");
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1373,9 +1445,6 @@ CmdProperty(w, cmd)
     TxCommand *cmd;
 {
     CellDef *def;
-    int printPropertiesFunc();
-    char *value;
-    bool propfound;
 
     /* Get the root definition of the window	*/
     /* If window is NULL, pick up the edit def	*/
@@ -1388,42 +1457,15 @@ CmdProperty(w, cmd)
     else
 	def = ((CellUse *) w->w_surfaceID)->cu_def;
 
-    if (cmd->tx_argc == 1)
-    {
-	/* print all properties and their values */
-	DBPropEnum(def, printPropertiesFunc);
-    }
-    else if (cmd->tx_argc == 2)
-    {
-	/* print the value of the indicated property */
-	value = (char *)DBPropGet(def, cmd->tx_argv[1], &propfound);
-	if (propfound)
-#ifdef MAGIC_WRAPPER
-	    Tcl_SetResult(magicinterp, value, NULL);
-#else
-	    TxPrintf("%s", value);
-#endif
-	else
-	    TxError("Property name %s is not defined\n");
-    }
-    else if (cmd->tx_argc == 3)
-    {
-	if (strlen(cmd->tx_argv[2]) == 0)
-	    DBPropPut(def, cmd->tx_argv[1], NULL);
-	else
-	{
-	    value = StrDup((char **)NULL, cmd->tx_argv[2]);
-	    DBPropPut(def, cmd->tx_argv[1], value);
-	}
-	def->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
-    }    
-    else
-    {
-	TxError("Usage: property [name] [value]\n");
-	TxError("If value is more than one word, enclose in quotes\n");
-    }
+    CmdDoProperty(def, cmd, 1);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * Callback function for printing a single property key:value pair
+ * ----------------------------------------------------------------------------
+ */
+
 int
 printPropertiesFunc(name, value)
     char *name;
@@ -1435,6 +1477,8 @@ printPropertiesFunc(name, value)
     keyvalue = (char *)mallocMagic(strlen(name) + strlen((char *)value) + 2);
     sprintf(keyvalue, "%s %s", name, (char *)value);
     Tcl_AppendElement(magicinterp, keyvalue);
+    freeMagic(keyvalue);
+    
 #else
     TxPrintf("%s = %s\n", name, value);
 #endif
@@ -1442,8 +1486,6 @@ printPropertiesFunc(name, value)
     return 0;	/* keep the search alive */
 }
 
-/* Added by NP 10/28/2004 */
-
 /*
  * ----------------------------------------------------------------------------
  *
