@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.5 2006/11/02 23:44:07 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.8 2007/03/30 21:35:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -757,6 +757,7 @@ CmdSelect(w, cmd)
     Rect r, selarea;
     int option;
     bool layerspec;
+    bool degenerate;
     bool more = FALSE, less = FALSE, samePlace = TRUE;
 #ifdef MAGIC_WRAPPER
     char *tclstr;
@@ -997,6 +998,7 @@ CmdSelect(w, cmd)
 	    goto Okay;
 	case SEL_CELL:
 	    layerspec = FALSE;
+	    degenerate = FALSE;
 Okay:
 	    if (!(more || less)) SelectClear();
 	    if (option == SEL_BOX || option == SEL_CHUNK || option == SEL_REGION
@@ -1023,6 +1025,7 @@ Okay:
 		}
 		scx.scx_area.r_xtop = scx.scx_area.r_xbot + 1;
 		scx.scx_area.r_ytop = scx.scx_area.r_ybot + 1;
+		degenerate = TRUE;
 	    }
 	    else
 	    {
@@ -1080,10 +1083,36 @@ Okay:
 
 		    TTMaskAndMask (&mask, &uMask);
 
-		    if (TTMaskIsZero (&mask))
+		    if (TTMaskIsZero(&mask))
 		    {
-			TxError ("No paint of this type under the cursor/box\n");
-			return;
+			/* If the area was degenerate (point or line box) */
+			/* try looking in the other direction before	  */
+			/* giving up (added by Tim 3/18/07)		  */
+
+			if (degenerate)
+			{
+			    scx.scx_area.r_xtop--;
+			    scx.scx_area.r_ytop--;
+			    scx.scx_area.r_xbot--;
+			    scx.scx_area.r_ybot--;
+
+			    DBSeeTypesAll(scx.scx_use, &scx.scx_area,
+					crec->dbw_bitmask, &mask);
+			    TTMaskAndMask(&mask, &crec->dbw_visibleLayers);
+			    TTMaskAndMask(&mask, &DBAllButSpaceAndDRCBits);
+			    TTMaskAndMask (&mask, &uMask);
+			    if (TTMaskIsZero(&mask))
+			    {
+				TxError ("No paint of this type under "
+					"or next to the cursor/box\n");
+				return;
+			    }
+			}
+			else
+			{
+			    TxError ("No paint of this type under the cursor/box\n");
+			    return;
+			}
 		    }
 		}
 
@@ -1123,13 +1152,25 @@ Okay:
 		     * use different types.
 		     */
 
-		    for (type += 1; ; type += 1)
+		    int count = 0;
+		    for (type++; count <= DBNumUserLayers; type++, count++)
 		    {
 			if (type >= DBNumUserLayers)	/* used to be DBNumTypes */
 			    type = TT_SELECTBASE;
 			if (TTMaskHasType(&mask, type)) break;
 		    }
 
+		    /* Check for selection between DBNumUserLayers & DBNumTypes */
+		    /* Normally we don't want to use these types;  only if one  */
+		    /* is the only bit set in the list.				*/
+
+		    if (count == DBNumUserLayers)
+			for (type = DBNumUserLayers; type < DBNumTypes; type++)
+			    if (TTMaskHasType(&mask, type)) break;
+
+		    /* Sanity check */
+		    if (type == DBNumTypes) return;
+
 		    SelectChunk(&scx, type, crec->dbw_bitmask, &chunkSelection, less);
 		    if (!less)
 		      DBWSetBox(scx.scx_use->cu_def, &chunkSelection);
@@ -1572,8 +1613,10 @@ CmdSnap(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    static char *names[] = { "off", "internal", "lambda", "grid", "user", "on", 0 };
-    int n, mingrid;
+    static char *names[] = { "off", "internal", "lambda", "grid", "user", "on",
+		"list", 0 };
+    int n = -1;
+    int mingrid;
     DBWclientRec *crec;
 
     if (cmd->tx_argc < 2) goto printit;
@@ -1598,8 +1641,20 @@ CmdSnap(w, cmd)
     }
 
 printit:
-    TxPrintf("Box is aligned to %s grid\n", (DBWSnapToGrid == DBW_SNAP_INTERNAL)
-	? "internal" : ((DBWSnapToGrid == DBW_SNAP_LAMBDA) ? "lambda" : "user"));
+    if (n == 6)  /* list */
+#ifdef MAGIC_WRAPPER
+	Tcl_SetResult(magicinterp, 
+		(DBWSnapToGrid == DBW_SNAP_INTERNAL) ? "internal" :
+		((DBWSnapToGrid == DBW_SNAP_LAMBDA) ? "lambda" : "user"),
+		TCL_VOLATILE);
+#else
+	TxPrintf("%s\n", (DBWSnapToGrid == DBW_SNAP_INTERNAL) ? "internal" :
+		((DBWSnapToGrid == DBW_SNAP_LAMBDA) ? "lambda" : "user"));
+#endif
+    else
+	TxPrintf("Box is aligned to %s grid\n",
+		(DBWSnapToGrid == DBW_SNAP_INTERNAL) ? "internal" :
+		((DBWSnapToGrid == DBW_SNAP_LAMBDA) ? "lambda" : "user"));
 }
 
 
