@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdSubrs.c,v 1.3 2006/10/04 17:30:17 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdSubrs.c,v 1.4 2007/03/18 15:20:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -369,6 +369,7 @@ CmdParseLayers(s, mask)
 #define LN_ALL		2
 #define LN_DOLLAR	3
 #define LN_ERRORS	4
+#define LN_CONNECT	5
     static struct
     {
 	char *layer_name;
@@ -381,6 +382,7 @@ CmdParseLayers(s, mask)
 	"errors",	LN_ERRORS,
 	"labels",	LN_LABELS,
 	"subcell",	LN_CELL,
+	"connect",	LN_CONNECT,
 	0,
     };
 
@@ -454,6 +456,19 @@ CmdParseLayers(s, mask)
 		    TTMaskSetType(&newmask, TT_ERROR_S);
 		    TTMaskSetType(&newmask, TT_ERROR_PS);
 		    break;
+		/*
+		 * Add in all layers connected to layers already parsed
+		 */
+		case LN_CONNECT:
+		    for (type = TT_TECHDEPBASE; type < DBNumTypes; type++)
+			if (TTMaskHasType(mask, type))
+			{
+			    TileType ttype;
+			    for (ttype = TT_TECHDEPBASE; ttype < DBNumTypes; ttype++)
+				if (DBConnectsTo(type, ttype))
+				    TTMaskSetType(&newmask, ttype);
+			}
+		    break;
 	    }
 	}
 	else
