@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.6 2006/11/02 23:54:53 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.8 2007/08/23 21:22:53 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -963,7 +963,8 @@ orderLabelFunc(one, two)
 
     if ((i = strcmp(one->lab_text, two->lab_text)) != 0)
         return i;
-    if ((i = strcmp(one->cell_name, two->cell_name)) != 0)
+    if (one->cell_name && two->cell_name && (i = strcmp(one->cell_name,
+		two->cell_name)) != 0)
         return i;
     if (one->lab_type != two->lab_type)
 	return (one->lab_type < two->lab_type) ? 1 : -1;
@@ -1070,7 +1071,8 @@ CmdWire(w, cmd)
 	"switch [layer width]   place contact and switch layers",
 	"type [layer width]     select the type and size of wires",
 	"vertical               add a new vertical wire leg",
-	"segment layer width x1 y1 x2 y2... [-noendcap]\n"
+	"segment layer width x1 y1 x2 y2... [-noendcap]  (or)\n"
+	"segment layer width filename [-noendcap]\n"
 	"			paint one or more wire segments",
 	NULL
     };
@@ -1274,9 +1276,10 @@ CmdWire(w, cmd)
 			locargc--;
 		    }
 		}
-		if ((locargc < 8) || (locargc & 0x1))
+		if (((locargc < 8) || (locargc & 0x1)) && (locargc != 5))
 		{
 		    TxError("Usage: wire segment layer width x1 x2 y1 y2\n");
+		    TxError(" (or)  wire segment layer width filename\n");
 		    return;
 		}
 		type = DBTechNoisyNameType(cmd->tx_argv[2]);
@@ -1294,12 +1297,64 @@ CmdWire(w, cmd)
 		/* Get the coordinates in 2x internal units, which is	*/
 		/* what is needed by routine PaintWireList.		*/
 
-		points = (locargc - 4) / 2;
-		plist = (Point *)mallocMagic(points * sizeof(Point));
-		for (i = 0, j = 4; i < points; i++)
+		if (locargc == 5)
 		{
-		    plist[i].p_x = cmdScaleCoord(w, cmd->tx_argv[j++], FALSE, TRUE, 2);
-		    plist[i].p_y = cmdScaleCoord(w, cmd->tx_argv[j++], FALSE, FALSE, 2);
+		    FILE *pfile;
+		    char line[128];
+		    char *pptr, *xptr, *yptr;
+
+		    if ((pfile = PaOpen(cmd->tx_argv[4], "r", NULL, Path, CellLibPath,
+				NULL)) == NULL)
+		    {
+			TxError("No such file or error opening %s\n", cmd->tx_argv[4]);
+			return;
+		    }
+		    else
+		    {
+			points = 0;
+			while (fgets(line, 128, pfile) != NULL) points++;
+			rewind(pfile);
+			plist = (Point *)mallocMagic(points * sizeof(Point));
+			i = 0;
+			while (fgets(line, 128, pfile) != NULL)
+			{
+			    /* Parse two coordinates (x and y) out of each line */
+			    for (pptr = line; isspace(*pptr) && *pptr != '\0'; pptr++);
+			    xptr = pptr;
+			    for (; !isspace(*pptr) && *pptr != '\0'; pptr++);
+			    *pptr++ = '\0';
+			    for (; isspace(*pptr) && *pptr != '\0'; pptr++);
+			    yptr = pptr;
+			    for (; !isspace(*pptr) && *pptr != '\0'
+					&& *pptr != '\n'; pptr++);
+			    *pptr++ = '\0';
+
+			    if (*xptr == '\0' || *yptr == '\0')
+			    {
+				TxError("Bad coordinate pair at %s line %d\n",
+					cmd->tx_argv[4], i + 1); 
+				freeMagic(plist);
+				return;
+			    }
+
+			    plist[i].p_x = cmdScaleCoord(w, xptr, FALSE, TRUE, 2);
+			    plist[i].p_y = cmdScaleCoord(w, yptr, FALSE, FALSE, 2);
+			    i++;
+			}
+			fclose(pfile);
+		    }
+		}
+		else
+		{
+		    points = (locargc - 4) / 2;
+		    plist = (Point *)mallocMagic(points * sizeof(Point));
+		    for (i = 0, j = 4; i < points; i++)
+		    {
+			plist[i].p_x = cmdScaleCoord(w, cmd->tx_argv[j++],
+					FALSE, TRUE, 2);
+			plist[i].p_y = cmdScaleCoord(w, cmd->tx_argv[j++],
+					FALSE, FALSE, 2);
+		    }
 		}
 
 		def->cd_flags |= CDMODIFIED | CDGETNEWSTAMP;
