@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdWizard.c,v 1.2 2001/01/12 22:12:17 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdWizard.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -247,7 +247,7 @@ showTech(outf, verbose)
     fprintf(outf, "\n\014Component Layers:\n");
     for (i = 0; i < DBNumUserLayers; i++)
 	for (j = 0; j < DBNumUserLayers; j++)
-	    if ((j != i) && TTMaskHasType(&DBComponentTbl[j], i))
+	    if ((j != i) && TTMaskHasType(DBResidueMask(j), i))
 		fprintf(outf, "%s is a component of %s\n",
 		    DBTypeLongName(i), DBTypeLongName(j));
     fprintf(outf, "\n");
