@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.3 2006/11/05 21:50:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.4 2007/04/20 17:56:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1232,9 +1232,9 @@ DBCellNewUse(cellDef, useName)
 
     /* Initial client field */
     /* (commands can use this field for whatever
-     * they like, but should restore its value to MINFINITY before exiting.) 
+     * they like, but should restore its value to CLIENTDEFAULT before exiting.) 
      */
-    cellUse->cu_client = (ClientData) MINFINITY;
+    cellUse->cu_client = (ClientData) CLIENTDEFAULT;
 
     cellDef->cd_parents = cellUse;
     DBComputeUseBbox(cellUse);
