@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -315,12 +315,12 @@ dbSrConnectFunc(tile, csa)
 
     if (csa->csa_clear)
     {
-	if (tile->ti_client == (ClientData) MINFINITY) return 0;
-	tile->ti_client = (ClientData) MINFINITY;
+	if (tile->ti_client == (ClientData) CLIENTDEFAULT) return 0;
+	tile->ti_client = (ClientData) CLIENTDEFAULT;
     }
     else
     {
-	if (tile->ti_client != (ClientData) MINFINITY) return 0;
+	if (tile->ti_client != (ClientData) CLIENTDEFAULT) return 0;
 	tile->ti_client = (ClientData) 1;
     }
 
@@ -370,9 +370,9 @@ dbSrConnectFunc(tile, csa)
 	{
 	    if (csa->csa_clear)
 	    {
-		if (t2->ti_client == (ClientData) MINFINITY) continue;
+		if (t2->ti_client == (ClientData) CLIENTDEFAULT) continue;
 	    }
-	    else if (t2->ti_client != (ClientData) MINFINITY) continue;
+	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) continue;
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
     }
@@ -399,9 +399,9 @@ bottomside:
 	{
 	    if (csa->csa_clear)
 	    {
-		if (t2->ti_client == (ClientData) MINFINITY) continue;
+		if (t2->ti_client == (ClientData) CLIENTDEFAULT) continue;
 	    }
-	    else if (t2->ti_client != (ClientData) MINFINITY) continue;
+	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) continue;
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
     }
@@ -427,9 +427,9 @@ rightside:
 	{
 	    if (csa->csa_clear)
 	    {
-		if (t2->ti_client == (ClientData) MINFINITY) goto nextRight;
+		if (t2->ti_client == (ClientData) CLIENTDEFAULT) goto nextRight;
 	    }
-	    else if (t2->ti_client != (ClientData) MINFINITY) goto nextRight;
+	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) goto nextRight;
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
 	nextRight: if (BOTTOM(t2) <= tileArea.r_ybot) break;
@@ -456,9 +456,9 @@ topside:
 	{
 	    if (csa->csa_clear)
 	    {
-		if (t2->ti_client == (ClientData) MINFINITY) goto nextTop;
+		if (t2->ti_client == (ClientData) CLIENTDEFAULT) goto nextTop;
 	    }
-	    else if (t2->ti_client != (ClientData) MINFINITY) goto nextTop;
+	    else if (t2->ti_client != (ClientData) CLIENTDEFAULT) goto nextTop;
 	    if (dbSrConnectFunc(t2, csa) != 0) return 1;
 	}
 	nextTop: if (LEFT(t2) <= tileArea.r_xbot) break;
