@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001/06/07 21:20:19 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.7 2006/11/02 23:44:07 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -30,10 +30,23 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/file.h>
+
+#ifdef HAVE_DIRENT_H
+#include <dirent.h>
+#include <unistd.h>
+#define direct dirent
+#else
+#include <sys/dir.h>
+#endif
+
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
 
+#ifdef HAVE_PATHS_H
+#include <paths.h>
+#endif
+
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -50,6 +63,10 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001
 #include "utils/malloc.h"
 #include "utils/signals.h"
 
+#ifndef _PATH_TMP
+#define _PATH_TMP "/tmp"
+#endif
+
 extern char *Path;
 
 /* Suffix for all Magic files */
@@ -61,6 +78,9 @@ int DBLambda[2] = {1, 1};
 /* If set to FALSE, don't print warning messages. */
 bool DBVerbose = TRUE;
 
+/* Global name of backup file for this session */
+static char *DBbackupFile = (char *)NULL;
+
 /* Forward declarations */
 char *dbFgets();
 FILE *dbReadOpen();
@@ -80,6 +100,7 @@ bool dbReadUse();
  *
  * ---------------------------------------------------------------------------
  */
+
 static int
 file_is_not_writeable(name)
     char *name;
@@ -88,13 +109,13 @@ file_is_not_writeable(name)
 
     /* attempt to read stat buffer for file */
 
-    if(lstat(name,&sbuf) < 0) return(-1);
+    if (lstat(name,&sbuf) < 0) return(-1);
 
     /* regular file? if not, error */
-    if(!S_ISREG(sbuf.st_mode)) { errno = EACCES; return(-1); }
+    if (!S_ISREG(sbuf.st_mode)) { errno = EACCES; return(-1); }
 
     /* can we write to it? */
-    if(access(name, W_OK) < 0) return(-1);
+    if (access(name, W_OK) < 0) return(-1);
 
     /* the OS thinks we can write to the file;
        but does the file think so?
@@ -114,16 +135,13 @@ file_is_not_writeable(name)
     return(0);
 }
 
-
-
 /*
  * ----------------------------------------------------------------------------
  *
- * DBCellRead --
+ * dbCellReadDef --
  *
- * If the cell is already marked as available (CDAVAILABLE), do nothing.
  *
- * Otherwise, read in the paint for a cell from its associated disk file.
+ * Read in the paint for a cell from its associated disk file.
  * If a filename for the cell is specified, we try to open it
  * somewhere in the search path.  Otherwise, we try the filename
  * already associated with the cell, or the name of the cell itself
@@ -145,18 +163,16 @@ file_is_not_writeable(name)
  *
  * Results:
  *	TRUE if the cell could be read successfully, FALSE
- *	otherwise.  If the cell is already read in, TRUE is
- *	also returned.
+ *	otherwise.
  *
  * Side effects:
+ 	Clears the cell's MODIFIED bit.
  *	Updates the tile planes for the cell definition.
  *	In the event of an error while reading in the cell,
  *	the external integer errno is set to the UNIX error
  *	encountered.
  *
- *	The cell definition is marked as available.
- *	The cell's MODIFIED bit is cleared by this routine.  If
- *	newTechOk is TRUE and the cell's technology is different
+ *	If newTechOk is TRUE and the cell's technology is different
  *	from the current one, the current technology is changed.
  *
  * Errors:
@@ -233,7 +249,8 @@ file_is_not_writeable(name)
  */
 
 bool
-DBCellRead(cellDef, name, ignoreTech)
+dbCellReadDef(f, cellDef, name, ignoreTech)
+    FILE *f;		/* The file, already opened by the caller */
     CellDef *cellDef;	/* Pointer to definition of cell to be read in */
     char *name;		/* Name of file from which to read definition.
 			 * If NULL, then use cellDef->cd_file; if that
@@ -251,22 +268,15 @@ DBCellRead(cellDef, name, ignoreTech)
     int cellStamp = 0, rectCount = 0, rectReport = 10000;
     char line[2048], tech[50], layername[50];
     PaintResultType *ptable;
-    bool result = TRUE;
-    FILE *f;
+    bool result = TRUE, scaleLimit = FALSE;
     Rect *rp;
     int c;
-    TileType type, rtype;
-    TileTypeBitMask *rmask;
+    TileType type, rtype, loctype;
+    TileTypeBitMask *rmask, typemask;
     Plane *plane;
     Rect r;
     int n = 1, d = 1;
 
-    if (cellDef->cd_flags & CDAVAILABLE)
-	return (TRUE);
-
-    if ((f = dbReadOpen(cellDef, name, TRUE)) == NULL)
-	return (FALSE);
-
     /*
      * It's very important to disable interrupts during the body of
      * this routine.  Otherwise, if the user types the interrupt key
@@ -313,11 +323,6 @@ DBCellRead(cellDef, name, ignoreTech)
 		TxError("\"cellname delete %s\" and \"load %s -force\" to"
 			" force the cell to load as technology %s\n",
 			cellDef->cd_name, cellDef->cd_name, DBTechName);
-		(void) fclose(f);
-#ifdef FILE_LOCKS
-		flock_close(f, cellDef, name);
-#endif
-		f = NULL;
 		SigEnableInterrupts();
 		return (FALSE);
 	    }
@@ -363,11 +368,14 @@ DBCellRead(cellDef, name, ignoreTech)
     n *= DBLambda[1];
     d *= DBLambda[0];
     ReduceFraction(&n, &d);
-    if (d > 1)
+    scaleLimit = CIFTechLimitScale(n, d);
+
+    if (!scaleLimit && (d > 1))
     {
 	CIFTechInputScale(1, d, TRUE);
 	CIFTechOutputScale(1, d);
 	DRCTechScale(1, d);
+	PlowAfterTech();
 	ExtTechScale(1, d);
 	WireTechScale(1, d);
 #ifdef LEF_MODULE
@@ -375,15 +383,27 @@ DBCellRead(cellDef, name, ignoreTech)
 #endif
 #ifdef ROUTE_MODULE
 	RtrTechScale(1, d);
+	MZAfterTech();
+	IRAfterTech();
 #endif
 	DBScaleEverything(d, 1);
 	DBLambda[1] *= d;
 	TxPrintf("Input cell %s scales magic internal geometry by factor of %d\n",
 			cellDef->cd_name, d);
+	d = 1;
     }
     if (n > 1)
-	TxPrintf("Scaled magic input cell %s geometry by factor of %d\n", 
+    {
+	TxPrintf("Scaled magic input cell %s geometry by factor of %d", 
 		cellDef->cd_name, n);
+	if (d > 1) 
+	{
+	    TxPrintf("/ %d", d);
+	    TxError("Warning:  Geometry may be lost because internal grid"
+			"cannot be reduced.\n");
+	}
+	TxPrintf("\n");
+    }
 
     /*
      * Next, get the paint, subcells, and labels for this cell.
@@ -402,11 +422,13 @@ DBCellRead(cellDef, name, ignoreTech)
 	 */
 	if (sscanf(line, "<< %s >>", layername) != 1)
 	{
-	    if (!dbReadUse(cellDef, line, sizeof line, f, n))
+	    if (!dbReadUse(cellDef, line, sizeof line, f, n, d))
 		goto badfile;
 	    continue;
 	}
 
+	TTMaskZero(&typemask);
+	rmask = &typemask;
 	type = DBTechNameType(layername);
 	if (type < 0)
 	{
@@ -419,12 +441,12 @@ DBCellRead(cellDef, name, ignoreTech)
 	     */
 	    if (!strcmp(layername, "labels"))
 	    {
-		if (!dbReadLabels(cellDef, line, sizeof line, f, n)) goto badfile;
+		if (!dbReadLabels(cellDef, line, sizeof line, f, n, d)) goto badfile;
 		continue;
 	    }
 	    else if (!strcmp(layername, "elements"))
 	    {
-		if (!dbReadElements(cellDef, line, sizeof line, f, n)) goto badfile;
+		if (!dbReadElements(cellDef, line, sizeof line, f, n, d)) goto badfile;
 		continue;
 	    }
 	    else if (!strcmp(layername, "properties"))
@@ -433,8 +455,11 @@ DBCellRead(cellDef, name, ignoreTech)
 		continue;
 	    }
 	    else if (!strcmp(layername, "end")) goto done;
-	    TxError("Unknown layer %s ignored in %s\n", layername,
-			cellDef->cd_name);
+	    else
+		DBTechNoisyNameMask(layername, rmask);
+
+	    //	TxError("Unknown layer %s ignored in %s\n", layername,
+	    //		cellDef->cd_name);
 	}
 
 	/*
@@ -444,7 +469,10 @@ DBCellRead(cellDef, name, ignoreTech)
 	if (DBPlane(type) > 0)
 	{
 	    if (type < DBNumUserLayers)
+	    {
 		TTMaskSetType(&cellDef->cd_types, type);
+		TTMaskSetType(rmask, type);
+	    }
 	    else
 	    {
 		/* Separate stacked contact types into their components */
@@ -476,13 +504,13 @@ nextrect:
 	    if (c == 't')
 	    {
 #ifdef NONMANHATTAN  
-		if ((dir = GetRect(f, 3, rp, n)) == 0) goto badfile;
+		if ((dir = GetRect(f, 3, rp, n, d)) == 0) goto badfile;
 		dir >>= 1;
 		dinfo = TT_DIAGONAL | ((dir & 0x2) ? TT_SIDE : 0) |
 			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
 			TT_DIRECTION : 0);
 #else
-		if (!GetRect(f, 3, rp, n)) goto badfile;
+		if (!GetRect(f, 3, rp, n, d)) goto badfile;
 		TxPrintf("Error: cell %s contains non-manhattan geometry\n",
 			cellDef->cd_name);
 		TxPrintf("unsupported by this magic executable.  Skipping.\n");
@@ -495,7 +523,7 @@ nextrect:
 #ifdef NONMANHATTAN  
 		dinfo = 0;
 #endif
-		if (!GetRect(f, 4, rp, n)) goto badfile;
+		if (!GetRect(f, 4, rp, n, d)) goto badfile;
 	    }
 
 	    if ((++rectCount % rectReport == 0) && DBVerbose)
@@ -524,30 +552,17 @@ nextrect:
 		/*				   	Tim 7/8/04	*/
 		/*------------------------------------------------------*/
 
-		TileType loctype;
-
-		if (type >= DBNumUserLayers)
+		for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
 		{
-		    rmask = DBResidueMask(type);
-		    for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
-			if (TTMaskHasType(rmask, rtype))
-			{
-			    loctype = rtype;
+		    if (TTMaskHasType(rmask, rtype))
+		    {
+			loctype = rtype;
 #ifdef NONMANHATTAN
-			    if (dinfo & TT_SIDE) loctype <<= 14;
-			    loctype |= dinfo;
+			if (dinfo & TT_SIDE) loctype <<= 14;
+			loctype |= dinfo;
 #endif
-			    DBPaint(cellDef, rp, loctype);
-			}
-		}
-		else
-		{
-		    loctype = type;
-#ifdef NONMANHATTAN
-		    if (dinfo & TT_SIDE) loctype <<= 14;
-		    loctype |= dinfo;
-#endif
-		    DBPaint(cellDef, rp, loctype);
+			DBPaint(cellDef, rp, loctype);
+		    }
 		}
 	    }
 	}
@@ -574,11 +589,6 @@ nextrect:
 
 done:
 
-    (void) fclose(f);
-#ifdef FILE_LOCKS
-    flock_close(f, cellDef, name);
-#endif
-    f = NULL;
     cellDef->cd_flags &= ~(CDMODIFIED|CDBOXESCHANGED|CDGETNEWSTAMP);
 
     /*
@@ -629,13 +639,323 @@ done:
 badfile:
     TxError("File %s contained format error\n", cellDef->cd_name);
     DRCCheckThis(cellDef, TT_CHECKPAINT, (Rect *) NULL);
-#ifdef FILE_LOCKS
-    flock_flag = 1;
-#endif
     result = FALSE;
     goto done;
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBRemoveBackup() --
+ *
+ * Remove any crash backup file.  This routine is normally called either on
+ * normal program exit, or after the successful read-in of a crash backup.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	If DBbackupFile is non-NULL, then memory is freed, and the backup
+ *	file is removed from the filesystem temp directory.  Otherwise,
+ *	nothing happens.
+ *	
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBRemoveBackup()
+{
+    if (DBbackupFile != (char *)NULL)
+    {
+	unlink(DBbackupFile);
+	freeMagic(DBbackupFile);
+	DBbackupFile = (char *)NULL;
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBFileRecovery() --
+ *
+ * Get the name of the first backup file found in the /tmp directory,
+ * prompt for action, and if action is "read", then load it.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Prompts for action.
+ *	Loads the contents of the backup file into the database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBFileRecovery(filename)
+    char *filename;
+{
+    DIR *cwd;
+    struct direct *dp;
+    struct stat sbuf;
+    uid_t userid = getuid();
+    time_t recent = 0;
+    char *snptr, *tempdir, tempname[256];
+    int pid;
+    static char *actionNames[] = {"read", "cancel", 0 };
+    char *prompt;
+    int action;
+
+    if (DBbackupFile != NULL)
+    {
+	TxError("Error:  Backup file in use for current session.\n");
+	return;
+    }
+
+    if (filename == NULL)
+    {
+
+	tempdir = getenv("TMPDIR");
+	if (tempdir == NULL) tempdir = _PATH_TMP;
+
+	cwd = opendir(tempdir);
+	if (cwd == NULL) return;
+
+	/* Find the most recent crash file in the temp directory */
+
+	while ((dp = readdir(cwd)) != NULL)
+	{
+	    char *doslash = (tempdir[strlen(tempdir) - 1] == '/') ? "" : "/";
+	    sprintf(tempname, "%s%s%s", tempdir, doslash, dp->d_name);
+	    snptr = tempname + strlen(tempdir);
+	    if (!strncmp(snptr, "MAG", 3))
+	    {
+		char *dotptr = strchr(snptr, '.');
+	        pid = -1;
+		if (dotptr && dotptr > snptr + 3)
+		{
+		    *dotptr = '\0';
+		    if (sscanf(snptr + 3, "%d", &pid) != 1)
+			pid = -1;
+		    *dotptr = '.';
+		}
+		if ((!stat(tempname, &sbuf)) && (sbuf.st_uid == userid))
+		{
+		    if ((recent == 0) || (sbuf.st_ctime > recent))
+		    {
+			/* If the PID encoded in the name belongs to an	*/
+			/* active process, then we should not try to	*/
+			/* open it.					*/
+
+			if (pid != -1)
+			    if (SigCheckProcess(pid) == TRUE)
+				continue;
+
+			recent = sbuf.st_ctime;
+			DBbackupFile = StrDup(&DBbackupFile, tempname);
+		    }
+		}  
+	    } 
+	}
+	closedir(cwd);
+    }
+    else
+    {
+	DBbackupFile = StrDup(&DBbackupFile, filename);
+	recent = 1;
+    }
+
+    if (recent > 0)
+    {			/* There exists at least one temporary file	*/
+			/* belonging to this user.  Ask to recover	*/
+			/* the most recent one.				*/
+
+	prompt = TxPrintString("Recover from backup file %s?", DBbackupFile);
+	action = TxDialog(prompt, actionNames, 0);
+
+	switch(action)
+	{
+	    case 0:	/* Read */
+		if (DBReadBackup(DBbackupFile) == TRUE)
+		    DBRemoveBackup();
+		break;
+	    case 1:	/* Cancel */
+		break;
+	}
+    }
+
+    /* Make sure we've cleared out the backup filename */
+
+    if (DBbackupFile != NULL)
+    {
+	freeMagic(DBbackupFile);
+	DBbackupFile = (char *)NULL;
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBReadBackup --
+ *
+ * This file reads a backup file containing multiple cell definitions.
+ *
+ * Results:
+ *	TRUE if the backup file was read successfully, FALSE otherwise.
+ *
+ * Side Effects:
+ *	Side effects are the side effects caused by dbCellReadDef()
+ *	(see above).  As many cells as are listed in the backup file
+ *	are created and added to the database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBReadBackup(name)
+    char *name;		/* Name of the backup file */
+{
+    FILE *f;
+    char *filename, *rootname, *chrptr;
+    char line[256];
+    CellDef *cellDef;
+    bool result = TRUE;
+
+    if ((f = PaOpen(name, "r", NULL, "", NULL, NULL)) == NULL)
+    {
+	TxError("Cannot open backup file \"%s\"\n", name);
+	return FALSE;
+    }
+
+    if (dbFgets(line, sizeof(line), f) == NULL)
+    {
+	TxError("Bad backup file %s; can't restore!\n", name);
+	return FALSE;
+    }
+
+    while (strncmp(line, "end", 3) != 0)
+    {
+	if (strncmp(line, "file", 4) == 0)
+	{
+	    filename = line + 4;
+
+	    /* Remove any trailing return character */
+	    chrptr = strrchr(filename, '\n');
+	    if (chrptr != NULL) *chrptr = '\0';
+
+	    /* Remove any trailing file extension */
+	    chrptr = strstr(filename, ".mag");
+	    if (chrptr != NULL) *chrptr = '\0';
+	    rootname = strrchr(filename, '/');
+	    if (rootname == NULL)
+		rootname = filename;
+	    else
+		rootname++;
+
+	    /* Remove any leading whitespace */
+	    while (isspace(*rootname) && *rootname != '\0') rootname++;
+	    if (strlen(rootname) == 0) return FALSE;
+
+ 	    cellDef = DBCellLookDef(rootname);
+	    if (cellDef == (CellDef *)NULL)
+		cellDef = DBCellNewDef(rootname, (char *)NULL);
+
+	    cellDef->cd_flags &= ~CDNOTFOUND;
+	    cellDef->cd_flags |= CDAVAILABLE;
+
+	    if (dbCellReadDef(f, cellDef, filename, TRUE) == FALSE)
+		return FALSE;
+
+	    if (dbFgets(line, sizeof(line), f) == NULL)
+	    {
+		TxError("Error in backup file %s; partial restore only!\n",
+			name);
+		return FALSE;
+	    }
+	}
+	else
+	{
+	    TxError("Error in backup file %s; expected keyword"
+			" \"file\", got \"%s\"!\n", name, line);
+	    return FALSE;
+	}
+    }
+    chrptr = strrchr(line, '\n');
+    if (chrptr > line + 4)
+    {
+	/* Remove the trailing return character */
+	*chrptr = '\0';
+	DBWreload(line + 4);
+    }
+    return TRUE;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBCellRead --
+ *
+ * This is the wrapper for DBCellReadDef.  The routine has been divided into
+ * parts so that a single backup file can be made and recovered, and in
+ * preparation for allowing certain cell definitions to be in-lined into the
+ * output file (such as polygonXXXXX cells generated by the gds read-in).
+ *
+ * Results:
+ *	TRUE if the cell could be read successfully, FALSE
+ *	otherwise.  If the cell is already read in, TRUE is
+ *	also returned.
+ *
+ * Side Effects:
+ *	If the cell is already marked as available (CDAVAILABLE), this
+ *	routine	does nothing (has no side effects).
+ *
+ *	Otherwise, side effects are the side effects caused by
+ *	dbCellReadDef() (see above).  In addition, the cell
+ *	definition is marked as available.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBCellRead(cellDef, name, ignoreTech)
+    CellDef *cellDef;	/* Pointer to definition of cell to be read in */
+    char *name;		/* Name of file from which to read definition.
+			 * If NULL, then use cellDef->cd_file; if that
+			 * is NULL try the name of the cell.
+			 */
+    bool ignoreTech;	/* If FALSE then the technology of the file MUST
+			 * match the current technology, or else the
+			 * subroutine will return an error condition
+			 * without reading anything.  If TRUE, a
+			 * warning will be printed if the technology
+			 * names do not match, but an attempt will be
+			 * made to read the file anyway.
+			 */
+{
+    FILE *f;
+    bool result;
+
+    if (cellDef->cd_flags & CDAVAILABLE)
+	result = TRUE;
+
+    else if ((f = dbReadOpen(cellDef, name, TRUE)) == NULL)
+	result = FALSE;
+
+    else
+    {
+	result = (dbCellReadDef(f, cellDef, name, ignoreTech));
+#ifdef FILE_LOCKS
+	/* Close files that were locked by another user */
+	if (cellDef->cd_fd == -1) fclose(f);
+#else
+	/* When using fcntl() to enforce file locks, we can't	*/
+	/* close the file descriptor without losing the lock.	*/
+	fclose(f);
+#endif
+    }
+    return result;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -672,23 +992,31 @@ dbReadOpen(cellDef, name, setFileName)
 			 */
 {
     FILE *f;
-    char *filename;
+    char *filename, *realname;
+    bool is_locked;
+
 #ifdef FILE_LOCKS
-    char *realname;
+    if (cellDef->cd_fd != -1)
+    {
+	close(cellDef->cd_fd);
+	cellDef->cd_fd = -1;
+    }
 #endif
 
     if (name != (char *) NULL)
     {
-	f = PaOpen(name, "r", DBSuffix, Path, CellLibPath, &filename);
+	f = PaLockOpen(name, "r", DBSuffix, Path,
+			CellLibPath, &filename, &is_locked);
     }
     else if (cellDef->cd_file != (char *) NULL)
     {
-	f = PaOpen(cellDef->cd_file, "r", "", ".", (char *) NULL, &filename);
+	f = PaLockOpen(cellDef->cd_file, "r", "", ".",
+			(char *) NULL, &filename, &is_locked);
     }
     else
     {
-	f = PaOpen(cellDef->cd_name, "r", DBSuffix,
-			Path, CellLibPath, &filename);
+	f = PaLockOpen(cellDef->cd_name, "r", DBSuffix, Path,
+			CellLibPath, &filename, &is_locked);
     }
 
     if (f == NULL)
@@ -703,32 +1031,33 @@ dbReadOpen(cellDef, name, setFileName)
 	    TxError("File %s couldn't be found\n", cellDef->cd_file);
 	else {
 	    TxError("Cell %s couldn't be found\n", cellDef->cd_name);
-#ifdef FILE_LOCKS
-	    realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name) + strlen(DBSuffix) + 1));
+	    realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
+			+ strlen(DBSuffix) + 1));
 	    (void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
 	    cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
-#endif
 	}
 	cellDef->cd_flags |= CDNOTFOUND;
 	return ((FILE *) NULL);
     }
+
 #ifdef FILE_LOCKS
     else
     {
-	if (file_is_not_writeable (filename) || (flock_flag != 0))
+	if (file_is_not_writeable(filename) || (is_locked == TRUE))
 	{
 	    cellDef->cd_flags |= CDNOEDIT;
-	    if ((flock_flag == 0) && DBVerbose)
+	    if ((is_locked == FALSE) && DBVerbose)
 		TxPrintf("Warning: cell <%s> from file %s is not writeable\n",
 			cellDef->cd_name, filename);
 	}
-	flock_flag = 0;
+	if (is_locked == FALSE)
+	    cellDef->cd_fd = fileno(f);
 	cellDef->cd_flags &= ~CDNOTFOUND;
     }
 #else
-    if (file_is_not_writeable (filename) && DBVerbose)
-	TxPrintf("Warning: cell <%s> from file %s is not writeable\n", cellDef->cd_name,
-		filename);
+    if (file_is_not_writeable(filename) && DBVerbose)
+	TxPrintf("Warning: cell <%s> from file %s is not writeable\n",
+		cellDef->cd_name, filename);
     TxFlushOut();
 
     cellDef->cd_flags &= ~CDNOTFOUND;
@@ -761,12 +1090,13 @@ dbReadOpen(cellDef, name, setFileName)
  */
 
 bool
-dbReadUse(cellDef, line, len, f, scale)
+dbReadUse(cellDef, line, len, f, scalen, scaled)
     CellDef *cellDef;	/* Cell whose cells are being read */
     char *line;		/* Line containing "use ..." */
     int len;		/* Size of buffer pointed to by line */
     FILE *f;		/* Input file */
-    int scale;
+    int scalen;		/* Multiply values in file by this */
+    int scaled;		/* Divide values in file by this */
 {
     int xlo, xhi, ylo, yhi, xsep, ysep, childStamp;
     int absa, absb, absd, abse;
@@ -800,10 +1130,15 @@ dbReadUse(cellDef, line, len, f, scale)
 	    TxError("Malformed \"array\" line: %s", line);
 	    return (FALSE);
 	}
-	if (scale > 1)
+	if (scalen > 1)
 	{
-	    xsep *= scale;
-	    ysep *= scale;
+	    xsep *= scalen;
+	    ysep *= scalen;
+	}
+	if (scaled > 1)
+	{
+	    xsep /= scaled;
+	    ysep /= scaled;
 	}
 	if (dbFgets(line, len, f) == NULL)
 	    return (FALSE);
@@ -864,14 +1199,23 @@ badTransform:
 	return (FALSE);
     }
 
-    if (scale > 1)
+    if (scalen > 1)
     {
-	t.t_c *= scale;
-	t.t_f *= scale;
-	r.r_xbot *= scale;
-	r.r_ybot *= scale;
-	r.r_xtop *= scale;
-	r.r_ytop *= scale;
+	t.t_c *= scalen;
+	t.t_f *= scalen;
+	r.r_xbot *= scalen;
+	r.r_ybot *= scalen;
+	r.r_xtop *= scalen;
+	r.r_ytop *= scalen;
+    }
+    if (scaled > 1)
+    {
+	t.t_c /= scaled;
+	t.t_f /= scaled;
+	r.r_xbot /= scaled;
+	r.r_ybot /= scaled;
+	r.r_xtop /= scaled;
+	r.r_ytop /= scaled;
     }
 
     /*
@@ -1019,6 +1363,10 @@ dbReadProperties(cellDef, line, len, f)
 	    }
 	    storedvalue = StrDup((char **)NULL, propertyvalue);
 	    (void) DBPropPut(cellDef, propertyname, storedvalue);
+
+	    /* Go ahead and process the vendor GDS property */
+	    if (!strcmp(propertyname, "GDS_FILE"))
+		cellDef->cd_flags |= CDVENDORGDS;
 	}
 
 nextproperty:
@@ -1050,12 +1398,13 @@ nextproperty:
  */
 
 bool
-dbReadElements(cellDef, line, len, f, scale)
+dbReadElements(cellDef, line, len, f, scalen, scaled)
     CellDef *cellDef;	/* Cell whose elements are being read */
     char *line;		/* Line containing << elements >> */
     int len;		/* Size of buffer pointed to by line */
     FILE *f;		/* Input file */
-    int scale;		/* Scale factor */
+    int scalen;		/* Scale up by this factor */
+    int scaled;		/* Scale down by this factor */
 {
     char elementname[128], styles[1024], *text, flags[100];
     int istyle, ntok;
@@ -1087,12 +1436,19 @@ dbReadElements(cellDef, line, len, f, scale)
 		TxError("Skipping bad \"rectangle\" element line: %s", line);
 		goto nextelement;
 	    }
-	    if (scale > 1)
+	    if (scalen > 1)
 	    {
-		r.r_xbot *= scale;
-		r.r_ybot *= scale;
-		r.r_xtop *= scale;
-		r.r_ytop *= scale;
+		r.r_xbot *= scalen;
+		r.r_ybot *= scalen;
+		r.r_xtop *= scalen;
+		r.r_ytop *= scalen;
+	    }
+	    if (scaled > 1)
+	    {
+		r.r_xbot /= scaled;
+		r.r_ybot /= scaled;
+		r.r_xtop /= scaled;
+		r.r_ytop /= scaled;
 	    }
 	    (void) DBWElementAddRect(NULL, elementname, &r, cellDef, 0);
 	    ntok -= 6;
@@ -1106,12 +1462,19 @@ dbReadElements(cellDef, line, len, f, scale)
 		TxError("Skipping bad \"line\" element line: %s", line);
 		goto nextelement;
 	    }
-	    if (scale > 1)
+	    if (scalen > 1)
 	    {
-		r.r_xbot *= scale;
-		r.r_ybot *= scale;
-		r.r_xtop *= scale;
-		r.r_ytop *= scale;
+		r.r_xbot *= scalen;
+		r.r_ybot *= scalen;
+		r.r_xtop *= scalen;
+		r.r_ytop *= scalen;
+	    }
+	    if (scaled > 1)
+	    {
+		r.r_xbot /= scaled;
+		r.r_ybot /= scaled;
+		r.r_xtop /= scaled;
+		r.r_ytop /= scaled;
 	    }
 	    (void) DBWElementAddLine(NULL, elementname, &r, cellDef, 0);
 	    ntok -= 6;
@@ -1131,10 +1494,15 @@ dbReadElements(cellDef, line, len, f, scale)
 	    text++;
 	    *textend = '\0';
 
-	    if (scale > 1)
+	    if (scalen > 1)
 	    {
-		r.r_xbot *= scale;
-		r.r_ybot *= scale;
+		r.r_xbot *= scalen;
+		r.r_ybot *= scalen;
+	    }
+	    if (scaled > 1)
+	    {
+		r.r_xbot /= scaled;
+		r.r_ybot /= scaled;
 	    }
 	    (void) DBWElementAddText(NULL, elementname, r.r_xbot, r.r_ybot, text,
 				cellDef, 0);
@@ -1204,12 +1572,13 @@ nextelement:
  */
 
 bool
-dbReadLabels(cellDef, line, len, f, scale)
+dbReadLabels(cellDef, line, len, f, scalen, scaled)
     CellDef *cellDef;	/* Cell whose labels are being read */
     char *line;		/* Line containing << labels >> */
     int len;		/* Size of buffer pointed to by line */
     FILE *f;		/* Input file */
-    int scale;		/* Scale factor */
+    int scalen;		/* Scale up by this factor */
+    int scaled;		/* Scale down by this factor */
 {
     char layername[50], text[1024], port_use[50], port_class[50];
     TileType type;
@@ -1345,18 +1714,35 @@ dbReadLabels(cellDef, line, len, f, scale)
 	    r.r_ytop = r.r_ybot;
 	}
 
-	if (scale > 1)
+	if (scalen > 1)
 	{
-	    r.r_xbot *= scale;
-	    r.r_ybot *= scale;
-	    r.r_xtop *= scale;
-	    r.r_ytop *= scale;
+	    r.r_xbot *= scalen;
+	    r.r_ybot *= scalen;
+	    r.r_xtop *= scalen;
+	    r.r_ytop *= scalen;
+	}
+	if (scaled > 1)
+	{
+	    r.r_xbot /= scaled;
+	    r.r_ybot /= scaled;
+	    r.r_xtop /= scaled;
+	    r.r_ytop /= scaled;
 	}
 	type = DBTechNameType(layername);
 	if (type < 0)
 	{
-	    TxError("\
-Warning: label \"%s\" attached to unknown type \"%s\"\n", text, layername);
+	    TileTypeBitMask rmask;
+
+	    /* Check against alias hash table.  Names that are	*/
+	    /* aliases for multiple types return the first type	*/
+	    /* encountered (lowest mask bit number).		*/
+
+	    type = DBTechNameTypes(layername, &rmask);
+	}
+	if (type < 0)
+	{
+	    TxError("Warning: label \"%s\" attached to unknown "
+			"type \"%s\"\n", text, layername);
 	    type = TT_SPACE;
 	}
 	else if (type >= DBNumUserLayers)
@@ -1885,7 +2271,7 @@ DBCellWrite(cellDef, fileName)
     FILE *realf, *tmpf;
     int tmpres;
     struct stat statb;
-    bool result;
+    bool result, exists;
 
     result = FALSE;
 
@@ -1908,7 +2294,8 @@ DBCellWrite(cellDef, fileName)
     }
     else if (cellDef->cd_name)
     {
-	realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)+strlen(DBSuffix)+1));
+	realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
+		+ strlen(DBSuffix) + 1));
 	(void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
     }
     else return (FALSE);
@@ -1940,24 +2327,37 @@ DBCellWrite(cellDef, fileName)
 	return(FALSE);
     }
 
-    /*
-     * Figure name of the temp file to write.
-     */
-    tmpname = (char *) mallocMagic((unsigned) (strlen(expandname) + strlen(template) + 1));
-    (void) sprintf(tmpname, "%s%s", expandname, template);
-    tmpres = mkstemp(tmpname);
+    /* Check if the .mag file exists.  If not, we don't need to deal	*/
+    /* with temporary file names.					*/
+    exists = (access(expandname, F_OK) == 0) ? TRUE : FALSE;
 
-    /* Critical: disable interrupts while we do our work */
-    SigDisableInterrupts();
-
-    /* Don't allow a write if the file isn't writeable, or if mkstemp returned	*/
-    /* an error condition.							*/
-
-    if ((tmpres == -1) || (access(expandname, F_OK) == 0 &&
-		file_is_not_writeable (expandname)))
+    if (exists)
     {
-	perror(expandname);
-	goto cleanup;
+
+	/*
+	 * Determine unique name for a temp file to write.
+	 */
+	tmpname = (char *) mallocMagic((unsigned) (strlen(expandname)
+		+ strlen(template) + 1));
+	(void) sprintf(tmpname, "%s%s", expandname, template);
+	tmpres = mkstemp(tmpname);
+	if (tmpres != -1) close(tmpres);	/* We'll re-open it as a stream, below */
+
+	/* Critical: disable interrupts while we do our work */
+	SigDisableInterrupts();
+
+	/* Don't allow a write if the file isn't writeable, or if mkstemp returned	*/
+	/* an error condition.							*/
+
+	if (file_is_not_writeable(expandname))
+	{
+	    perror(expandname);
+	    goto cleanup;
+	}
+    }
+    else
+    {
+	tmpname = StrDup((char **)NULL, expandname);
     }
 
     /*
@@ -1967,6 +2367,18 @@ DBCellWrite(cellDef, fileName)
      */
     if (tmpf = fopen(tmpname, "w"))
     {
+	/* mkstemp() generates a file with permissions 0600.   	*/
+	/* We want instead to respect the user's umask value	*/
+	/* so that the final and original .mag files have the	*/
+	/* same permissions.					*/
+
+	if (exists)
+	{
+	    mode_t userumask = umask((mode_t)0);
+	    umask(userumask);
+	    fchmod(fileno(tmpf), (mode_t)0666 & ~userumask);
+	}
+
 	result = DBCellWriteFile(cellDef, tmpf);
 	(void) fclose(tmpf);
 	tmpf = NULL;
@@ -1980,12 +2392,20 @@ DBCellWrite(cellDef, fileName)
 	    goto cleanup;
 	}
 
+#ifdef FILE_LOCKS
+	if (cellDef->cd_fd != -1)
+	{
+	    close(cellDef->cd_fd);
+	    cellDef->cd_fd = -1;
+	}
+#endif  
+
 	/*
 	 * The temp file is in good shape -- rename it to the real name,
 	 * thereby completing the write.  The error below should NEVER
 	 * normally happen.
 	 */
-	if (rename(tmpname, expandname) < 0)
+	if (exists && (rename(tmpname, expandname) < 0))
 	{
 	    result = FALSE;
 	    perror("rename");
@@ -1996,18 +2416,15 @@ DBCellWrite(cellDef, fileName)
 		    tmpname, expandname, expandname, cellDef->cd_name, tmpname);
 	    goto cleanup;
 	}
+
+#ifdef FILE_LOCKS
 	else
-	{
-	    /* mkstemp() generates a file with permissions 0600 */
-	    /* This is okay for the temporary file, but we want	*/
-	    /* to respect the user's umask value when renaming	*/
-	    /* the file to ".mag".				*/
-	    mode_t userumask = umask((mode_t)0);
-	    umask(userumask);
-	    chmod(expandname, (mode_t)0666 & ~userumask);
-	}
+	    /* Re-aquire the lock on the new file by opening it. */
+	    DBCellRead(cellDef, NULL, TRUE);
+#endif  
+
     }
-    else
+    else if (exists)
     {
 	/*
 	 * Couldn't create a temp file in this directory.  Instead, open
@@ -2081,9 +2498,6 @@ DBCellWrite(cellDef, fileName)
 	      TxError("Warning: I/O error in writing file\n");
 	 }
 	 fclose(realf);
-#ifdef FILE_LOCKS
-	 flock_close(realf, cellDef, expandname);
-#endif
 	 realf = NULL;
     }
 
@@ -2268,10 +2682,7 @@ DBGetTech(cellName)
     static char line[512];
     char *p;
 
-    /* The use of "rb" below is a kludge to prevent this call from	*/
-    /* checking if the cell is locked. mdg 09/14/03			*/
-
-    f = PaOpen(cellName, "rb", DBSuffix, Path, CellLibPath, (char **) NULL);
+    f = PaOpen(cellName, "r", DBSuffix, Path, CellLibPath, (char **) NULL);
     if (f == NULL) return NULL;
 
     p = (char *) NULL;
@@ -2290,55 +2701,156 @@ ret:
     f = NULL;
     return (p);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
- * DBPanicSave --
+ * DBWriteBackup --
  *
- * Save all modified cells to disk, in files "cellname.save".
- * This is intended as an emergency measure for cases when magic
- * has to die (eg, upon receiving a SIGTERM signal).
+ * Save all modified cells to "filename", if specified, or the current
+ * setting of DBbackupFile, which is the current name of the crash
+ * backup file.  If "filename" is NULL and no name has been set for
+ * DBbackupFile, then DBbackupFile is generated as a unique filename
+ * in the temp directory.  If "filename" is non-null, then DBbackupFile
+ * is set to this name, erasing any previous value.  If "filename" is
+ * an empty string, then the DBbackupFile reverts to NULL.
  *
  * Results:
- *	None.
+ *	TRUE if the backup file was created, FALSE if an error was
+ *	encountered.
  *
  * Side effects:
- *	Writes cells to disk.
+ *	Writes cells to disk (in a single file).
+ *	Allocates and sets global variable DBbackupFile.
  *	Does NOT clear the modified bits.
  *
  * ----------------------------------------------------------------------------
  */
 
-void
-DBPanicSave()
+bool
+DBWriteBackup(filename)
+    char *filename;
 {
-    int dbPanicFunc();
-    int flags = CDMODIFIED | CDBOXESCHANGED | CDSTAMPSCHANGED;
+    FILE *f;
+    int fd, pid;
+    char *tempdir;
+    MagWindow *mw;
 
-    DBUpdateStamps();
-    (void) DBCellSrDefs(flags, dbPanicFunc, (ClientData) NULL);
+    int dbWriteBackupFunc(), dbCheckModifiedCellsFunc();
+    int flags = CDMODIFIED;
+    int result;
+
+    /* First check if there are any modified cells that need to be written */
+
+    result = DBCellSrDefs(flags, dbCheckModifiedCellsFunc, (ClientData)NULL);
+    if (result == 0) return TRUE;		/* Nothing to write */
+
+    if (filename == NULL)
+    {
+	if (DBbackupFile == (char *)NULL)
+	{
+	    char *doslash, *template;
+
+	    tempdir = getenv("TMPDIR");
+	    if (tempdir == NULL) tempdir = _PATH_TMP;
+	    template = (char *)mallocMagic(20 + strlen(tempdir));
+	    pid = (int)getpid();
+
+	    doslash = (tempdir[strlen(tempdir) - 1] == '/') ? "" : "/";
+	    sprintf(template, "%s/MAG%d.XXXXXX", tempdir, pid);
+
+	    fd = mkstemp(template);
+	    if (fd == -1)
+	    {
+		TxError("Error generating backup file\n");
+		freeMagic(template);
+		return FALSE;
+	    }
+	    close(fd);
+	    DBbackupFile = StrDup(&DBbackupFile, template);
+	    freeMagic(template);
+	    TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
+	}
+	filename = DBbackupFile;
+    }
+    else
+    {
+	if (strlen(filename) == 0)
+	{
+	    DBbackupFile = StrDup(&DBbackupFile, (char *)NULL);
+	    return TRUE;
+	}
+	DBbackupFile = StrDup(&DBbackupFile, filename);
+	TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
+    }
+
+    f = fopen(filename, "w");
+    if (f == NULL)
+    {
+	TxError("Backup file %s cannot be opened for writing.\n", filename);
+	return FALSE;
+    }
+
+    result = DBCellSrDefs(flags, dbWriteBackupFunc, (ClientData)f);
+
+    /* End by printing the keyword "end" followed by the cell to load	*/
+    /* into the first available window, so that we don't have a default	*/
+    /* blank display after crash recovery.				*/
+
+    mw = WindSearchWid(0);
+    if (mw != NULL)
+	fprintf(f, "end %s\n", ((CellUse *)mw->w_surfaceID)->cu_def->cd_name);
+    else
+	fprintf(f, "end\n");
+    fclose(f);
+    return TRUE;
 }
 
 /*
- * Filter function used by DBPanicSave() above.
- * This function is called for each known CellDef whose modified bit
- * is set.
+ * Filter function used by DBWriteBackup() above.
+ * This function writes a single cell definition to the crash backup
+ * file.  Only editable cells whose paint, labels, or subcells have
+ * changed are considered.
  */
 
-    /*ARGSUSED*/
 int
-dbPanicFunc(def)
+dbWriteBackupFunc(def, f)
     CellDef *def;	/* Pointer to CellDef to be saved */
+    FILE *f;		/* File to append to */
 {
-    static char suffix[] = ".save";
-    char savename[512], *name = def->cd_file;
+    char *name = def->cd_file;
+    int result, save_flags;
+
+    if (def->cd_flags & (CDINTERNAL | CDNOEDIT | CDNOTFOUND)) return 0;
+    else if (!(def->cd_flags & CDAVAILABLE)) return 0;
 
-    if (def->cd_flags & CDINTERNAL) return 0;
     if (name == NULL) name = def->cd_name;
-    strncpy(savename, name, sizeof savename - sizeof suffix);
-    savename[sizeof savename - sizeof suffix] = '\0';
-    strcat(savename, ".save");
-    (void) DBCellWrite(def, savename);
-    return 0;
+
+    fprintf(f, "file %s\n", name);
+    
+    /* Save/restore flags such that the crash recovery file write does	*/
+    /* *not* clear the CDMODIFIED, et al., bits				*/
+
+    save_flags = def->cd_flags;
+    def->cd_flags &= ~(CDGETNEWSTAMP);
+    result = DBCellWriteFile(def, f);
+    def->cd_flags = save_flags;
+    return (result == TRUE) ? FALSE : TRUE;
+}
+
+/*
+ * Filter function used by DBWriteBackup() above.
+ * This function checks if at least one cell needs to be written to the
+ * crash backup file.  Only editable cells whose paint, labels, or
+ * subcells have changed are considered.
+ */
+
+int
+dbCheckModifiedCellsFunc(def, cdata)
+    CellDef *def;	/* Pointer to CellDef to be saved */
+    ClientData cdata;	/* Unused */
+{
+    if (def->cd_flags & (CDINTERNAL | CDNOEDIT | CDNOTFOUND)) return 0;
+    else if (!(def->cd_flags & CDAVAILABLE)) return 0;
+    return 1;
 }
