@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBlabel.c,v 1.2 2006/09/13 21:02:54 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBlabel.c,v 1.3 2006/12/06 23:03:34 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
@@ -748,7 +748,20 @@ DBPickLabelLayer(def, lab, noreconnect)
 	}
 	if (TTMaskHasType(&types[0], i))
 	{
-	    choice5 = i;
+	    /* A type that connects to more than itself is preferred */
+	    if (choice5 == TT_SPACE)
+		choice5 = i;
+	    else
+	    {
+		TileTypeBitMask ctest;
+		TTMaskZero(&ctest);
+		TTMaskSetMask(&ctest, &DBConnectTbl[i]);
+		TTMaskClearType(&ctest, i);
+		if (!TTMaskIsZero(&ctest))
+		    choice5 = i;
+		else if (TTMaskHasType(&types[1], i))
+		    choice6 = i;
+	    }
 	    continue;
 	}
 	else if (TTMaskHasType(&types[1], i))
