@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.3 2007/03/03 19:22:41 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -823,6 +823,39 @@ DBTechFinalContact()
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * DBTechTypesOnPlane --
+ *
+ * This is a substitute for DBTechSubsetLayers() that properly handles
+ * stacked contacts.  Given a tile type bitmask and a plane index, 
+ * check if all types in the bitmask have an image on the indicated plane.
+ *
+ * Results:
+ *	TRUE if all types in "src" have at least one image on plane.
+ *	FALSE if any type in "src" does not contain any image in plane.
+ *
+ * Side effects:
+ *	None.
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBTechTypesOnPlane(src, plane)
+    TileTypeBitMask *src;
+    int plane;
+{
+    int i;
+    PlaneMask pmask;
+
+    for (i = 0; i < DBNumTypes; i++)
+	if (TTMaskHasType(src, i))
+	    if (!PlaneMaskHasPlane(DBTypePlaneMaskTbl[i], plane))
+		return FALSE;
+
+    return TRUE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -936,7 +969,7 @@ DBFullResidueMask(type, rmask)
     }
     else
     {
-	for (t = DBNumUserLayers; t < DBNumTypes; t++)
+	for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
 	    if (TTMaskHasType(lmask, t))
 	    {
 		lr = &dbLayerInfo[t];
