@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtpaint2.c,v 1.4 2001/01/12 22:12:22 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtpaint2.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -46,7 +46,6 @@ LayerInfo *dbTechLpPaint;
 /* Forward declarations */
 
 extern void dbTechPaintErasePlanes();
-extern void dbTechComponents();
 extern void dbComposePaintAllImages();
 extern void dbComposeResidues();
 extern void dbComposeContacts();
@@ -98,7 +97,6 @@ DBTechFinalCompose()
 
     /* Build up exported tables */
     dbTechPaintErasePlanes();
-    dbTechComponents();
 
     /* Diagnostic */
     /* dbTechPrintPaint("DBTechFinalCompose", TRUE, FALSE); */
@@ -149,41 +147,6 @@ dbTechPaintErasePlanes()
 	    }
     }
 }
-
-/* 
- * ----------------------------------------------------------------------------
- *
- * dbTechComponents --
- *
- * Fill in the table that tells for each tile type which
- * other types are "components" of it.
- *
- * Marks a tile type "s" as a component of a type "r" if:
- *	- s and r live on the same plane, and
- *	- the result of painting s on r is still r.
- *
- * Make a second pass and merge the information for all of the
- * images of each contact.  Each type that is a component of a
- * contact's image is also a component of the contact type.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-dbTechComponents()
-{
-    TileType image, paint;
-
-    /* Components are those types that can be painted without effect */
-    for (image = TT_TECHDEPBASE; image < DBNumTypes; image++)
-    {
-	TTMaskZero(&DBComponentTbl[image]);
-	for (paint = TT_TECHDEPBASE; paint < DBNumTypes; paint++)
-	    if (DBPlane(image) == DBPlane(paint)
-		    && DBStdPaintEntry(image, paint, DBPlane(image)) == image)
-		TTMaskSetType(&DBComponentTbl[image], paint);
-    }
-}
 
 /*
  * ----------------------------------------------------------------------------
@@ -368,7 +331,8 @@ void
 dbComposePaintContact(lpImage, lpPaint)
     LayerInfo *lpImage, *lpPaint;
 {
-    int pNum, pmask, pshared;
+    int pNum;
+    PlaneMask pmask, pshared;
     LayerInfo *lp;
     TileTypeBitMask rmask, cmask;
     TileType newtype, ptype, itype;
@@ -659,7 +623,8 @@ void
 dbComposeEraseContact(lpImage, lpErase)
     LayerInfo *lpImage, *lpErase;
 {
-    int pNum, pmask;
+    int pNum;
+    PlaneMask pmask;
     LayerInfo *lp;
     TileTypeBitMask cmask;
     TileType itype;
@@ -865,7 +830,7 @@ dbComposeCompose(imageType, existingType, paintType)
 {
     int pNum = DBPlane(imageType);
 
-    if (PlaneMaskHasPlane(PlaneMask(existingType), pNum))
+    if (PlaneMaskHasPlane(LayerPlaneMask(existingType), pNum))
     {
 	dbSetPaintEntry(existingType, paintType, pNum, imageType);
 	TTMaskSetType(&dbNotDefaultPaintTbl[existingType], paintType);
