@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBundo.c,v 1.3 2001/03/29 19:50:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBundo.c,v 1.2 2006/05/12 05:19:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -372,14 +372,14 @@ dbUndoPaintBack(up)
     typedef struct
     {
 	Rect	 lue_rect;	/* Location of label */
-	char	 lue_pos;	/* Relative position of label */
-	char	 lue_type;	/* Type of tile labelled */
+	int	 lue_pos;	/* Relative position of label */
+	TileType lue_type;	/* Type of tile labelled */
+	unsigned int lue_flags;	/* Flags for label */
 	char	 lue_text[4];	/* Text of label.  This is a place
 				 * holder only; the actual structure
 				 * is allocated to hold all the bytes
 				 * in the label, plus the null byte.
 				 */
-	int	 lue_flags;	/* Flags for label */
     } labelUE;
 
     /*
@@ -553,8 +553,7 @@ dbUndoLabelBack(up)
 	 * The remainder contains a copy of the important
 	 * information from the CellUse.
 	 */
-	unsigned	 cue_expandMask;
-	unsigned	 cue_flags;
+	unsigned int	 cue_expandMask;
 	Transform	 cue_transform;
 	ArrayInfo	 cue_array;
 	CellDef		*cue_def;
@@ -612,7 +611,6 @@ DBUndoCellUse(use, action)
     up->cue_array = use->cu_array;
     up->cue_def = use->cu_def;
     up->cue_parent = use->cu_parent;
-    up->cue_flags = use->cu_flags;
     up->cue_expandMask = use->cu_expandMask;
     up->cue_bbox = use->cu_bbox;
     strcpy(up->cue_id, use->cu_id);
@@ -647,7 +645,6 @@ dbUndoCellForw(up)
 	    use = DBCellNewUse(up->cue_def, (char *) NULL);
 	    use->cu_transform = up->cue_transform;
 	    use->cu_array = up->cue_array;
-	    use->cu_flags = up->cue_flags;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
@@ -708,7 +705,6 @@ dbUndoCellBack(up)
 	    use = DBCellNewUse(up->cue_def, (char *) NULL);
 	    use->cu_transform = up->cue_transform;
 	    use->cu_array = up->cue_array;
-	    use->cu_flags = up->cue_flags;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
