@@ -17,7 +17,7 @@
  *     *********************************************************************
  *
  *
- * rcsid $Header: /ufs/repository/magic/dbwind/dbwind.h,v 1.3 2001/01/12 22:12:25 jsolomon Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/dbwind/dbwind.h,v 1.2 2006/05/24 07:06:22 tim Exp $
  */
 
 #ifndef _DBWIND_H
@@ -87,7 +87,7 @@ typedef struct DBW1 {
     int dbw_scale;
 } DBWclientRec;
 
-/* Flag values for dwb_flags:
+/* Flag values for dbw_flags:
  *
  * DBW_GRID:		Means grid is to be displayed in window.
  * DBW_WATCHDEMO:	Use `demo' style of watching (arrows, not addresses)
@@ -172,6 +172,7 @@ extern void DBWBoxHandler();
 extern MagWindow *ToolGetPoint();
 extern MagWindow *ToolGetBoxWindow();
 extern bool ToolGetBox();
+extern void ToolSnapToGrid();
 extern bool ToolGetEditBox(Rect *);
 extern void ToolMoveBox(), ToolMoveCorner();
 extern int ToolGetCorner();
@@ -187,6 +188,7 @@ extern void DBWHLRemoveClient();
 extern void DBWHLRedraw();
 extern int DBWHLRedrawWind();
 extern void DBWDrawBox();
+extern void DBWDrawCrosshair();
 
 /* Exported procedures and variables relating to feedback: */
 
@@ -210,7 +212,6 @@ extern void DBWElementAddRect();
 extern void DBWElementAddLine();
 extern void DBWElementAddText();
 extern void DBWElementDelete();
-extern void DBWElementInit();
 extern void DBWElementNames();
 extern void DBWElementInbox();
 extern void DBWElementParseFlags();
@@ -225,4 +226,7 @@ extern void DBWUndoBox();
 extern void DBWHLUpdate();
 extern void DBWFeedbackShow();
 
+extern void dbwElementInit();
+extern void dbwCrosshairInit();
+
 #endif /* _DBWIND_H */
