@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCbasic.c,v 1.2 2006/04/14 04:07:02 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCbasic.c,v 1.5 2007/10/11 17:23:51 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
@@ -230,6 +230,7 @@ DRCBasicCheck (celldef, checkRect, clipRect, function, cdata)
         (void) DBSrPaintArea ((Tile *) NULL, celldef->cd_planes[planeNum],
 		checkRect, &DBAllTypeBits, drcTile, (ClientData) &arg);
     }
+    drcCifCheck(&arg);
     return (errors);
 }
 
@@ -292,8 +293,35 @@ drcTile (tile, arg)
     }
 
 #ifdef NONMANHATTAN
-    /* This drc is only for the left edge of the tile */
-    if (IsSplit(tile) && SplitSide(tile)) goto checkbottom;
+    if (IsSplit(tile))
+    {
+	/* Check rules for DRC_ANGLES rule and process */
+	TileType tt = TiGetLeftType(tile);
+	if (tt != TT_SPACE)
+	{
+	    for (cptr = DRCCurStyle->DRCRulesTbl[TT_SPACE][tt];
+			cptr != (DRCCookie *) NULL; cptr = cptr->drcc_next)
+		if (cptr->drcc_flags & DRC_ANGLES)
+		{
+		    drcCheckAngles(tile, arg, cptr);
+		    break;
+		}
+	}
+	tt = TiGetRightType(tile);
+	if (tt != TT_SPACE)
+	{
+	    for (cptr = DRCCurStyle->DRCRulesTbl[TT_SPACE][tt];
+			cptr != (DRCCookie *) NULL; cptr = cptr->drcc_next)
+		if (cptr->drcc_flags & DRC_ANGLES)
+		{
+		    drcCheckAngles(tile, arg, cptr);
+		    break;
+		}
+	}
+
+        /* This drc is only for the left edge of the tile */
+	if (SplitSide(tile)) goto checkbottom;
+    }
 #endif  
 
     /*
@@ -408,6 +436,8 @@ drcTile (tile, arg)
 		for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 			cptr = cptr->drcc_next)
 		{
+		    if (cptr->drcc_flags & DRC_ANGLES) continue;
+
 		    /* Find the rule distances according to the scale factor */
 		    dist = cptr->drcc_dist;
 		    cdist = cptr->drcc_cdist;
@@ -450,9 +480,10 @@ drcTile (tile, arg)
 
 		    if (cptr->drcc_flags & DRC_AREA)
 		    {
-			drcCheckArea(tile,arg,cptr);
+			drcCheckArea(tile, arg, cptr);
 			continue;
 		    }
+
 		    if ((cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS)) ==
 				(DRC_MAXWIDTH | DRC_BENDS))
 		    {
@@ -579,7 +610,6 @@ drcTile (tile, arg)
 			}
 			errRect.r_xtop = edgeX;
 			arg->dCD_initial = tile;
-			arg->dCD_which = DRC_LEFT;
 		    }
 		    else  /* FORWARD */
 		    {
@@ -654,7 +684,6 @@ drcTile (tile, arg)
 			}
 			errRect.r_xbot = edgeX;
 			arg->dCD_initial= tpleft;
-			arg->dCD_which = DRC_RIGHT;
 		    }
 		    if (arg->dCD_radial)
 		    {
@@ -774,7 +803,6 @@ checkbottom:
 	/* Go right across bottom of tile */
         for (tpbot = LB(tile); LEFT(tpbot) < right; tpbot = TR(tpbot))
         {
-
 	    /* Get the tile types to the top and bottom of the edge */
 
 	    tt = TiGetBottomType(tile);
@@ -841,6 +869,8 @@ checkbottom:
 		for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 				cptr = cptr->drcc_next)
 		{
+		    if (cptr->drcc_flags & DRC_ANGLES) continue;
+
 		    /* Find the rule distances according to the scale factor */
 		    dist = cptr->drcc_dist;
 		    cdist = cptr->drcc_cdist;
@@ -908,7 +938,8 @@ checkbottom:
 
 			continue;
 		    }
-		    else if (cptr->drcc_flags & (DRC_AREA|DRC_RECTSIZE|DRC_MAXWIDTH))
+		    else if (cptr->drcc_flags & (DRC_AREA | DRC_RECTSIZE
+				| DRC_MAXWIDTH))
 		    {
 			/* only have to do these checks in one direction */
 			if (trigpending) cptr = cptr->drcc_next;
@@ -968,7 +999,6 @@ checkbottom:
 			}
 			errRect.r_ytop = edgeY;
 			arg->dCD_initial = tile;
-			arg->dCD_which = DRC_BOTTOM;
 		    }
 		    else  /* FORWARD */
 		    {
@@ -1023,7 +1053,6 @@ checkbottom:
 			}
 			errRect.r_ybot = edgeY;
 			arg->dCD_initial = tpbot;
-			arg->dCD_which = DRC_TOP;
 		    }
 		    if (arg->dCD_radial)
 		    {
