@@ -24,7 +24,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcif.c,v 1.2 2001/01/12 22:12:28 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCcif.c,v 1.1 2007/10/12 22:14:36 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -48,8 +48,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcif.c,v 1.2 2001/01
 #include "utils/malloc.h"
 #include "utils/utils.h"
 
-#ifdef DRC_CIF_EXTENSIONS
-
 extern char *drcWhyDup(); 
 extern int  drcCifTile();
 extern int  areaCifCheck();
@@ -65,42 +63,70 @@ extern Stack	*DRCstack;
     }
 
 extern CIFStyle	*drcCifStyle;
+extern bool DRCForceReload;
 TileTypeBitMask drcCifGenLayers;
 
-typedef struct drccifcookie
-{
-    int    	       drcc_dist;	/* Extent of rule from edge. */
-    TileTypeBitMask      drcc_mask;	/* Legal types on RHS */
-    TileTypeBitMask      drcc_corner;	/* Types that trigger corner check */
-    struct drccifcookie * drcc_next;
-    char             * drcc_why;	/* Explanation of error found */
-    int		       drcc_cdist;	/* Size of corner extension. */
-    int		       drcc_flags;	/* Miscellaneous flags, see below. */
-    int		       drcc_plane;	
-} DRCCifCookie;
-
-DRCCifCookie	*drcCifRules[MAXCIFLAYERS][2];
+DRCCookie *drcCifRules[MAXCIFLAYERS][2];
+DRCCookie *drcCifCur=NULL;
 int	drcCifValid = FALSE;
+
 #define DRC_CIF_SPACE		0
 #define DRC_CIF_SOLID		1
-DRCCifCookie	*drcCifCur=NULL;
 
-#define ASSIGN(cookie,dist,next,mask,corner,why,cdist,flags,layer) \
-	((cookie)->drcc_dist = dist, \
-	(cookie)->drcc_next = next, \
-	(cookie)->drcc_mask = mask, \
-	(cookie)->drcc_corner = corner, \
-	(cookie)->drcc_why =  why, \
-	(cookie)->drcc_cdist = cdist, \
-	(cookie)->drcc_flags = flags, \
-        (cookie)->drcc_plane = layer)
-
-/* The following DRC cookie is used when there are tiles of type
- * TT_ERROR_S found during the basic DRC.  These arise during
- * hierarchical checking when there are illegal overlaps.
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcCifSetStyle --
+ *
+ * Process a declaration of the cif style.
+ * This is of the form:
+ *
+ *	cifstyle cif_style
+ *
+ * e.g,
+ *
+ *	cifstyle pg
+ *
+ * Results:
+ *	Returns 0.
+ *
+ * Side effects:
+ *	Updates drcCifStyle.  Do NOT attempt to update the CIF style
+ *	in the middle of reading the DRC section.  Instead, if the
+ *	reported CIF style is not current, flag a warning.  The DRC
+ *	will be re-read with the CIF extensions when the CIF output
+ *	style is changed.
+ *
+ * ----------------------------------------------------------------------------
  */
 
-
+int
+drcCifSetStyle(argc, argv)
+    int argc;
+    char *argv[];
+{
+    CIFKeep     *new;
+
+    for (new = CIFStyleList; new != NULL; new = new->cs_next)
+    {
+	if (!strcmp(new->cs_name, argv[1]))
+	{
+	    DRCForceReload = TRUE;
+	    if (!strcmp(new->cs_name, CIFCurStyle->cs_name))
+		drcCifStyle = CIFCurStyle;
+	    else
+	    {
+		TechError("DRC cif extensions are not enabled.\n");
+		TechError("Use \"cif ostyle %s\" to enable them.\n", new->cs_name);
+		drcCifStyle = NULL;
+	    }
+	    return 0;
+	}
+    }
+    TechError("Unknown DRC cifstyle %s\n",argv[1]);
+    return (0);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -127,7 +153,7 @@ drcCifWidth(argc, argv)
     char *why = drcWhyDup(argv[3]);
     TileTypeBitMask set, setC, tmp1;
     int	thislayer = -1;
-    DRCCifCookie *dpnew,*dpnext;
+    DRCCookie *dpnew,*dpnext;
     TileType i;
 
     if (drcCifStyle == NULL)
@@ -153,19 +179,19 @@ drcCifWidth(argc, argv)
 
     scalefactor = drcCifStyle->cs_scaleFactor;
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
-    dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-    ASSIGN(dpnew,centidistance,dpnext,CIFSolidBits,
+    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+    drcAssign(dpnew,centidistance,dpnext,CIFSolidBits,
     		CIFSolidBits,why,centidistance,DRC_FORWARD,thislayer);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
     return ((centidistance+scalefactor-1)/scalefactor);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
  * drcCifSpacing -- same this as drcSpacing, except that it works on cif 
- 	layers.
+ * 	layers.
  *
  * Results:
  *	Returns distance.
@@ -183,7 +209,7 @@ drcCifSpacing(argc, argv)
 {
     char *adjacency = argv[4];
     char *why = drcWhyDup(argv[5]);
-    DRCCifCookie *dpnext, *dpnew;
+    DRCCookie *dpnext, *dpnew;
     int needReverse = FALSE;
     TileType i, j;
     int scalefactor;
@@ -245,8 +271,8 @@ drcCifSpacing(argc, argv)
     
     scalefactor = drcCifStyle->cs_scaleFactor;
     dpnext = drcCifRules[layer[0]][DRC_CIF_SOLID];
-    dpnew = (DRCCifCookie *) mallocMagic((unsigned) sizeof (DRCCifCookie));
-    ASSIGN(dpnew,centidistance,dpnext,DBSpaceBits,
+    dpnew = (DRCCookie *) mallocMagic((unsigned) sizeof (DRCCookie));
+    drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		cmask,why,centidistance,DRC_FORWARD,layer[1]);
     drcCifRules[layer[0]][DRC_CIF_SOLID] = dpnew;
     
@@ -254,8 +280,8 @@ drcCifSpacing(argc, argv)
     {
          dpnew->drcc_flags |= DRC_BOTHCORNERS|DRC_XPLANE;
          dpnext = drcCifRules[layer[1]][DRC_CIF_SOLID];
-         dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-         ASSIGN(dpnew,centidistance,dpnext,DBSpaceBits,
+         dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+         drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		cmask,why,centidistance,
 		DRC_XPLANE|DRC_FORWARD|DRC_BOTHCORNERS,layer[0]);
          drcCifRules[layer[1]][DRC_CIF_SOLID] = dpnew;
@@ -263,15 +289,15 @@ drcCifSpacing(argc, argv)
 	 if (layer[0] == layer[1])
 	 {
               dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
-              dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-              ASSIGN(dpnew,centidistance,dpnext,DBSpaceBits,
+              dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+              drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		     cmask,why,centidistance,
 		     DRC_XPLANE|DRC_REVERSE|DRC_BOTHCORNERS,layer[0]);
                  drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 	 
               dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
-              dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-              ASSIGN(dpnew,centidistance,dpnext,DBSpaceBits,
+              dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+              drcAssign(dpnew,centidistance,dpnext,DBSpaceBits,
     		     cmask,why,centidistance,
 		     DRC_XPLANE|DRC_REVERSE|DRC_BOTHCORNERS,layer[1]);
                  drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
@@ -281,15 +307,15 @@ drcCifSpacing(argc, argv)
     if (layer[0] != layer[1]) /* make sure they don't overlap exactly */
     {
          dpnext = drcCifRules[layer[1]][DRC_CIF_SPACE];
-         dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-         ASSIGN(dpnew,scalefactor,dpnext,DBSpaceBits,
+         dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+         drcAssign(dpnew,scalefactor,dpnext,DBSpaceBits,
     		DBZeroTypeBits,why,scalefactor,
 		DRC_FORWARD,layer[0]);
          drcCifRules[layer[1]][DRC_CIF_SPACE] = dpnew;
 
          dpnext = drcCifRules[layer[0]][DRC_CIF_SPACE];
-         dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-         ASSIGN(dpnew,scalefactor,dpnext,DBSpaceBits,
+         dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+         drcAssign(dpnew,scalefactor,dpnext,DBSpaceBits,
     		DBZeroTypeBits,why,scalefactor,
 		DRC_FORWARD,layer[1]);
          drcCifRules[layer[0]][DRC_CIF_SPACE] = dpnew;
@@ -297,95 +323,151 @@ drcCifSpacing(argc, argv)
     
     return ((centidistance+scalefactor-1)/scalefactor);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ * ----------------------------------------------------------------------------
+ */
+
+void
+drcCifFreeStyle()
+{
+    DRCCookie *dp;
+    int i;
+    char *old;
+
+    if (DRCCurStyle != NULL)
+    {
+	for (i = 0; i != MAXCIFLAYERS; i++)
+	{
+	    dp = drcCifRules[i][DRC_CIF_SPACE];
+	    while (dp != NULL)
+	    {
+		old = (char *)dp;
+	 	dp = dp->drcc_next;
+		freeMagic(old);
+	    }
+	    dp = drcCifRules[i][DRC_CIF_SOLID];
+	    while (dp != NULL)
+	    {
+		old = (char *)dp;
+	 	dp = dp->drcc_next;
+		freeMagic(old);
+	    }
+	}
+    }
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * ----------------------------------------------------------------------------
+ */
 
 void
 drcCifInit()
 {
-     int	i;
+    int i;
 
-     for (i=0; i != MAXCIFLAYERS; i++)
-     {
-     	  drcCifRules[i][DRC_CIF_SPACE] = NULL;
-     	  drcCifRules[i][DRC_CIF_SOLID] = NULL;
-     }
-     drcCifValid= FALSE;
-     TTMaskZero(&drcCifGenLayers);
+    if (drcCifValid == TRUE)
+	drcCifFreeStyle();
+
+    for (i = 0; i != MAXCIFLAYERS; i++)
+    {
+	drcCifRules[i][DRC_CIF_SPACE] = NULL;
+	drcCifRules[i][DRC_CIF_SOLID] = NULL;
+    }
+    drcCifValid = FALSE;
+    TTMaskZero(&drcCifGenLayers);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * ----------------------------------------------------------------------------
+ */
+
 void
 drcCifFinal()
 {
-     int i;
+    int i;
 
-     for (i=0; i != MAXCIFLAYERS; i++)
-     {
-          DRCCifCookie *dp;
+    for (i = 0; i != MAXCIFLAYERS; i++)
+    {
+        DRCCookie *dp;
      	  
-	  for (dp =  drcCifRules[i][DRC_CIF_SPACE];dp;dp=dp->drcc_next)
-	  {
-     	       drcCifValid= TRUE;
-               TTMaskSetType(&drcCifGenLayers,i);
-               TTMaskSetType(&drcCifGenLayers,dp->drcc_plane);
-	  }
-	  for (dp =  drcCifRules[i][DRC_CIF_SOLID];dp;dp=dp->drcc_next)
-	  {
-     	       drcCifValid= TRUE;
-               TTMaskSetType(&drcCifGenLayers,i);
-               TTMaskSetType(&drcCifGenLayers,dp->drcc_plane);
-	  }
-     }
+	for (dp = drcCifRules[i][DRC_CIF_SPACE]; dp; dp = dp->drcc_next)
+	{
+	    drcCifValid = TRUE;
+            TTMaskSetType(&drcCifGenLayers, i);
+            TTMaskSetType(&drcCifGenLayers, dp->drcc_plane);
+	}
+	for (dp = drcCifRules[i][DRC_CIF_SOLID]; dp; dp = dp->drcc_next)
+	{
+     	    drcCifValid = TRUE;
+            TTMaskSetType(&drcCifGenLayers, i);
+            TTMaskSetType(&drcCifGenLayers, dp->drcc_plane);
+	}
+    }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * drcCifCheck---
+ *
+ * This is the primary routine for design-rule checking on CIF layers.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Error paint, CIF layer generation, lots of stuff going on.
+ *	
+ * ----------------------------------------------------------------------------
+ */
+
 void
 drcCifCheck(arg)
-    struct drcClientData	*arg;
-
+    struct drcClientData *arg;
 {
-     Rect		*checkRect = arg->dCD_rect;
-     Rect		cifrect;
-     int		scale;
-     CIFStyle		*tmp;
-     int		i,j;
-     int oldTiles;
+    Rect	*checkRect = arg->dCD_rect;
+    Rect	cifrect;
+    int		scale;
+    int		i,j;
+    int		oldTiles;
 
-     if (drcCifValid == FALSE) return;
-     scale = drcCifStyle->cs_scaleFactor;
-     tmp = CIFCurStyle;
-     CIFCurStyle = drcCifStyle;
-     cifrect = *checkRect;
-     cifrect.r_xbot *= scale;
-     cifrect.r_xtop *= scale;
-     cifrect.r_ybot *= scale;
-     cifrect.r_ytop *= scale;
-     arg->dCD_rect = &cifrect;
-     oldTiles = DRCstatTiles;
+    if (drcCifValid == FALSE) return;
+    else if (CIFCurStyle != drcCifStyle) return;
 
-/*   CIFGen(arg->dCD_celldef,checkRect,CIFPlanes,&drcCifGenLayers,TRUE,FALSE); */
-     CIFGen(arg->dCD_celldef,checkRect,CIFPlanes,&DBAllTypeBits,TRUE,TRUE);
-     for (i = 0; i < drcCifStyle->cs_nLayers;i++)
-     {
-          for (j = 0; j != 2;j++)
-	  {
-	       for (drcCifCur=drcCifRules[i][j]; 
-	       			drcCifCur;drcCifCur=drcCifCur->drcc_next)
-               {
-     	            Plane *plane = CIFPlanes[i];
-	  	    TileTypeBitMask	*mask;
-		    
-		    mask = (j == DRC_CIF_SOLID)?(&DBSpaceBits):&CIFSolidBits;
+    scale = drcCifStyle->cs_scaleFactor;
+    cifrect = *checkRect;
+    cifrect.r_xbot *= scale;
+    cifrect.r_xtop *= scale;
+    cifrect.r_ybot *= scale;
+    cifrect.r_ytop *= scale;
+    arg->dCD_rect = &cifrect;
+    oldTiles = DRCstatTiles;
+
+    CIFGen(arg->dCD_celldef, checkRect, CIFPlanes, &DBAllTypeBits, TRUE, TRUE);
+
+    for (i = 0; i < drcCifStyle->cs_nLayers; i++)
+    {
+        for (j = 0; j != 2; j++)
+	{
+	    for (drcCifCur = drcCifRules[i][j]; 
+	       		drcCifCur; drcCifCur = drcCifCur->drcc_next)
+            {
+     	        Plane *plane = CIFPlanes[i];
+	  	TileTypeBitMask	*mask;
 	  
-	            DBSrPaintArea((Tile *) NULL,plane,&cifrect,mask,
+	        DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &cifrect,
+			(j == DRC_CIF_SOLID) ? &DBSpaceBits : &CIFSolidBits,
 	  		drcCifTile, arg);
-     	       }
-	       
-	  }
+     	     }
+	 }
      }
      arg->dCD_rect = checkRect;
-     CIFCurStyle = tmp;
      DRCstatCifTiles += DRCstatTiles - oldTiles;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -406,13 +488,14 @@ drcCifTile (tile, arg)
     Tile *tile;	/* Tile being examined */
     struct drcClientData *arg;
 {
-    DRCCifCookie *cptr;	/* Current design rule on list */
+    DRCCookie *cptr;	/* Current design rule on list */
     Tile *tp;		/* Used for corner checks */
     Rect *rect = arg->dCD_rect;	/* Area being checked */
     Rect errRect;		/* Area checked for an individual rule */
     TileTypeBitMask tmpMask;
 
     arg->dCD_constraint = &errRect;
+    arg->dCD_radial = 0;
 
     /*
      * If we were interrupted, we want to
@@ -457,12 +540,12 @@ drcCifTile (tile, arg)
 
     if (drcCifCur->drcc_flags & DRC_AREA)
     {
-    	 drcCheckCifArea(tile,arg,drcCifCur);
+    	 drcCheckCifArea(tile, arg, drcCifCur);
 	 return 0;
     }
     if (drcCifCur->drcc_flags & DRC_MAXWIDTH)
     {
-    	 drcCheckCifMaxwidth(tile,arg,drcCifCur);
+    	 drcCheckCifMaxwidth(tile, arg, drcCifCur);
 	 return 0;
     }
     if (LEFT(tile) >= rect->r_xbot)		/* check tile against rect */
@@ -482,7 +565,7 @@ drcCifTile (tile, arg)
 	     * involving such edges are encountered during technology readin
 	     * the user is warned that such edges are not checked).
 	     */
-	    if(TiGetType(tpleft) == TiGetType(tile))
+	    if (TiGetRightType(tpleft) == TiGetLeftType(tile))
 	        continue;
 
 	    /*
@@ -508,8 +591,12 @@ drcCifTile (tile, arg)
 		     */
 		    for (tp = tile; BOTTOM(tp) >= errRect.r_ybot; tp = LB(tp))
 			/* Nothing */;
-		    if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+		    if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
+		    {
 			errRect.r_ybot -= cptr->drcc_cdist;
+			if (DRCEuclidean)
+			    arg->dCD_radial |= 0x1000;
+		    }
 
 		    if (cptr->drcc_flags & DRC_BOTHCORNERS)
 		    {
@@ -520,8 +607,12 @@ drcCifTile (tile, arg)
 			if (TOP(tp = tile) <= errRect.r_ytop)
 			    for (tp = RT(tp); LEFT(tp) > edgeX; tp = BL(tp))
 				/* Nothing */;
-			if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+			if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
+			{
 			    errRect.r_ytop += cptr->drcc_cdist;
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x2000;
+			}
 		    }
 
 		    /*
@@ -538,7 +629,7 @@ drcCifTile (tile, arg)
 		    errRect.r_xtop = edgeX;
 		    arg->dCD_initial = tile;
 		}
-		else
+		else	/* FORWARD */
 		{
 		    /*
 		     * Determine corner extensions.
@@ -546,8 +637,12 @@ drcCifTile (tile, arg)
 		     */
 		    for (tp = tpleft; TOP(tp) <= errRect.r_ytop; tp = RT(tp))
 			/* Nothing */;
-		    if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+		    if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
+		    {
 			errRect.r_ytop += cptr->drcc_cdist;
+			if (DRCEuclidean)
+			    arg->dCD_radial |= 0x8000;
+		    }
 
 		    if (cptr->drcc_flags & DRC_BOTHCORNERS)
 		    {
@@ -558,8 +653,12 @@ drcCifTile (tile, arg)
 			if (BOTTOM(tp = tpleft) >= errRect.r_ybot)
 			    for (tp = LB(tp); RIGHT(tp) < edgeX; tp = TR(tp))
 				/* Nothing */;
-			if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+			if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
+			{
 			    errRect.r_ybot -= cptr->drcc_cdist;
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x4000;
+			}
 		    }
 
 		    /*
@@ -571,11 +670,16 @@ drcCifTile (tile, arg)
 			&& BOTTOM(tile) <= errRect.r_ybot
 			&& TOP(tile) >= errRect.r_ytop
 			&& !(cptr->drcc_flags & DRC_XPLANE)
-			&& TTMaskHasType(&cptr->drcc_mask, TiGetType(tile)))
+			&& TTMaskHasType(&cptr->drcc_mask, TiGetLeftType(tile)))
 			    continue;
 		    errRect.r_xbot = edgeX;
 		    arg->dCD_initial= tpleft;
 		}
+		if (arg->dCD_radial)
+		{
+		    arg->dCD_radial &= 0xf0000;
+		    arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
+		}
 
 		DRCstatSlow++;
 		arg->dCD_cptr = (DRCCookie *)cptr;
@@ -636,7 +740,7 @@ drcCifTile (tile, arg)
 	     * involving such edges are encountered during technology readin
 	     * the user is warned that such edges are not checked).
 	     */
-	    if(TiGetType(tpbot) == TiGetType(tile))
+	    if(TiGetTopType(tpbot) == TiGetBottomType(tile))
 	        continue;
 
 	    /*
@@ -665,8 +769,12 @@ drcCifTile (tile, arg)
 		    if (RIGHT(tp = tile) <= errRect.r_xtop)
 			for (tp = TR(tp); BOTTOM(tp) > edgeY; tp = LB(tp))
 			    /* Nothing */;
-		    if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+		    if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
+		    {
 			errRect.r_xtop += cptr->drcc_cdist; 	
+			if (DRCEuclidean)
+			    arg->dCD_radial |= 0x4000;
+		    }
 
 		    if (cptr->drcc_flags & DRC_BOTHCORNERS)
 		    {
@@ -676,8 +784,12 @@ drcCifTile (tile, arg)
 			 */
 			for (tp = tile; LEFT(tp) >= errRect.r_xbot; tp = BL(tp))
 			    /* Nothing */;
-			if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+			if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
+			{
 			    errRect.r_xbot -= cptr->drcc_cdist; 	
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x1000;
+			}
 		    }
 
 		    /*
@@ -689,12 +801,12 @@ drcCifTile (tile, arg)
 			&& LEFT(tpbot) <= errRect.r_xbot
 			&& RIGHT(tpbot) >= errRect.r_xtop
 			&& !(cptr->drcc_flags & DRC_XPLANE)
-			&& TTMaskHasType(&cptr->drcc_mask, TiGetType(tpbot)))
+			&& TTMaskHasType(&cptr->drcc_mask, TiGetTopType(tpbot)))
 			    continue;
 		    errRect.r_ytop = edgeY;
 		    arg->dCD_initial = tile;
 		}
-		else
+		else	/* FORWARD */
 		{
 		    /*
 		     * Determine corner extensions.
@@ -704,8 +816,12 @@ drcCifTile (tile, arg)
 			for (tp = BL(tp); TOP(tp) < edgeY; tp = RT(tp))
 			    /* Nothing */;
 
-		    if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+		    if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
+		    {
 			errRect.r_xbot -= cptr->drcc_cdist;
+			if (DRCEuclidean)
+			    arg->dCD_radial |= 0x2000;
+		    }
 
 		    if (cptr->drcc_flags & DRC_BOTHCORNERS)
 		    {
@@ -715,8 +831,12 @@ drcCifTile (tile, arg)
 			 */
 			for (tp=tpbot; RIGHT(tp) <= errRect.r_xtop; tp=TR(tp))
 			    /* Nothing */;
-			if (TTMaskHasType(&cptr->drcc_corner, TiGetType(tp)))
+			if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
+			{
 			    errRect.r_xtop += cptr->drcc_cdist;
+			    if (DRCEuclidean)
+				arg->dCD_radial |= 0x8000;
+			}
 		    }
 
 		    /*
@@ -733,6 +853,11 @@ drcCifTile (tile, arg)
 		    errRect.r_ybot = edgeY;
 		    arg->dCD_initial = tpbot;
 		}
+		if (arg->dCD_radial)
+		{
+		    arg->dCD_radial &= 0xf000;
+		    arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
+		}
 
 		DRCstatSlow++;
 		arg->dCD_cptr = (DRCCookie *)cptr;
@@ -746,7 +871,7 @@ drcCifTile (tile, arg)
     }
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -810,7 +935,7 @@ areaCifCheck(tile, arg)
     (*(arg->dCD_errors))++;
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -844,7 +969,7 @@ drcCifArea(argc, argv)
     int	centihorizon = atoi(argv[3]);
     char *why = drcWhyDup(argv[4]);
     TileTypeBitMask set, setC, tmp1;
-    DRCCifCookie *dpnext, *dpnew;
+    DRCCookie *dpnext, *dpnew;
     TileType i, j;
     int plane;
     int	thislayer;
@@ -873,15 +998,15 @@ drcCifArea(argc, argv)
 
     scalefactor = drcCifStyle->cs_scaleFactor;
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
-    dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-    ASSIGN(dpnew,centihorizon,dpnext,CIFSolidBits,
+    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+    drcAssign(dpnew,centihorizon,dpnext,CIFSolidBits,
     		CIFSolidBits,why,centidistance,DRC_AREA|DRC_FORWARD,thislayer);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
     return ((centihorizon+scalefactor-1)/scalefactor);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -906,7 +1031,7 @@ drcCifMaxwidth(argc, argv)
     char *bends = argv[3];
     char *why = drcWhyDup(argv[4]);
     TileTypeBitMask set, setC, tmp1;
-    DRCCifCookie *dpnext, *dpnew;
+    DRCCookie *dpnext, *dpnew;
     TileType i, j;
     int plane;
     int bend;
@@ -944,15 +1069,15 @@ drcCifMaxwidth(argc, argv)
     
     scalefactor = drcCifStyle->cs_scaleFactor;
     dpnext = drcCifRules[thislayer][DRC_CIF_SPACE];
-    dpnew = (DRCCifCookie *) mallocMagic((unsigned) (sizeof (DRCCifCookie)));
-    ASSIGN(dpnew,centidistance,dpnext,CIFSolidBits,
+    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+    drcAssign(dpnew,centidistance,dpnext,CIFSolidBits,
     		CIFSolidBits,why,centidistance,DRC_MAXWIDTH|bend,thislayer);
     drcCifRules[thislayer][DRC_CIF_SPACE] = dpnew;
 
 
     return ((centidistance+scalefactor-1)/scalefactor);
 }
-
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -974,7 +1099,7 @@ void
 drcCheckCifArea(starttile, arg, cptr)
     Tile		 *starttile;
     struct drcClientData *arg;
-    DRCCifCookie	 *cptr;
+    DRCCookie	 *cptr;
 {
      int		arealimit = cptr->drcc_cdist;
      int		area=0;
@@ -1032,10 +1157,6 @@ drcCheckCifArea(starttile, arg, cptr)
 	 if (!GEO_RECTNULL(&rect)) {
 	     (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
 		 arg->dCD_cptr, arg->dCD_clientData);
-	     /***
-	     DBWAreaChanged(arg->dCD_celldef,&rect, DBW_ALLWINDOWS, 
-						    &DBAllButSpaceBits);
-	     ***/
 	     (*(arg->dCD_errors))++;
 	 }
 	 
@@ -1083,7 +1204,6 @@ forgetit:
      }
 }
 
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -1103,16 +1223,16 @@ forgetit:
 
 void
 drcCheckCifMaxwidth(starttile,arg,cptr)
-	Tile	*starttile;
-	struct drcClientData	*arg;
-	DRCCifCookie	*cptr;
+    Tile	*starttile;
+    struct drcClientData *arg;
+    DRCCookie	*cptr;
 
 {
-     int		edgelimit = cptr->drcc_dist;
-     Rect		boundrect;
-     TileTypeBitMask	*oktypes = &cptr->drcc_mask;
-     Tile		*tile,*tp;
-     int 		scale = drcCifStyle->cs_scaleFactor;
+    int			edgelimit = cptr->drcc_dist;
+    Rect		boundrect;
+    TileTypeBitMask	*oktypes = &cptr->drcc_mask;
+    Tile		*tile,*tp;
+    int 		scale = drcCifStyle->cs_scaleFactor;
      
     arg->dCD_cptr = (DRCCookie *)cptr;
     if (DRCstack == (Stack *) NULL)
@@ -1136,10 +1256,6 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
 	      if (!GEO_RECTNULL(&rect)) {
 		  (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
 			arg->dCD_cptr, arg->dCD_clientData);
-		  /***
-		  DBWAreaChanged(arg->dCD_celldef,&rect, DBW_ALLWINDOWS, 
-						    &DBAllButSpaceBits);
-		  ***/
 		  (*(arg->dCD_errors))++;
 	     }
 	 }
@@ -1194,10 +1310,6 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
 	      if (!GEO_RECTNULL(&rect)) {
 		  (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
 			arg->dCD_cptr, arg->dCD_clientData);
-		  /***
-		  DBWAreaChanged(arg->dCD_celldef,&rect, DBW_ALLWINDOWS, 
-						    &DBAllButSpaceBits);
-		  ***/
 		  (*(arg->dCD_errors))++;
 	      }
 	 }
@@ -1244,4 +1356,4 @@ drcCheckCifMaxwidth(starttile,arg,cptr)
 
      }
 }
-#endif
+
