@@ -1,6 +1,6 @@
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCextend.c,v 1.4 2006/07/31 16:00:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCextend.c,v 1.5 2007/10/05 00:29:18 tim Exp $";
 #endif	
 
 #include <sys/types.h>
@@ -26,7 +26,66 @@ Stack *DRCstack = (Stack *)NULL;
         (tp)->ti_client = (ClientData)  DRC_PENDING; \
         STACKPUSH((ClientData) (tp), DRCstack); \
     }
-
+
+#ifdef NONMANHATTAN
+
+/*
+ *-------------------------------------------------------------------------
+ *
+ * drcCheckAngles --- checks whether a tile conforms to orthogonal-only
+ *	geometry (90 degree angles only) or 45-degree geometry (x must
+ *	be equal to y on all non-Manhattan tiles).
+ *
+ * Results: none
+ *
+ * Side Effects: may cause errors to be painted.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+
+void
+drcCheckAngles(tile, arg, cptr)
+    Tile	*tile;
+    struct drcClientData *arg;
+    DRCCookie	*cptr;
+{
+    Rect rect;
+    int ortho = (cptr->drcc_flags & 0x01);  /* 1 = orthogonal, 0 = 45s */
+
+    if (IsSplit(tile))
+    {
+	if (ortho || (RIGHT(tile) - LEFT(tile)) != (TOP(tile) - BOTTOM(tile)))
+	{
+	    TiToRect(tile, &rect);
+	    GeoClip(&rect, arg->dCD_clip);
+	    if (!GEO_RECTNULL(&rect))
+	    {
+		arg->dCD_cptr = cptr;
+		(*(arg->dCD_function)) (arg->dCD_celldef, &rect,
+			arg->dCD_cptr, arg->dCD_clientData);
+		(*(arg->dCD_errors))++;
+	    }
+	}
+    }
+}
+
+#else
+
+/* When non-Manhattan geometry is not compiled in, this routine	*/
+/* is merely a placeholder.					*/
+
+void
+drcCheckAngles(tile, arg, cptr)
+    Tile	*starttile;
+    struct drcClientData *arg;
+    DRCCookie	*cptr;
+{
+    /* Do nothing */
+}
+
+#endif	/* NONMANHATTAN */
+
 /*
  *-------------------------------------------------------------------------
  *
