@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCsubcell.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCsubcell.c,v 1.2 2007/10/11 17:23:51 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -571,15 +571,14 @@ DRCInteractionCheck(def, area, func, cdarg)
 	    scx.scx_use = DRCDummyUse;
 	    scx.scx_trans = GeoIdentityTransform;
 	    DBCellClearDef(DRCdef);
-/*
+
 	    savedPaintTable = DBNewPaintTable(DRCCurStyle->DRCPaintTable);
-	    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce);
-*/
+	    /* savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce); */
+
 	    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, DRCuse);
-/*
+
 	    (void) DBNewPaintTable(savedPaintTable);
-	    (void) DBNewPaintPlane(savedPaintPlane);
-*/
+	    /* (void) DBNewPaintPlane(savedPaintPlane); */
 
 	    /* Run the basic checker over the interaction area. */
 
@@ -639,15 +638,15 @@ DRCFlatCheck(use, area)
 	    scx.scx_use = use;
 	    scx.scx_trans = GeoIdentityTransform;
 	    DBCellClearDef(DRCdef);
-/*
+
 	    savedPaintTable = DBNewPaintTable(DRCCurStyle->DRCPaintTable);
-	    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce);
-*/
+	    /* savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce); */
+
 	    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, DRCuse);
-/*
+
 	    (void) DBNewPaintTable(savedPaintTable);
-	    (void) DBNewPaintPlane(savedPaintPlane);
-*/
+	    /* (void) DBNewPaintPlane(savedPaintPlane); */
+
 	    (void) DRCBasicCheck(DRCdef, &scx.scx_area, &chunk,
 		drcIncCount, (ClientData) &drcFlatCount);
 	}
