@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCtech.c,v 1.11 2006/11/17 22:02:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCtech.c,v 1.21 2007/10/16 17:47:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -35,6 +35,11 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/tech.h"
 #include "textio/textio.h"
 #include "utils/malloc.h"
+#include "cif/cif.h"
+#include "cif/CIFint.h"
+
+CIFStyle *drcCifStyle = NULL;
+bool	 DRCForceReload = FALSE;
 
 /*
  * DRC interaction radius being used (not necessarily the same as
@@ -63,7 +68,9 @@ int drcWidth(), drcSpacing(), drcEdge(), drcNoOverlap(), drcExactOverlap();
 int drcExtend();
 int drcSurround(), drcRectOnly(), drcOverhang();
 int drcStepSize();
-int drcMaxwidth(), drcArea(), drcRectangle();
+int drcMaxwidth(), drcArea(), drcRectangle(), drcAngles();
+int drcCifSetStyle(), drcCifWidth(), drcCifSpacing();
+int drcCifMaxwidth(), drcCifArea();
 
 void DRCTechStyleInit();
 void drcLoadStyle();
@@ -372,6 +379,41 @@ drcLoadStyle(stylename)
     DRCTechScale(DBLambda[0], DBLambda[1]);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * DRCReloadCurStyle ---
+ *
+ * This routine is used by CIFLoadStyle whenever the DRC section makes
+ * reference to CIF layers.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	DRC rule database is deleted and regenerated.
+ *	
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DRCReloadCurStyle()
+{
+    char *stylename;
+    DRCKeep * style;
+
+    if (DRCCurStyle == NULL) return;
+
+    for (style = DRCStyleList; style != NULL; style = style->ds_next)
+    {
+	if (!strcmp(style->ds_name, DRCCurStyle->ds_name))
+	{
+	    DRCCurStyle->ds_name = NULL;
+	    drcLoadStyle(style->ds_name);
+	    break;
+	}
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  * DRCTechInit --
@@ -440,7 +482,8 @@ DRCTechStyleInit()
     TTMaskZero(&DRCCurStyle->DRCExactOverlapTypes);
     DRCCurStyle->DRCWhyList = NULL;
     DRCCurStyle->DRCTechHalo = 0;
-    DRCCurStyle->DRCScaleFactor = 1;
+    DRCCurStyle->DRCScaleFactorN = 1;
+    DRCCurStyle->DRCScaleFactorD = 1;
     DRCCurStyle->DRCStepSize = 0;
 
     /* Put a dummy rule at the beginning of the rules table for each entry */
@@ -502,6 +545,8 @@ DRCTechStyleInit()
 		else
 		    DRCCurStyle->DRCPaintTable[plane][i][j] = result;
 	    }
+
+    drcCifInit();
 }
 
 /*
@@ -672,6 +717,16 @@ DRCTechLine(sectionName, argc, argv)
 	if (DRCTechLine(sectionName, 2, locargv) == FALSE)
 	    return FALSE;
     }
+    else if (DRCStyleList->ds_next == NULL)
+    {
+	/* On reload, if there is only one style, use it.  This is	*/
+	/* necessary for the DRC-CIF extensions, since even though	*/
+	/* the one-and-only DRC section may have been loaded, the DRC-	*/
+	/* CIF parts of it become enabled or disabled depending on what	*/
+	/* CIFCurStyle is active.					*/
+
+	DRCCurStyle->ds_status = TECH_PENDING;
+    }
 
     /* Only continue past this point if we are loading the DRC style */
 
@@ -683,20 +738,27 @@ DRCTechLine(sectionName, argc, argv)
 
     if (strcmp(argv[0], "scalefactor") == 0)
     {
-	int scale;
+	int scaleN, scaleD;
 
-	if (argc != 2) goto wrongNumArgs;
+	if (argc != 2 && argc != 3) goto wrongNumArgs;
 	
-	scale = atoi(argv[1]);
+	scaleN = atof(argv[1]);
 
-	if (scale <= 0 || scale > 255)
+	if (argc == 3)
+	    scaleD = atof(argv[2]);
+	else
+	    scaleD = 1;
+
+	if (scaleN <= 0 || scaleN > 255 || scaleD <= 0 || scaleD > 255)
 	{
 	    TechError("Scale factor must be between 1 and 255.\n");
 	    TechError("Setting scale factor to default value 1.\n");
-	    DRCCurStyle->DRCScaleFactor = 1;
+	    DRCCurStyle->DRCScaleFactorN = 1;
+	    DRCCurStyle->DRCScaleFactorD = 1;
 	    return TRUE;
 	}
-	DRCCurStyle->DRCScaleFactor = scale;
+	DRCCurStyle->DRCScaleFactorN = scaleN;
+	DRCCurStyle->DRCScaleFactorD = scaleD;
 	return TRUE;
     }
 
@@ -832,6 +894,8 @@ DRCTechAddRule(sectionName, argc, argv)
 	int    (*rk_proc)();	/* Procedure implementing this keyword */
 	char	*rk_err;	/* Error message */
     } ruleKeys[] = {
+	"angles",	 4,	4,	drcAngles,
+    "layers 45|90 why",
 	"edge",		 8,	9,	drcEdge,
     "layers1 layers2 distance okTypes cornerTypes cornerDistance why [plane]",
 	"edge4way",	 8,	9,	drcEdge,
@@ -859,6 +923,16 @@ DRCTechAddRule(sectionName, argc, argv)
         "area",		 5,	5,	drcArea,
     "layers area horizon why",
         "maxwidth",	 4,	5,	drcMaxwidth,
+    "layers maxwidth bends why",
+	"cifstyle",	 2,	2,	drcCifSetStyle,
+    "cif_style",
+	"cifwidth",	 4,	4,	drcCifWidth,
+    "layers width why",
+	"cifspacing",	 6,	6,	drcCifSpacing,
+    "layers1 layers2 separation adjacency why",
+	"cifarea",	 5,	5,	drcCifArea,
+    "layers area horizon why",
+	"cifmaxwidth",	 5,	5,	drcCifMaxwidth,
     "layers maxwidth bends why",
 	"rectangle",	5,	5,	drcRectangle,
     "layers maxwidth [even|odd|any] why",
@@ -1045,29 +1119,30 @@ drcWidth(argc, argv)
     char *layers = argv[1];
     int distance = atoi(argv[2]);
     char *why = drcWhyDup(argv[3]);
-    TileTypeBitMask set, setC, tmp1;
+    TileTypeBitMask set, setC;
     PlaneMask pmask;
     DRCCookie *dp, *dpnew;
     TileType i, j;
     int plane;
 
-    pmask = DBTechNoisyNameMask(layers, &tmp1);
+    pmask = DBTechNoisyNameMask(layers, &set);
     if (pmask == 0)
     {
 	TechError("All layers for \"width\" must be on same plane\n");
 	return (0);
     }
 
-    /* In the case of contacts, DBTechSubsetLayers may match in more	*/
-    /* than one plane, in which case we want to consider all planes.	*/
+    /* In the case of contacts, all layers in "layers" may match in	*/
+    /* more than one plane, in which case we want to consider all valid	*/
+    /* matching planes.							*/
 
     for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
     {
-	if (DBTechSubsetLayers(tmp1, DBPlaneTypes[plane], &set))
-	{
-	    /* Use the smallest subset of specified & matching planes */
-	    if ((pmask & PlaneNumToMaskBit(plane)) == 0) continue;
+	/* Ignore planes that are not in pmask */
+	if (!PlaneMaskHasPlane(pmask, plane)) continue;
 
+	if (DBTechTypesOnPlane(&set, plane))
+	{
 	    TTMaskCom2(&setC, &set);
 
 	    for (i = 0; i < DBNumUserLayers; i++)
@@ -1079,19 +1154,17 @@ drcWidth(argc, argv)
 		     * to the right of any edge between a type in '~set'
 		     * and a type in 'set'.
 		     */
-		    for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
-		    {
-			if (DBTypeOnPlane(i, plane) && DBTypeOnPlane(j, plane) &&
+
+		    if (DBTypeOnPlane(i, plane) && DBTypeOnPlane(j, plane) &&
 		    		TTMaskHasType(&setC, i) && TTMaskHasType(&set, j))
-			{
-			    /* Find bucket preceding the new one we wish to insert */
-			    dp = drcFindBucket(i, j, distance);
-			    dpnew = (DRCCookie *)mallocMagic(
+		    {
+			/* Find bucket preceding the new one we wish to insert */
+			dp = drcFindBucket(i, j, distance);
+			dpnew = (DRCCookie *)mallocMagic(
 					(unsigned)sizeof (DRCCookie));
-			    drcAssign(dpnew, distance, dp->drcc_next, set, set,
+			drcAssign(dpnew, distance, dp->drcc_next, set, set,
 					why, distance, DRC_FORWARD, plane);
-			    dp->drcc_next = dpnew;
-			}
+			dp->drcc_next = dpnew;
 		    }
 		}
 	    }
@@ -1199,8 +1272,8 @@ drcArea(argc, argv)
  * This routine was updated 3/6/05 to match the "canonical" definition of
  * a maxwidth region, which is any rectangle containing <layers> that is
  * at least <distance> in both width and height.  If the keyword
- * "bends_illegal" is present, then the definition reverts to the original
- * (see below) for backwards-compatibility.  Otherwise ("bends_ok" or
+ * "bend_illegal" is present, then the definition reverts to the original
+ * (see below) for backwards-compatibility.  Otherwise ("bend_ok" or
  * nothing), the new routine is used.
  *
  *	maxwidth metal1 389 "metal1 width > 35um must be slotted"
@@ -1259,7 +1332,13 @@ drcMaxwidth(argc, argv)
 
     if (argc == 4)
     {
-	bend = DRC_BENDS;
+	/* "bends" is irrelevent if distance is zero, so choose the	*/
+	/* faster algorithm to process					*/
+
+	if (distance == 0)
+	    bend = 0;
+	else
+	    bend = DRC_BENDS;
 	why = drcWhyDup(argv[3]);
     }
     else
@@ -1304,6 +1383,75 @@ drcMaxwidth(argc, argv)
     return (distance);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * drcAngles --
+ *
+ * Process an "angles" rule specifying that geometry for a certain layer
+ * must be limited to 90 degrees or 45 degrees.  If not specified, any
+ * angle is allowed, although width rules will flag errors on acute angles.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+#ifdef NONMANHATTAN
+
+int
+drcAngles(argc, argv)
+    int argc;
+    char *argv[];
+{
+    char *layers = argv[1];
+    int angles = atoi(argv[2]);
+    char *why = drcWhyDup(argv[3]);
+    TileTypeBitMask set;
+    DRCCookie *dp, *dpnew;
+    int plane;
+    TileType i, j;
+
+    DBTechNoisyNameMask(layers, &set);
+
+    angles /= 45;
+    angles--;		/* angles now 0 for 45s and 1 for 90s */
+
+    if ((angles != 0) && (angles != 1))
+    {
+	TechError("angles must be 45 or 90\n");
+	return 0;
+    }
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	if (TTMaskHasType(&set, i))
+	{
+	    /* Insert rule at boundary of tile and TT_SPACE.  This is	*/
+	    /* processed for each tile, separately from other rules, so	*/
+	    /* we don't really care what the edge is;  TT_SPACE is	*/
+	    /* chosen as an arbitrary place to hold the rule.		*/
+
+	    dp = drcFindBucket(TT_SPACE, i, 1);
+	    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+	    drcAssign(dpnew, 1, dp->drcc_next, set, set, why,
+			1, DRC_ANGLES | angles, DBPlane(i));
+	    dp->drcc_next = dpnew;
+	}
+    }
+    return 1;
+}
+
+#else
+
+int
+drcAngles(argc, argv)
+    int argc;
+    char *argv[];
+{
+    TechError("Angle limits ignored (no non-Manhattan geometry allowed).\n");
+    return 0;
+}
+
+#endif	/* NONMANHATTAN */
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2358,71 +2506,75 @@ drcSurround(argc, argv)
 		    }
 		}
 	    }
+
+	    if (strcmp(presence, "absence_illegal")) continue;
+	    if (!DBTechSubsetLayers(setM, DBPlaneTypes[plane2], &tmp2)) continue;
+	    if ((pmask & PlaneNumToMaskBit(plane2)) == 0) continue;
+
+	    /* Add an extra rule when presence of the surrounding	*/
+	    /* layer is required.  Rule is different if planes match.	*/
+
+	    if (plane1 == plane2)
+	    {
+		setM = tmp2;
+		TTMaskZero(&invM);
+		TTMaskSetMask(&invM, &setM);
+		TTMaskSetMask(&invM, &set2);
+		TTMaskCom(&invM);
+		TTMaskAndMask(&invM, &DBPlaneTypes[plane1]);
+
+		TTMaskZero(&set1);
+
+		for (i = 0; i < DBNumUserLayers; i++)
+		    for (j = 0; j < DBNumUserLayers; j++)
+			if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
+			{
+			    /* Find bucket preceding the new one we wish to insert */
+			    dp = drcFindBucket(i, j, distance);
+			    dpnew = (DRCCookie *) mallocMagic((unsigned)
+					(sizeof (DRCCookie)));
+			    drcAssign(dpnew, distance, dp->drcc_next, set1,
+					invM, why, distance, DRC_FORWARD, plane2);
+			    dpnew->drcc_flags |= DRC_BOTHCORNERS;
+			    dp->drcc_next = dpnew;
+
+			    /* find bucket preceding new one we wish to insert */
+			    dp = drcFindBucket(j, i, distance);
+			    dpnew = (DRCCookie *) mallocMagic((unsigned)
+					(sizeof (DRCCookie)));
+			    drcAssign(dpnew,distance,dp->drcc_next, set1,
+					invM, why, distance, DRC_REVERSE, plane2);
+			    dpnew->drcc_flags |= DRC_BOTHCORNERS;
+			    dp->drcc_next = dpnew;
+			}
+	    }
+	    else
+	    {
+		for (i = 0; i < DBNumUserLayers; i++)
+		    for (j = 0; j < DBNumUserLayers; j++)
+			if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
+			{
+			    /* Find bucket preceding the new one we wish to insert */
+			    dp = drcFindBucket(i, j, distance);
+			    dpnew = (DRCCookie *) mallocMagic((unsigned)
+					(sizeof (DRCCookie)));
+			    drcAssign(dpnew, distance, dp->drcc_next,
+					set2, invM, why, distance, DRC_FORWARD, plane2);
+			    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
+			    dp->drcc_next = dpnew;
+
+			    /* find bucket preceding new one we wish to insert */
+			    dp = drcFindBucket(j, i, distance);
+			    dpnew = (DRCCookie *) mallocMagic((unsigned)
+					(sizeof (DRCCookie)));
+			    drcAssign(dpnew,distance,dp->drcc_next,
+					set2, invM, why, distance, DRC_REVERSE, plane2);
+			    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
+			    dp->drcc_next = dpnew;
+			}
+	    }
 	}
     }
-
-    if (strcmp(presence, "absence_illegal")) return distance;
-    if (!DBTechSubsetLayers(setM, DBPlaneTypes[plane2], &tmp2)) return distance;
-    if ((pmask & PlaneNumToMaskBit(plane2)) == 0) return distance;
-
-    /* Add an extra rule when presence of the surrounding	*/
-    /* layer is required.  Rule is different if planes match.	*/
-
-    if (plane1 == plane2)
-    {
-	setM = tmp2;
-	TTMaskZero(&invM);
-	TTMaskSetMask(&invM, &setM);
-	TTMaskSetMask(&invM, &set2);
-	TTMaskCom(&invM);
-	TTMaskAndMask(&invM, &DBPlaneTypes[plane1]);
-
-	TTMaskZero(&set1);
-
-	for (i = 0; i < DBNumUserLayers; i++)
-	    for (j = 0; j < DBNumUserLayers; j++)
-		if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
-		{
-		    /* Find bucket preceding the new one we wish to insert */
-		    dp = drcFindBucket(i, j, distance);
-		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
-		    drcAssign(dpnew, distance, dp->drcc_next, set1,
-				invM, why, distance, DRC_FORWARD, plane2);
-		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
-		    dp->drcc_next = dpnew;
-
-		    /* find bucket preceding new one we wish to insert */
-		    dp = drcFindBucket(j, i, distance);
-		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
-		    drcAssign(dpnew,distance,dp->drcc_next, set1,
-				invM, why, distance, DRC_REVERSE, plane2);
-		    dpnew->drcc_flags |= DRC_BOTHCORNERS;
-		    dp->drcc_next = dpnew;
-		}
-    }
-    else
-    {
-	for (i = 0; i < DBNumUserLayers; i++)
-	    for (j = 0; j < DBNumUserLayers; j++)
-		if (TTMaskHasType(&setM, i) && TTMaskHasType(&invM, j))
-		{
-		    /* Find bucket preceding the new one we wish to insert */
-		    dp = drcFindBucket(i, j, distance);
-		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
-		    drcAssign(dpnew, distance, dp->drcc_next,
-				set2, invM, why, distance, DRC_FORWARD, plane2);
-		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
-		    dp->drcc_next = dpnew;
-
-		    /* find bucket preceding new one we wish to insert */
-		    dp = drcFindBucket(j, i, distance);
-		    dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
-		    drcAssign(dpnew,distance,dp->drcc_next,
-				set2, invM, why, distance, DRC_REVERSE, plane2);
-		    dpnew->drcc_flags |= DRC_BOTHCORNERS | DRC_XPLANE;
-		    dp->drcc_next = dpnew;
-		}
-    }
     return distance;
 }
 
@@ -2756,14 +2908,15 @@ DRCTechFinal()
  */
 
 void
-drcScaleDown(style)
-   DRCStyle *style;
+drcScaleDown(style, scalefactor)
+    DRCStyle *style;
+    int scalefactor;
 {
     TileType i, j;
     DRCCookie  *dp;
     int dist;
 
-    if (style->DRCScaleFactor > 1)
+    if (scalefactor > 1)
     {
 	for (i = 0; i < TT_MAXTYPES; i++)
 	    for (j = 0; j < TT_MAXTYPES; j++)
@@ -2772,17 +2925,19 @@ drcScaleDown(style)
 		    if  (dp->drcc_dist > 0)
 		    {
 			dist = dp->drcc_dist;
-			dp->drcc_dist /= style->DRCScaleFactor;
-			if ((dp->drcc_mod = (unsigned char)(dist
-				% style->DRCScaleFactor)) != 0)
-			    dp->drcc_dist++;
+			dp->drcc_dist /= scalefactor;
+			if ((dp->drcc_mod = (unsigned char)(dist % scalefactor)) != 0)
+			    if (!(dp->drcc_flags & DRC_MAXWIDTH))
+				dp->drcc_dist++;
 		    }
 		    if  (dp->drcc_cdist > 0)
 		    {
+			int locscale = scalefactor;
+			if (dp->drcc_flags & DRC_AREA) locscale *= scalefactor;
+
 			dist = dp->drcc_cdist;
-			dp->drcc_cdist /= style->DRCScaleFactor;
-			if ((dp->drcc_cmod = (unsigned char)(dist
-				% style->DRCScaleFactor)) != 0)
+			dp->drcc_cdist /= locscale;
+			if ((dp->drcc_cmod = (unsigned char)(dist % locscale)) != 0)
 			    dp->drcc_cdist++;
 		    }
 		}
@@ -2805,8 +2960,9 @@ drcScaleDown(style)
  */
 
 void
-drcScaleUp(style)
-   DRCStyle *style;
+drcScaleUp(style, scalefactor)
+    DRCStyle *style;
+    int scalefactor;
 {
     TileType i, j;
     DRCCookie  *dp;
@@ -2814,7 +2970,7 @@ drcScaleUp(style)
 
     if (style == NULL) return;
 
-    if (style->DRCScaleFactor > 1)
+    if (scalefactor > 1)
     {
 	for (i = 0; i < TT_MAXTYPES; i++)
 	    for (j = 0; j < TT_MAXTYPES; j++)
@@ -2824,8 +2980,9 @@ drcScaleUp(style)
 		    {
 			dist = dp->drcc_dist;
 			if (dp->drcc_mod != 0)
-			    dp->drcc_dist--;
-			dp->drcc_dist *= style->DRCScaleFactor;
+			    if (!(dp->drcc_flags & DRC_MAXWIDTH))
+				dp->drcc_dist--;
+			dp->drcc_dist *= scalefactor;
 			dp->drcc_dist += (short)dp->drcc_mod;
 			dp->drcc_mod = 0;
 		    }
@@ -2834,7 +2991,9 @@ drcScaleUp(style)
 			dist = dp->drcc_cdist;
 			if (dp->drcc_cmod != 0)
 			    dp->drcc_cdist--;
-			dp->drcc_cdist *= style->DRCScaleFactor;
+			dp->drcc_cdist *= scalefactor;
+			if (dp->drcc_flags & DRC_AREA)
+			    dp->drcc_cdist *= scalefactor;
 			dp->drcc_cdist += (short)dp->drcc_cmod;
 			dp->drcc_cmod = 0;
 		    }
@@ -2875,7 +3034,8 @@ drcTechFinalStyle(style)
     /* the scale factor, take the ceiling, and save the (negative)	*/
     /* remainder.							*/
 
-    drcScaleDown(style);
+    drcScaleUp(style, style->DRCScaleFactorD);
+    drcScaleDown(style, style->DRCScaleFactorN);
 
     /* Set maximum halo */
     style->DRCTechHalo = DRCTechHalo;
@@ -2902,6 +3062,7 @@ drcTechFinalStyle(style)
 	    freeMagic((char *) dummy); 
 	}
     }
+    drcCifFinal();
 
     if (!DRCRuleOptimization) return;
 
@@ -3062,27 +3223,29 @@ drcTechFinalStyle(style)
 
     /*
      * Add stacked types to all the rules which affect either of the
-     * stacked contact type's residue contacts.
+     * stacked contact type's residue contacts.  For the drcc_mask
+     * mask, all residue types must be allowed for the stacked type to
+     * be allowed.  For the drcc_corner mask, if any residue type is
+     * allowed, then the stacked type is also allowed.
      */
 
     for (i = 0; i < DBNumUserLayers; i++)
     {
-	int k, l;
-	TileTypeBitMask *rMask;
+	int k;
+	TileTypeBitMask *rMask, tmpMask;
 
 	for (j = 0; j < DBNumUserLayers; j++)
 	    for (dp = style->DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
 		for (k = DBNumUserLayers; k < DBNumTypes; k++)
 		{
 		    rMask = DBResidueMask(k);
-		    for (l = 0; l < DBNumUserLayers; l++)
-			if (TTMaskHasType(rMask, l))
-			{
-			    if (TTMaskHasType(&dp->drcc_mask, l))
-				TTMaskSetType(&dp->drcc_mask, k);
-			    if (TTMaskHasType(&dp->drcc_corner, l))
-				TTMaskSetType(&dp->drcc_corner, k);
-			}
+		    TTMaskAndMask3(&tmpMask, rMask, &dp->drcc_mask);
+		    if (TTMaskEqual(&tmpMask, rMask))
+			TTMaskSetType(&dp->drcc_mask, k);
+
+		    TTMaskAndMask3(&tmpMask, rMask, &dp->drcc_corner);
+		    if (!TTMaskIsZero(&tmpMask))
+			TTMaskSetType(&dp->drcc_corner, k);
 		}
     }
 }
@@ -3169,18 +3332,8 @@ DRCTechScale(scalen, scaled)
     if (DRCCurStyle == NULL) return;
     else if (scalen == scaled == 1) return;
 
-    drcScaleUp(DRCCurStyle);
-
-    for (i = 0; i < DBNumTypes; i++)
-	for (j = 0; j < DBNumTypes; j++)
-	    for (dp = DRCCurStyle->DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
-		if (dp->drcc_dist > 0)
-		{
-		    dp->drcc_dist *= scaled;
-		    dp->drcc_dist /= scalen;
-		    dp->drcc_cdist *= scaled;
-		    dp->drcc_cdist /= scalen;
-		}
+    drcScaleUp(DRCCurStyle, scaled);
+    drcScaleDown(DRCCurStyle, scalen);
 
     DRCTechHalo *= scaled;
     DRCTechHalo /= scalen;
@@ -3193,8 +3346,6 @@ DRCTechScale(scalen, scaled)
 
     DRCCurStyle->DRCStepSize *= scaled;
     DRCCurStyle->DRCStepSize /= scalen;
-
-    drcScaleDown(DRCCurStyle);
 }
 
 /* The following routines are used by the "tech" command (and in other places,
