@@ -17,7 +17,7 @@
  *
  * Needs to include: magic.h database.h
  *
- * rcsid $Header: /usr/cvsroot/magic-7.5/drc/drc.h,v 1.2 2006/07/28 15:57:57 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/drc/drc.h,v 1.7 2007/10/12 22:14:36 tim Exp $
  */
 
 #ifndef _DRC_H
@@ -66,10 +66,11 @@ typedef struct drccookie
 #define		DRC_MAXWIDTH		0x10000
 #define		DRC_BENDS		0x20000
 #define		DRC_RECTSIZE		0x40000
-#define 	DRC_WEIRDONES		(DRC_AREA|DRC_MAXWIDTH|DRC_RECTSIZE)
+#define		DRC_ANGLES		0x80000
+#define 	DRC_WEIRDONES		(DRC_AREA|DRC_MAXWIDTH|DRC_RECTSIZE|DRC_ANGLES)
 
 #define	DRC_PENDING			0
-#define DRC_UNPROCESSED 		MINFINITY
+#define DRC_UNPROCESSED 		CLIENTDEFAULT
 #define DRC_PROCESSED 			1
 
 /* This is client data passed down through the various DRC checking
@@ -87,8 +88,6 @@ struct drcClientData
 					 */
     Rect	* dCD_clip;		/* Clip error tiles against this. */
     int		* dCD_errors;		/* Count of errors found. */
-    int		  dCD_which;		/* tells which edge of initial we 
-    					   started from */
     int		  dCD_radial;		/* Radial check for corner extensions */
     DRCCookie	* dCD_cptr;		/* Rule being checked. */
     Rect	* dCD_constraint;	/* Constraint area from rule. */
@@ -152,7 +151,8 @@ typedef struct drcstyle
     char		*ds_name;	/* Name of this DRC style */
     DRCCookie        	*DRCRulesTbl[TT_MAXTYPES][TT_MAXTYPES];
     TileTypeBitMask     DRCExactOverlapTypes;
-    int			DRCScaleFactor;	/* Divide dist by this to get magic units */
+    int			DRCScaleFactorN; /* Divide dist by this to get magic units */
+    int			DRCScaleFactorD; /* Multiply dist by this to get magic units */
     int			DRCTechHalo;	/* largest action distance of design rules */
     int			DRCStepSize;	/* chunk size for decomposing large areas */
     drcWhyList		*DRCWhyList;
@@ -190,6 +190,9 @@ extern bool DRCEuclidean;	/* global flag to enable/disable
 				 * Euclidean distance measure
 				 */
 extern int  dbDRCDebug;
+extern bool DRCForceReload;	/* TRUE if we have to reload DRC on a
+				 * change of the CIF output style
+				 */
 
 extern DRCKeep  *DRCStyleList;	/* List of available DRC styles */
 extern DRCStyle *DRCCurStyle;	/* Current DRC style in effect */
@@ -222,6 +225,7 @@ extern void DRCTechStyleInit();
 extern void DRCTechFinal();
 extern void DRCTechRuleStats();
 extern void DRCTechScale();
+extern void DRCReloadCurStyle();
 
 extern void DRCInit();
 extern void DRCContinuous();
