@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFread.c,v 1.2 2001/01/12 22:12:35 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extflat/EFread.c,v 1.2 2006/05/12 06:21:47 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -135,7 +135,7 @@ EFReadFile(name, dosubckt, resist, noscale)
 
     rc = efReadDef(def, dosubckt, resist, noscale);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
-    if (EFScale == 0) EFScale = 1;
+    if (EFScale == 0.0) EFScale = 1.0;
 
     return rc;
 }
@@ -171,7 +171,7 @@ efReadDef(def, dosubckt, resist, noscale)
     char line[1024], *argv[64], *name, *attrs;
     int rscale = 1;	/* Multiply resistances by this */
     int cscale = 1;	/* Multiply capacitances by this */
-    int lscale = 1;	/* Multiply lambda by this */
+    float lscale = 1.0;	/* Multiply lambda by this */
     FILE *inf;
     Use *use;
     Rect r;
@@ -227,18 +227,18 @@ readfile:
 		    efReadError("Bad capacitance scaling = 0; reset to 1.\n");
 		    cscale = 1;
 		}
-		lscale = atoi(argv[3]);
-		if (lscale == 0)
+		lscale = (float)atof(argv[3]);
+		if (lscale == 0.0)
 		{
 		    efReadError("Bad linear scaling = 0; reset to 1.\n");
-		    lscale = 1;
+		    lscale = 1.0;
 		}
 		if (noscale == FALSE)
 		{
 		    def->def_scale = lscale;
 		    if (EFScale != lscale)
 		    {
-			if (EFScale != 0) efScaleChanged = TRUE, EFScale = 1;
+			if (EFScale != 0) efScaleChanged = TRUE, EFScale = 1.0;
 			else EFScale = lscale;
 		    }
 		}
@@ -490,7 +490,8 @@ resistChanged:
 	    /* distance driver receiver min max */
 	    case DIST:
 		efBuildDist(def, argv[1], argv[2],
-			lscale*atoi(argv[3]), lscale*atoi(argv[4]));
+			(int)(lscale*atoi(argv[3])),
+			(int)(lscale*atoi(argv[4])));
 		break;
 
 	    /* killnode nodename */
@@ -503,9 +504,12 @@ resistChanged:
 		efBuildResistor(def, argv[1], argv[2], rscale*atoi(argv[3]));
 		break;
 
+	    /* To-do: compare timestamp against the cell */
+	    case TIMESTAMP:
+		break;
+
 	    /* Ignore the rest for now */
 	    case ADJUST:	/* Unused */
-	    case TIMESTAMP:	/* Unused */
 	    default:
 		break;
 	}
