@@ -689,6 +689,11 @@ extComputeCapLW(rlengthptr, rwidthptr)
  * to compute L and W for a resistor.  The sense of "length" and
  * "width" as used in the routine are appropriate for a transistor.
  *
+ * Also note that this algorithm will tend to over-estimate the width
+ * of transistors with angled bends.  This problem would be eliminated
+ * if non-Manhattan geometry were evaluated directly rather than being
+ * first converted to Manhattan geometry.
+ *
  * Results:
  *	None
  *
@@ -707,6 +712,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
     LinkedBoundary *lb, *lb2;
     int oppdir, length, loclength, testlen, width, locwidth, testwid;
     int segp, segn, segc, sege;
+    bool isComplex = FALSE;
 
     /* For each segment in the primary list, find the closest
      * segment in the other list which lies on the opposite
@@ -739,7 +745,13 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
     for (lb = extSpecialBounds[p]; lb != NULL; lb = lb->b_next)
     {
 	loclength = INFINITY;
-	oppdir = (lb->dir + 2) % 4;
+	switch (lb->dir)
+	{
+	    case BD_LEFT:   oppdir = BD_RIGHT;  break;
+	    case BD_RIGHT:  oppdir = BD_LEFT;   break;
+	    case BD_TOP:    oppdir = BD_BOTTOM; break;
+	    case BD_BOTTOM: oppdir = BD_TOP;    break;
+	}
 
 	/* First pass: Find the distance of the closest segment within	*/
 	/* the range of	its corner extension.  We do two passes because */
@@ -760,7 +772,15 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 				testlen = lb2->r.r_xbot - lb->r.r_xbot;
 				if (lb2->r.r_ybot < lb->r.r_ytop + testlen &&
 				    	lb2->r.r_ytop > lb->r.r_ybot - testlen)
+				{
+				    /* Adjustments for offset segments */
+				    if (lb2->r.r_ybot > lb->r.r_ytop)
+					testlen += lb2->r.r_ybot - lb->r.r_ytop;
+				    else if (lb2->r.r_ytop < lb->r.r_ybot)
+					testlen += lb->r.r_ybot - lb2->r.r_ytop;
+
 				    if (testlen < loclength) loclength = testlen;
+				}
 			    }
 			    break;
 			case BD_RIGHT:
@@ -769,7 +789,15 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 				testlen = lb->r.r_xtop - lb2->r.r_xtop;
 				if (lb2->r.r_ybot < lb->r.r_ytop + testlen &&
 				    	lb2->r.r_ytop > lb->r.r_ybot - testlen)
+				{
+				    /* Adjustments for offset segments */
+				    if (lb2->r.r_ybot > lb->r.r_ytop)
+					testlen += lb2->r.r_ybot - lb->r.r_ytop;
+				    else if (lb2->r.r_ytop < lb->r.r_ybot)
+					testlen += lb->r.r_ybot - lb2->r.r_ytop;
+
 				    if (testlen < loclength) loclength = testlen;
+				}
 			    }
 			    break;
 			case BD_TOP:
@@ -778,7 +806,15 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 				testlen = lb->r.r_ytop - lb2->r.r_ytop;
 				if (lb2->r.r_xbot < lb->r.r_xtop + testlen &&
 				    	lb2->r.r_xtop > lb->r.r_xbot - testlen)
+				{
+				    /* Adjustments for offset segments */
+				    if (lb2->r.r_xbot > lb->r.r_xtop)
+					testlen += lb2->r.r_xbot - lb->r.r_xtop;
+				    else if (lb2->r.r_xtop < lb->r.r_xbot)
+					testlen += lb->r.r_xbot - lb2->r.r_xtop;
+
 				    if (testlen < loclength) loclength = testlen;
+				}
 			    }
 			    break;
 			case BD_BOTTOM:
@@ -787,7 +823,15 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 				testlen = lb2->r.r_ybot - lb->r.r_ybot;
 				if (lb2->r.r_xbot < lb->r.r_xtop + testlen &&
 				    	lb2->r.r_xtop > lb->r.r_xbot - testlen)
+				{
+				    /* Adjustments for offset segments */
+				    if (lb2->r.r_xbot > lb->r.r_xtop)
+					testlen += lb2->r.r_xbot - lb->r.r_xtop;
+				    else if (lb2->r.r_xtop < lb->r.r_xbot)
+					testlen += lb->r.r_xbot - lb2->r.r_xtop;
+
 				    if (testlen < loclength) loclength = testlen;
+				}
 			    }
 			    break;
 		    }
@@ -821,8 +865,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 		width /= length;
 		length = loclength;
 	    } 
-	    TxError("Device has multiple lengths:  scaling"
-			" all widths to length %d\n", length);
+	    isComplex = TRUE;
 	}
 
 	/* fprintf(stderr, "   segment length = %d\n", loclength); */
@@ -913,6 +956,10 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
 
 	/* fprintf(stderr, "total L = %d, W = %d\n", length, width); */
 	/* fflush(stderr); */
+
+	if (isComplex)
+	    TxError("Device has multiple lengths:  scaling"
+			" all widths to length %d\n", length);
     }
 }
 	
@@ -1196,6 +1243,8 @@ extOutputTrans(def, transList, outFile)
 				extTransRec.tr_termnode[n] != NULL; n++)
 		{
 		    width += extTransRec.tr_termlen[n];
+
+		    /* Mark annular transistors as requiring extra processing */
 		    if (extTransRec.tr_termvector[n].p_x == 0 &&
 				extTransRec.tr_termvector[n].p_y == 0)
 			isAnnular = TRUE;
@@ -1215,7 +1264,14 @@ extOutputTrans(def, transList, outFile)
 		/* device.						*/
 		/*------------------------------------------------------*/
 
+		/* NOTE: tr_termvector for non-Manhattan bent devices	*/
+		/* is the same as for a single rectangle.		*/
+
+#ifdef NONMANHATTAN
+		if (1)
+#else
 		if (isAnnular)
+#endif
 		{
 		    LinkedBoundary *lb;
 
@@ -1282,8 +1338,19 @@ extOutputTrans(def, transList, outFile)
 		    arg.fra_each = extResistorTileFunc;
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
-		    extSeparateBounds(n - 1);
-		    extComputeEffectiveLW(&width, &length, n);
+		    if (extSpecialBounds[0] != NULL)
+		    {
+			extSeparateBounds(n - 1);
+			extComputeEffectiveLW(&width, &length, n);
+		    }
+		    else
+		    {
+			if (ExtDoWarn)
+			    extTransBad(def, reg->treg_tile,
+					"Could not determine resistor boundary");
+			width = 0;
+			length = 0;
+		    }
 
 		    /* Free the lists */
 
@@ -1506,8 +1573,9 @@ extTransFindSubsFunc(tile, pnreg)
  */
 
 int
-extTransTileFunc(tile)
+extTransTileFunc(tile, pNum)
     Tile *tile;
+    int pNum;
 {
     TileTypeBitMask mask;
     TileType loctype;
@@ -1543,7 +1611,7 @@ extTransTileFunc(tile)
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
-    (void) extEnumTilePerim(tile, mask, extTransPerimFunc, (ClientData) NULL);
+    (void) extEnumTilePerim(tile, mask, pNum, extTransPerimFunc, (ClientData)NULL);
     return (0);
 }
 
@@ -1650,7 +1718,10 @@ extTransPerimFunc(bp)
      * Total perimeter (separate from terminals, for dcaps
      * that might not be surrounded by terminals on all sides).
      */
-    extTransRec.tr_perim += len;
+
+    /* Don't double-count contact perimeters (added by Tim 1/9/07) */
+    if (bp->b_plane == extTransRec.tr_gatenode->nreg_pnum)
+	extTransRec.tr_perim += len;
 
     return (0);
 }
@@ -1675,8 +1746,9 @@ extTransPerimFunc(bp)
  */
 
 int
-extAnnularTileFunc(tile)
+extAnnularTileFunc(tile, pNum)
     Tile *tile;
+    int pNum;
 {
     TileTypeBitMask mask;
     TileType loctype;
@@ -1696,7 +1768,7 @@ extAnnularTileFunc(tile)
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
-    (void) extEnumTilePerim(tile, mask, extSpecialPerimFunc, (ClientData) TRUE);
+    (void) extEnumTilePerim(tile, mask, pNum, extSpecialPerimFunc, (ClientData) TRUE);
     return (0);
 }
 
@@ -1714,7 +1786,7 @@ extAnnularTileFunc(tile)
  *
  * Ideally, one wants to call both of these functions to check
  * both the case of L > W and the case W > L, assuming that both
- * are
+ * are legal resistor layouts.
  *
  * Results:
  *	Returns 0 always.
@@ -1725,8 +1797,9 @@ extAnnularTileFunc(tile)
  */
 
 int
-extResistorTileFunc(tile)
+extResistorTileFunc(tile, pNum)
     Tile *tile;
+    int pNum;
 {
     TileTypeBitMask mask;
     TileType loctype;
@@ -1748,7 +1821,7 @@ extResistorTileFunc(tile)
     TTMaskSetMask(&mask, &ExtCurStyle->exts_transSDTypes[loctype]);
     TTMaskCom(&mask);
 
-    (void) extEnumTilePerim(tile, mask, extSpecialPerimFunc, (ClientData) FALSE);
+    extEnumTilePerim(tile, mask, pNum, extSpecialPerimFunc, (ClientData)FALSE);
 
     return (0);
 }
@@ -1764,13 +1837,52 @@ extSpecialPerimFunc(bp, sense)
     Boundary *bp;
     bool sense;
 {
-    TileType tinside = TiGetType(bp->b_inside);
-    TileType toutside = TiGetType(bp->b_outside);
+    TileType tinside, toutside;
     NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
     int thisterm;
     int extended;
     LinkedBoundary *newBound, *lb, *lastlb;
 
+#ifdef NONMANHATTAN
+    /* Note that extEnumTilePerim() assumes for the non-Manhattan case	*/
+    /* that non-Manhattan tiles should be incorporated into the device	*/
+    /* gate for purposes of computing effective length and width.  In	*/
+    /* most cases this will be only a slight deviation from the true	*/
+    /* result.								*/
+
+    if (IsSplit(bp->b_inside))
+    {
+	switch (bp->b_direction)
+	{
+	    case BD_TOP:
+		tinside = TiGetBottomType(bp->b_inside);
+		toutside = TiGetBottomType(bp->b_outside);
+		break;
+	    case BD_BOTTOM:
+		tinside = TiGetTopType(bp->b_inside);
+		toutside = TiGetTopType(bp->b_outside);
+		break;
+	    case BD_RIGHT:
+		tinside = TiGetLeftType(bp->b_inside);
+		toutside = TiGetLeftType(bp->b_outside);
+		break;
+	    case BD_LEFT:
+		tinside = TiGetRightType(bp->b_inside);
+		toutside = TiGetRightType(bp->b_outside);
+		break;
+	}
+    }
+    else
+    {
+	tinside = TiGetTypeExact(bp->b_inside);
+	toutside = TiGetTypeExact(bp->b_outside);
+    }
+
+#else
+    tinside = TiGetType(bp->b_inside);
+    toutside = TiGetType(bp->b_outside);
+#endif
+
     if (!sense || TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside], toutside))
     {
 	/*
@@ -2141,7 +2253,10 @@ extTransEach(tile, pNum, arg)
 	    reg->treg_tile = tile;
 #endif
 
-    reg->treg_area += area;
+    /* Don't double-count area on contacts (added by Tim 1/9/07) */
+    if (pNum == DBPlane(TiGetType(tile)))
+	reg->treg_area += area;
+
     extSetNodeNum((LabRegion *) reg, pNum, tile);
 
     return (0);
