@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtHier.c,v 1.5 2001/07/23 17:49:52 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtHier.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -400,8 +400,8 @@ extOutputConns(table, outf)
 		/* Subsequent merges */
 		for (nn = nnext; nnext = nn->nn_next; nn = nnext)
 		{
-		    fprintf(outf, "merge \"%s\" \"%s\" %lg\n",
-				    nn->nn_name, nnext->nn_name, c);
+		    fprintf(outf, "merge \"%s\" \"%s\"\n",
+				    nn->nn_name, nnext->nn_name);
 		    nn->nn_node = (Node *) NULL;	/* Processed */
 		}
 	    }
