@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtLength.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtLength.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -653,7 +653,7 @@ extPathPairDistance(lab1, lab2, pMin, pMax)
 	{
 	    epa.epa_pNum = pNum;
 	    (void) DBSrPaintClient((Tile *) NULL, extPathDef->cd_planes[pNum],
-		    &r, &mask, (ClientData) MINFINITY,
+		    &r, &mask, (ClientData) CLIENTDEFAULT,
 		    extPathPairFunc, (ClientData) &epa);
 	}
 
@@ -673,13 +673,13 @@ extPathPairDistance(lab1, lab2, pMin, pMax)
  * extPathResetClient --
  *
  * Called by DBSrPaintClient() on behalf of extPathPairDistance()
- * above to reset each tile's ti_client field to MINFINITY.
+ * above to reset each tile's ti_client field to CLIENTDEFAULT.
  *
  * Results:
  *	Always returns 0.
  *
  * Side effects:
- *	Sets tile->ti_client to MINFINITY.
+ *	Sets tile->ti_client to CLIENTDEFAULT.
  * ----------------------------------------------------------------------------
  */
 
@@ -687,7 +687,7 @@ int
 extPathResetClient(tile)
     Tile *tile;
 {
-    tile->ti_client = (ClientData) MINFINITY;
+    tile->ti_client = (ClientData) CLIENTDEFAULT;
     return (0);
 }
 
@@ -697,7 +697,7 @@ extPathResetClient(tile)
  * extPathPairFunc --
  *
  * Called by DBSrPaintClient() on behalf of extPathPairDistance()
- * above for each unprocessed tile (ti_client == MINFINITY)
+ * above for each unprocessed tile (ti_client == CLIENTDEFAULT)
  * overlapped by epa->epa_lab1 that is connected to it.
  * Floods outward in depth-first search toward the destination
  * epa->epa_lab2.  Remembers the min and max delay to the
@@ -855,7 +855,7 @@ extPathFlood(tile, p, distance, epa)
 		plane->pl_hint = tp;
 
 		/* If not yet visited, process tp */
-		if (tp->ti_client == (ClientData) MINFINITY
+		if (tp->ti_client == (ClientData) CLIENTDEFAULT
 			&& DBConnectsTo(type, TiGetType(tp)))
 		{
 		    epa->epa_pNum = pNum;
@@ -892,7 +892,7 @@ extPathFlood(tile, p, distance, epa)
 		epa->epa_pNum = pNum;
 		(void) DBSrPaintClient((Tile *) NULL,
 			extPathDef->cd_planes[pNum], &biggerArea,
-			&DBConnectTbl[type], (ClientData) MINFINITY,
+			&DBConnectTbl[type], (ClientData) CLIENTDEFAULT,
 			extPathFloodFunc, (ClientData) &epfa);
 	    }
 	epa->epa_pNum = saveplane;
