@@ -842,17 +842,13 @@ ExtTechSimpleAreaCap(argc, argv)
 	    pshield |= PlaneNumToMaskBit(plane3);
 	}
 	else if (pnum3 <= pnum2)
+	{
 	    TTMaskSetMask(&subtypes, &DBPlaneTypes[plane3]);
+	    TTMaskClearType(&subtypes, TT_SPACE);
+	}
+	TTMaskClearType(&shields, TT_SPACE);
     }
 
-    /* Remove contacts from the shield types---contact parasitics	 */
-    /* are computed from the contact residues, not the contact type. */
-
-    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
-	if (TTMaskHasType(&shields, t))
-	    if (DBIsContact(t))
-		TTMaskClearType(&shields, t);
-
     /* Now record all of the overlap capacitances */
 
     for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
@@ -860,7 +856,7 @@ ExtTechSimpleAreaCap(argc, argv)
 	if (TTMaskHasType(&types, s))
 	{
 	    /* Contact overlap caps are determined from residues */
-	    if (DBIsContact(s)) continue;
+    	    if (DBIsContact(s)) continue;
 
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	    {
@@ -966,23 +962,19 @@ ExtTechSimplePerimCap(argc, argv)
 	    pshield |= PlaneNumToMaskBit(plane3);
 	}
 	else if (pnum3 <= pnum2)
+	{
 	    TTMaskSetMask(&subtypes, &DBPlaneTypes[plane3]);
+	}
     }
-
-    /* Remove contacts from the shield types---contact parasitics	 */
-    /* are computed from the contact residues, not the contact type. */
-
-    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
-	if (TTMaskHasType(&shields, t))
-	    if (DBIsContact(t))
-		TTMaskClearType(&shields, t);
+    TTMaskClearType(&shields, TT_SPACE);
+    TTMaskClearType(&subtypes, TT_SPACE);
 
     /* Record all of the sideoverlap capacitances */
 
     for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
     {
 	/* Side overlap computed from residues */
-	if (DBIsContact(s)) continue;
+    	if (DBIsContact(s)) continue;
 
 	if (!TTMaskHasType(&types, s))
 	{
@@ -992,7 +984,7 @@ ExtTechSimplePerimCap(argc, argv)
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskHasType(&nottypes, t)) continue;
-		if (DBIsContact(t)) continue;
+  		if (DBIsContact(t)) continue;
 
 		TTMaskSetMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t], &subtypes);
 		ExtCurStyle->exts_sideOverlapOtherPlanes[s][t] |=
@@ -1001,7 +993,7 @@ ExtTechSimplePerimCap(argc, argv)
 		cnew->ec_cap = capVal;
 		cnew->ec_far = shields;		/* Types that shield */
 		cnew->ec_near = subtypes;	/* Types we create cap with */
-		cnew->ec_plane = plane2;
+		cnew->ec_pmask = PlaneNumToMaskBit(plane2);
 		cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
 		ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
 
@@ -1019,7 +1011,7 @@ ExtTechSimplePerimCap(argc, argv)
 	    TTMaskSetType(&ExtCurStyle->exts_sideTypes[plane2], s);
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	    {
-		if (DBIsContact(t)) continue;
+  		if (DBIsContact(t)) continue;
 
 		TTMaskSetMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t], &types);
 		ExtCurStyle->exts_sideOverlapOtherPlanes[s][t] |=
@@ -1028,7 +1020,7 @@ ExtTechSimplePerimCap(argc, argv)
 		cnew->ec_cap = capVal;
 		cnew->ec_far = shields;		/* Types that shield */
 		cnew->ec_near = types;		/* Types we create cap with */
-		cnew->ec_plane = plane1;
+		cnew->ec_pmask = PlaneNumToMaskBit(plane1);
 		cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
 		ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
 
@@ -1097,7 +1089,7 @@ ExtTechSimpleSidewallCap(argv)
 		cnew->ec_near = types2;
 		cnew->ec_far = types1;
 		cnew->ec_next = ExtCurStyle->exts_sideCoupleCap[s][t];
-		cnew->ec_plane = -1;
+		cnew->ec_pmask = 0;
 		ExtCurStyle->exts_sideCoupleCap[s][t] = cnew;
 	    }
 	}
@@ -1166,15 +1158,7 @@ ExtTechSimpleOverlapCap(argv)
 	    pshield |= PlaneNumToMaskBit(plane3);
 	}
     }
-
-    /* Remove contacts from the shield types---contact parasitics	 */
-    /* are computed from the contact residues, not the contact type. */
-
-    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
-	if (TTMaskHasType(&shields, t))
-	    if (DBIsContact(t))
-		TTMaskClearType(&shields, t);
-
+    TTMaskClearType(&shields, TT_SPACE);
 
     /* Now record all of the overlap capacitances */
 
@@ -1183,12 +1167,12 @@ ExtTechSimpleOverlapCap(argv)
 	if (TTMaskHasType(&types1, s))
 	{
 	    /* Contact overlap caps are determined from residues */
-	    if (DBIsContact(s)) continue;
+  	    if (DBIsContact(s)) continue;
 
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskHasType(&types2, t)) continue;
-		if (DBIsContact(t)) continue;
+  		if (DBIsContact(t)) continue;
 
 		if (s == t) continue;	/* shouldn't happen */
 		if (plane1 == plane2) continue;  /* shouldn't happen */
@@ -1274,6 +1258,7 @@ ExtTechSimpleSideOverlapCap(argv)
 	    pshield |= PlaneNumToMaskBit(plane3);
 	}
     }
+    TTMaskClearType(&shields, TT_SPACE);
 
     /* Now record all of the sideoverlap capacitances */
 
@@ -1283,27 +1268,19 @@ ExtTechSimpleSideOverlapCap(argv)
 	return;
     }
 
-    /* Remove contact types from shields */
-
-    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
-	if (TTMaskHasType(&shields, t))
-	    if (DBIsContact(t))
-		TTMaskClearType(&shields, t);
-
-
     /* Record all of the sideoverlap capacitances */
 
     for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
     {
 	/* Side overlap computed from residues */
-	if (DBIsContact(s)) continue;
+  	if (DBIsContact(s)) continue;
 
-	if (!TTMaskHasType(&types, s))
+	if (TTMaskHasType(&types, s))
 	{
 	    ExtCurStyle->exts_sidePlanes |= PlaneNumToMaskBit(plane1);
 	    TTMaskSetType(&ExtCurStyle->exts_sideTypes[plane1], s);
 	    TTMaskSetMask(&ExtCurStyle->exts_sideEdges[s], &nottypes);
-	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	    for (t = TT_SPACE; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskHasType(&nottypes, t)) continue;
 		if (DBIsContact(t)) continue;
@@ -1315,7 +1292,7 @@ ExtTechSimpleSideOverlapCap(argv)
 		cnew->ec_cap = capVal;
 		cnew->ec_far = shields;		/* Types that shield */
 		cnew->ec_near = ov;		/* Types we create cap with */
-		cnew->ec_plane = plane2;
+		cnew->ec_pmask = PlaneNumToMaskBit(plane2);
 		cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
 		ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
 
@@ -1332,10 +1309,10 @@ ExtTechSimpleSideOverlapCap(argv)
 	    ExtCurStyle->exts_sidePlanes |= PlaneNumToMaskBit(plane2);
 	    TTMaskSetType(&ExtCurStyle->exts_sideTypes[plane2], s);
 	    TTMaskSetMask(&ExtCurStyle->exts_sideEdges[s], &notov);
-	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	    for (t = TT_SPACE; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskHasType(&notov, t)) continue;
-		if (DBIsContact(t)) continue;
+  		if (DBIsContact(t)) continue;
 
 		TTMaskSetMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t], &types);
 		ExtCurStyle->exts_sideOverlapOtherPlanes[s][t] |=
@@ -1344,7 +1321,7 @@ ExtTechSimpleSideOverlapCap(argv)
 		cnew->ec_cap = capVal;
 		cnew->ec_far = shields;		/* Types that shield */
 		cnew->ec_near = types;		/* Types we create cap with */
-		cnew->ec_plane = plane1;
+		cnew->ec_pmask = PlaneNumToMaskBit(plane1);
 		cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
 		ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
 
@@ -2018,7 +1995,7 @@ ExtTechLine(sectionName, argc, argv)
 			bad = TRUE;
 			continue;
 		    }
-		    p1 = DBPlane(s), p2 = DBPlane(t);
+  		    p1 = DBPlane(s), p2 = DBPlane(t);
 		    if (p1 == p2)
 		    {
 			TechError("Can't have overlap capacitance between"
@@ -2079,23 +2056,22 @@ ExtTechLine(sectionName, argc, argv)
 	    capVal = aToCap(argv[4]);
 	    shield = DBZeroTypeBits;
 	    if (argc == 6) DBTechNoisyNameMask(argv[5], &shield);
-	    p3 = -1;
 	    if (TTMaskHasType(&types1, TT_SPACE))
 		TechError("Can't have space on inside of edge [ignored]\n");
 	    /* It's ok to have the overlap be to space as long as a plane is */
 	    /* specified.						     */
 	    if (TTMaskHasType(&ov, TT_SPACE))
 	    {
-	    	 if ((cp = index(argv[3],'/')) == NULL)
-		 {
-		      TechError("Must specify plane for sideoverlap to space\n");
-		 }
-		 cp++;
-		 p3 =  (spointertype) dbTechNameLookup(cp, &dbPlaneNameLists);
-		 if (p3 < 0)
-		 {
-		      TechError("Unknown overlap plane %s\n",argv[3]);
-		 }
+	    	if ((cp = index(argv[3],'/')) == NULL)
+		{
+		    TechError("Must specify plane for sideoverlap to space\n");
+		}
+		cp++;
+		p3 = (spointertype) dbTechNameLookup(cp, &dbPlaneNameLists);
+		if (p3 < 0)
+		    TechError("Unknown overlap plane %s\n",argv[3]);
+		else
+		    pov = PlaneNumToMaskBit(p3);
 	    }
 	    for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
 	    {
@@ -2128,7 +2104,7 @@ ExtTechLine(sectionName, argc, argv)
 		    cnew->ec_cap = capVal;
 		    cnew->ec_far = shield; /* Really types that shield */
 		    cnew->ec_near = ov;  /* Really types we create cap with */
-		    cnew->ec_plane = p3;
+		    cnew->ec_pmask = pov;
 		    cnew->ec_next = ExtCurStyle->exts_sideOverlapCap[s][t];
 		    ExtCurStyle->exts_sideOverlapCap[s][t] = cnew;
 
@@ -2189,7 +2165,7 @@ ExtTechLine(sectionName, argc, argv)
 		    cnew->ec_near = near;
 		    cnew->ec_far = far;
 		    cnew->ec_next = ExtCurStyle->exts_sideCoupleCap[s][t];
-		    cnew->ec_plane = -1;
+		    cnew->ec_pmask = 0;
 		    ExtCurStyle->exts_sideCoupleCap[s][t] = cnew;
 		}
 	    }
@@ -2348,6 +2324,64 @@ extTechFinalStyle(style)
 			&style->exts_typesByResistClass[s]);
     }
 
+    /*
+     * Residue check:
+     * We have ignored all contact types when parsing parasitic
+     * capacitances.  Now we need to add them.  For each contact
+     * type, add the contact type to the types lists accordingly.
+     * Note that we don't have to record any cap values, since the
+     * extraction routine dissolves contacts into their residue
+     * types when computing the parasitics.  But, the type must be
+     * in the type lists or contact tiles will be passed over during
+     * searches.
+     */
+
+    for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
+    {
+	TileTypeBitMask rmask;
+	PlaneMask pMask;
+	int p;
+	TileType q;
+
+	if (!DBIsContact(s)) continue;
+
+	pMask = DBTypePlaneMaskTbl[s];
+	for (p = 0; p < DBNumPlanes; p++)
+	{
+	    if (PlaneMaskHasPlane(pMask, p))
+	    {
+		TTMaskSetType(&style->exts_overlapTypes[p], s);
+		TTMaskSetType(&style->exts_sideTypes[p], s);
+	    }
+	}
+	DBFullResidueMask(s, &rmask);
+	for (r = TT_TECHDEPBASE; r < DBNumUserLayers; r++)
+	{
+	    if (!TTMaskHasType(&rmask, r)) continue;
+
+	    TTMaskSetMask(&style->exts_sideEdges[s], &style->exts_sideEdges[r]);
+
+	    for (q = TT_TECHDEPBASE; q < DBNumUserLayers; q++)
+	    {
+		if (TTMaskHasType(&style->exts_overlapOtherTypes[q], r)) 
+		    TTMaskSetType(&style->exts_overlapOtherTypes[q], s);
+
+	        for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
+		    if (TTMaskHasType(&style->exts_overlapShieldTypes[q][t], r)
+				&& !TTMaskHasType(&rmask, q)
+				&& !TTMaskHasType(&rmask, t))
+			TTMaskSetType(&style->exts_overlapShieldTypes[q][t], s);
+
+		/* For sideOverlap, t is "outtypes" and includes space, so we	*/
+		/* must count from TT_SPACE, not TT_TECHDEPBASE.		*/
+
+	        for (t = TT_SPACE; t < DBNumUserLayers; t++)
+		    if (TTMaskHasType(&style->exts_sideOverlapOtherTypes[q][t], r)) 
+			TTMaskSetType(&style->exts_sideOverlapOtherTypes[q][t], s);
+	    }
+	}
+    }
+
     /*
      * Consistency check:
      * If a type R shields S from T, make sure that R is listed as
@@ -2442,7 +2476,7 @@ zinit:
 	    /* exts_unitsPerLambda is in centimicrons, so divide by	*/
 	    /* 100 to get microns, and invert to get the scalefactor.	*/
 
-	    CapValue scalefac = 100.0 / style->exts_unitsPerLambda;
+	    CapValue scalefac = style->exts_unitsPerLambda / 100.0;
 	    CapValue sqfac = scalefac * scalefac;
 
 	    style->exts_areaCap[r] *= sqfac;
