@@ -4,7 +4,7 @@
  * Defines things shared internally by the extract module of Magic,
  * but not generally needed outside the extract module.
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/extract/extractInt.h,v 1.5 2006/11/10 18:17:52 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/extract/extractInt.h,v 1.7 2007/10/15 22:07:21 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -248,7 +248,7 @@ typedef struct
     Tile	*b_inside;	/* Pointer to tile just inside segment */
     Tile	*b_outside;	/* Pointer to tile just outside segment */
     Rect	 b_segment;	/* Actual coordinates of segment */
-    int		 b_direction;	/* Direction following segment (see below) */
+    u_char	 b_direction;	/* Direction following segment (see below) */
     int		 b_plane;	/* extract argument for extSideOverlap   */
 } Boundary;
 
@@ -256,7 +256,7 @@ typedef struct
 	((bp)->b_segment.r_xtop - (bp)->b_segment.r_xbot \
     +    (bp)->b_segment.r_ytop - (bp)->b_segment.r_ybot)
 
-/* Directions in which we can be following the boundary of a perimeter */
+/* Directions in which we can be following the boundary of a perimeter	*/
 
 #define	BD_LEFT		0	/* Inside is to right */
 #define	BD_TOP		1	/* Inside is below */
@@ -420,9 +420,8 @@ typedef struct edgecap
 					 * types that shield the edge from
 					 * the overlaped tile.
 					 */
-    int			 ec_plane;	/* if ec_near is TT_SPACE, ec_plane */
-    					/* specifies which plane is to be   */
-					/* used.  -1 if ec_near isn't space */
+    int			 ec_pmask;	/* specifies which planes are to be	*/
+					/* used.				*/
 } EdgeCap;
 
 
