@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/graphics/grMain.c,v 1.8 2001/04/13 20:02:59 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/graphics/grMain.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 /*
@@ -145,8 +145,6 @@ bool (*GrInitPtr)()		= NULL;
 void (*GrClosePtr)()		= NULL;
 void (*GrSetCMapPtr)()		= NULL;
 
-void (*GrEnableTabletPtr)()	= NULL;
-void (*GrDisableTabletPtr)()	= NULL;
 void (*GrSetCursorPtr)()	= NULL;
 void (*GrTextSizePtr)()		= NULL;
 void (*GrDrawGlyphPtr)()	= NULL;
@@ -165,6 +163,14 @@ int (*GrWindowIdPtr)()   	= NULL;
 char *(*GrWindowNamePtr)()	= NULL;
 bool (*GrGetCursorPosPtr)()   	= NULL;
 
+void (*GrEnableTabletPtr)()	= NULL;
+void (*GrDisableTabletPtr)()	= NULL;
+
+bool (*GrGetBackingStorePtr)() 	  = NULL;
+bool (*GrScrollBackingStorePtr)() = NULL;
+void (*GrPutBackingStorePtr)() 	  = NULL;
+void (*GrFreeBackingStorePtr)()   = NULL;
+void (*GrCreateBackingStorePtr)() = NULL;
 
 /* variables similar to the above, except that they are only used
  * internal to the graphics package
@@ -197,15 +203,6 @@ char *grDStyleType = "7bit";
 char *grCMapType = "7bit";
 char *grCursorType = "bw";
 
-/* The following variable is a device-dependent limit on how many
- * stipples are permitted.  It defaults to something, but may be reset
- * by device initializers to a different number, depending
- * on what the devices can actually support.  Must not be greater
- * than GR_NUM_STIPPLES.
- */
-
-int grMaxStipples = GR_NUM_STIPPLES;
-
 int grNumBitPlanes = 0;      /* Number of bit-planes we are using. */
 int grBitPlaneMask = 0;      /* Mask of the valid bit-plane bits. */
 
@@ -358,7 +355,7 @@ GrIsDisplay(disp1, disp2)
  *
  *	Try to guess what sort of machine we are on, and set the display
  *	ports and type appropriately.  This info is overridden by
- *	$CAD_HOME/lib/displays and by command line switches.
+ *	$CAD_ROOT/magic/displays and by command line switches.
  *
  * Results:
  *	None.
