@@ -65,7 +65,7 @@ grtoglDrawGrid (prect, outline, clip)
 
     xsize = prect->r_xtop - prect->r_xbot;
     ysize = prect->r_ytop - prect->r_ybot;
-    if (!xsize || !ysize || GRID_TOO_SMALL(xsize, ysize, GR_NUM_GRIDS))
+    if (!xsize || !ysize || GRID_TOO_SMALL(xsize, ysize))
 	return FALSE;
     
     xstart = prect->r_xbot % xsize;
@@ -82,7 +82,6 @@ grtoglDrawGrid (prect, outline, clip)
     hi = clip->r_ytop;
     for (x = xstart; x < (clip->r_xtop+1) << SUBPIXELBITS; x += xsize)
     {
-        if (snum >= GR_NUM_GRIDS) break;
 	shifted = x >> SUBPIXELBITS;
 	glVertex2i(shifted, low);
 	glVertex2i(shifted, hi);
@@ -94,7 +93,6 @@ grtoglDrawGrid (prect, outline, clip)
     hi = clip->r_xtop;
     for (y = ystart; y < (clip->r_ytop+1) << SUBPIXELBITS; y += ysize)
     {
-        if (snum >= GR_NUM_GRIDS) break;
 	shifted = y >> SUBPIXELBITS;
 	glVertex2i(low, shifted);
 	glVertex2i(hi, shifted);
