@@ -16,7 +16,7 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/graphics/graphicsInt.h,v 1.2 2006/06/16 16:16:12 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/graphics/graphicsInt.h,v 1.4 2007/10/09 19:03:22 tim Exp $"
  */
 
 #ifndef _GRAPHICSINT_H
@@ -84,15 +84,9 @@ extern bool grDriverInformed;
 extern void grInformDriver();
 
 /* Macro to check for a bogusly small grid. 
- * Turn off grid if there are more than GR_NUM_GRIDS gridlines on the
- * screen in either direction, or if they are less than 4 pixels apart.
+ * Turn off grid if gridlines are less than 4 pixels apart.
  */
-#define GR_NUM_GRIDS 160
-#define GRID_TOO_SMALL(x,y,max) ( \
-     ((((GrScreenRect.r_xtop - GrScreenRect.r_xbot) << SUBPIXELBITS) / (x)) \
-     >= (max)) || \
-     ((((GrScreenRect.r_ytop - GrScreenRect.r_ybot) << SUBPIXELBITS) / (y)) \
-     >= (max)) || \
+#define GRID_TOO_SMALL(x,y) ( \
      (((x) >> SUBPIXELBITS) < 4) || (((y) >> SUBPIXELBITS) < 4) \
 )
 
