@@ -11,14 +11,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/lef/defWrite.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/defWrite.c,v 1.3 2006/05/12 06:21:49 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef SYSV
 #include <string.h>
-#endif
 #include <errno.h>
 
 #include "tcltk/tclmagic.h"
@@ -220,7 +218,7 @@ defCountNets(rootDef, allSpecial)
     /* routine to initialize a few things such as the search path.	*/
     EFArgs(0, NULL, NULL, NULL, NULL);
 
-    EFScale = 0;	/* Allows EFScale to be set to the scale value	*/
+    EFScale = 0.0;	/* Allows EFScale to be set to the scale value	*/
 
     if (EFReadFile(rootDef->cd_name, TRUE, FALSE, TRUE))
     {
@@ -1514,6 +1512,7 @@ defGetType(ttype, lefptr)
     HashSearch hs;
     HashEntry *he;
     lefLayer *lefl;
+    int contact = DBIsContact(ttype) ? CLASS_VIA : CLASS_ROUTE;
 
     /* Pick up information from the original LefInfo hash table	*/
     /* entries created during read-in of the tech file.		*/
@@ -1524,7 +1523,7 @@ defGetType(ttype, lefptr)
 	while (he = HashNext(&LefInfo, &hs))
 	{
 	    lefl = (lefLayer *)HashGetValue(he);
-	    if (lefl && lefl->type == ttype)
+	    if (lefl && (lefl->type == ttype) && (contact == lefl->lefClass))
 	    {
 		if (lefptr) *lefptr = lefl;
 		return lefl->canonName;
