@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/lefWrite.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/lefWrite.c,v 1.2 2007/04/20 17:57:00 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -313,7 +313,7 @@ lefWriteGeometry(tile, cdata)
     if (lefdata->lefMode == LEF_MODE_OBSTRUCT)
 	if (tile->ti_client == (ClientData)1)
 	{
-	    tile->ti_client = (ClientData)MINFINITY;
+	    tile->ti_client = (ClientData)CLIENTDEFAULT;
 	    return 0;
 	}
 
