@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzMain.c,v 1.15 2006/10/21 04:36:07 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzMain.c,v 1.17 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 /*--- includes --- */
@@ -339,7 +339,6 @@ MZCopyParms(oldParms)
 	    /* copy the rC */
 	    *rCNew = *rCOld;
 
-	    /* allocate new blockage planes */
 	    rCNew->rc_routeType.rt_hBlock = DBNewPlane((ClientData) TT_SPACE);
 	    rCNew->rc_routeType.rt_vBlock = DBNewPlane((ClientData) TT_SPACE);
 	}
@@ -662,7 +661,7 @@ MZAddStart(point, type)
  *      passed to this func), also keeps list of marked tiles for cleanup. 
  *
  *      Tiles are marked with TRUE on the clientdata field.  The default 
- *      clientdata value of MINFINITY should be restored by the router 
+ *      clientdata value of CLIENTDEFAULT should be restored by the router 
  *      before it returns.
  *
  * ----------------------------------------------------------------------------
@@ -938,14 +937,14 @@ MZCleanupPath(pathList)
 				hdist, vdist,
 				path->rp_entry.p_x, path->rp_entry.p_y);
 
-			/* Replace orient code of one contact with 'X', */
+			/* Replace orient code of one contact with 'C', */
 			/* to be handled by mzPaintContact		*/
 
 			if (n1path->rp_extendCode > EC_ALL
-					&& n1path->rp_orient != 'X')
-			    spath->rp_orient = 'X';
+					&& n1path->rp_orient != 'C')
+			    spath->rp_orient = 'C';
 			else
-			    n1path->rp_orient = 'X';
+			    n1path->rp_orient = 'C';
 				
 			break;
 		    }
@@ -1276,26 +1275,26 @@ MZClean()
 	{
 	    List *l;
 
-	    /* Reset Marked tiles client fields to MINFINITY */
+	    /* Reset Marked tiles client fields to CLIENTDEFAULT */
 	    for(l=mzMarkedTilesList; l!=NULL; l=LIST_TAIL(l))
 	    {
 		Tile *tile = (Tile *) LIST_FIRST(l);
 		
 		/* Restore tile client field to its "unmarked" value */
-		tile->ti_client = (ClientData) MINFINITY;
+		tile->ti_client = (ClientData) CLIENTDEFAULT;
 	    }
 
 	    /* Dealloc list of marked tiles */
 	    ListDealloc(mzMarkedTilesList);
 	    mzMarkedTilesList = NULL;
 
-	    /* Reset Marked subcell client fields to MINFINITY */
+	    /* Reset Marked subcell client fields to CLIENTDEFAULT */
 	    for(l=mzMarkedCellsList; l!=NULL; l=LIST_TAIL(l))
 	    {
 		CellUse *cu = (CellUse *) LIST_FIRST(l);
 		
 		/* Restore celluse client field to its "unmarked" value */
-		cu->cu_client = (ClientData) MINFINITY;
+		cu->cu_client = (ClientData) CLIENTDEFAULT;
 	    }
 
 	    /* Dealloc list of marked cells */
