@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotPS.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotPS.c,v 1.2 2007/10/16 17:47:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -82,10 +82,13 @@ float fscale;
  *			a cross through it (used for contacts).
  * BORDER:		Same as CROSS, except draw the outline with
  *			no cross through it.
+ * SOLID:		This is the same as a solid stipple but renders
+ *			much faster.
  */
 
-#define CROSS -1
+#define CROSS  -1
 #define BORDER -2
+#define SOLID  -3
 
 /* The definitions below give the integers used for various PS
  * line drawing styles (brushes).
@@ -271,11 +274,13 @@ PlotPSTechLine(sectionName, argc, argv)
 	    stipple = CROSS;
 	else if (strcmp(argv[2], "B") == 0)
 	    stipple = BORDER;
+	else if (strcmp(argv[2], "S") == 0)
+	    stipple = SOLID;
 	else
 	{
 	    if (!StrIsInt(argv[2]))
 	    {
-		TechError("3rd field must be an integer or \"X\" or \"B\".\n");
+		TechError("3rd field must be an integer or \"S\", \"X\", or \"B\".\n");
 		return TRUE;
 	    }
 	    stipple = atoi(argv[2]);
@@ -313,11 +318,18 @@ PlotPSTechLine(sectionName, argc, argv)
  */
 
 void
-plotPSFlushRect()
+plotPSFlushRect(style)
+    int style;
 {
     if (curwidth > 0)
-	fprintf(file, "%d %d %d %d fb\n", curxbot, curybot,
-		curxbot + curwidth, curybot + curheight);
+    {
+	if (style == SOLID)
+	    fprintf(file, "%d %d %d %d ms\n", curxbot, curybot,
+			curwidth, curheight);
+	else
+	    fprintf(file, "%d %d %d %d fb\n", curxbot, curybot,
+			curxbot + curwidth, curybot + curheight);
+    }
 }
 
 /*
@@ -461,7 +473,8 @@ plotPSRect(rect, style)
     if ((y < 0) || (rect->r_ybot > bbox.r_ytop)) return;
     h = rect->r_ytop - rect->r_ybot;
 
-    fprintf(file, "%d %d %d %d m%c\n", x, y, w, h, (style == CROSS) ? 'x' : 'r');
+    fprintf(file, "%d %d %d %d m%c\n", x, y, w, h, (style == CROSS) ? 'x' :
+		(style == SOLID) ? 's' : 'r');
 }
 
 
@@ -530,7 +543,7 @@ plotPSPaint(tile, cxp)
 	TileType dinfo;
 	Point polyp[5];
 
-	plotPSFlushRect();
+	plotPSFlushRect(curStyle->grs_stipple);
 	plotPSFlushLine();
 	PSReset();
 
@@ -601,7 +614,7 @@ plotPSPaint(tile, cxp)
     }
     else
     {
-	plotPSFlushRect();
+	plotPSFlushRect(curStyle->grs_stipple);
 	curheight = height;
 	curwidth = width;
 	curxbot = xbot;
@@ -1282,7 +1295,7 @@ PlotPS(fileName, scx, layers, xMask)
 	TTMaskAndMask3(&curMask, layers, &curStyle->grs_layers);
 	(void) DBTreeSrTiles(scx, &curMask, xMask, plotPSPaint,
 		(ClientData) NULL);
-	plotPSFlushRect();
+	plotPSFlushRect(curStyle->grs_stipple);
 	plotPSFlushLine();
 	PSReset();
     }
@@ -1292,7 +1305,7 @@ PlotPS(fileName, scx, layers, xMask)
     if (TTMaskHasType(layers, L_CELL))
     {
 	(void) DBTreeSrCells(scx, xMask, plotPSCell, (ClientData) NULL);
-	plotPSFlushRect();
+	plotPSFlushRect(BORDER);
 	plotPSFlushLine();
     }
 
@@ -1304,7 +1317,7 @@ PlotPS(fileName, scx, layers, xMask)
 	TTMaskSetType(&curMask, TT_SPACE);
 	(void) DBTreeSrLabels(scx, &curMask, xMask, (TerminalPath *) NULL,
 		plotPSLabelBox, (ClientData) NULL);
-	plotPSFlushRect();
+	plotPSFlushRect(BORDER);
 	plotPSFlushLine();
 	PSReset();
         fprintf(file, "grestore\n");	/* end clipping rectangle */
