@@ -21,7 +21,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/plot/plotRutils.c,v 1.3 2001/01/12 22:13:10 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotRutils.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -123,7 +123,7 @@ Font *PlotFontList;		/* Linked list of all fonts that have
 void
 PlotRastInit()
 {
-#ifdef IS_BIG_ENDIAN
+#ifdef WORDS_BIGENDIAN
     int i;
 
     for (i = 0; i < 32; i++)
@@ -132,7 +132,7 @@ PlotRastInit()
 	rightBits[i] = PlotSwapBytes(rightBits[i]);
 	singleBit[i] = PlotSwapBytes(singleBit[i]);
     }
-#endif /* IS_BIG_ENDIAN */
+#endif /* WORDS_BIGENDIAN */
 }
 
 /*
