@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plow/PlowYank.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plow/PlowYank.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -461,7 +461,7 @@ plowUpdateCell(use, origDef)
     Transform newTrans;
     int x, y;
 
-    if (use->cu_client == (ClientData)MINFINITY || use->cu_client == (ClientData)0)
+    if (use->cu_client == (ClientData)CLIENTDEFAULT || use->cu_client == (ClientData)0)
 	return (0);
 
     /*
