@@ -4,17 +4,13 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResConDCS.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResConDCS.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
 #include <math.h>
-#ifdef	SYSV
-#include <string.h>
-#else
-#include <strings.h>
-#endif
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
