@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResPrint.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResPrint.c,v 1.2 2006/11/10 18:17:53 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -379,11 +379,12 @@ ResPrintFHNodes(fp, nodelist, nodename, nidx, celldef)
 		(contact_res->rr_cl > 1 ||
 		contact_res->rr_width > 1))
 	{
-	    int i, j, del;
-	    float cx, cy, cxb, cyb;
+	    int i, j, edge, spacing;
+	    float del, cx, cy, cxb, cyb;
 
-	    del = ExtCurStyle->exts_viaSpacing[contact_res->rr_tt]
-			 + ExtCurStyle->exts_viaSize[contact_res->rr_tt];
+	    CIFGetContactSize(contact_res->rr_tt, &edge, &spacing, NULL);
+
+	    del = (float)(spacing + edge) / (oscale * 100);
 
 	    cxb = (float)(contact_res->rr_cl - 1) / 2;
 	    for (i = 0; i < contact_res->rr_cl; i++)
@@ -499,6 +500,7 @@ ResPrintFHRects(fp, reslist, nodename, eidx)
 {
     resResistor	*resistors;
     float	oscale, thick, cwidth;
+    int		edge;
 
     if (fp == NULL) return;
 
@@ -512,8 +514,10 @@ ResPrintFHRects(fp, reslist, nodename, eidx)
 	{
 	    int i, j;
 
-	    cwidth = (float)ExtCurStyle->exts_viaSize[resistors->rr_tt];
-	    cwidth *= oscale;
+	    CIFGetContactSize(resistors->rr_tt, &edge, NULL, NULL);
+
+	    /* 100 is for centimicrons to microns conversion */
+	    cwidth = (float)edge / 100;
 
 	    /* for contacts, rr_cl = squares in x, rr_width = squares in y */
 
@@ -545,8 +549,9 @@ ResPrintFHRects(fp, reslist, nodename, eidx)
 
 	    if (DBIsContact(resistors->rr_tt))
 	    {
-		cwidth = (float)ExtCurStyle->exts_viaSize[resistors->rr_tt];
-		cwidth *= oscale;
+		CIFGetContactSize(resistors->rr_tt, &edge, NULL, NULL);
+		/* 100 for centimicrons to microns conversion */
+		cwidth = (float)edge / 100;
 		fprintf(fp, " w=%1.2f h=%1.2f\n", cwidth, cwidth);
 	    }
 	    else
