@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResUtils.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResUtils.c,v 1.2 2007/04/20 17:57:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -151,7 +151,7 @@ ResAddPlumbing(tile, arg)
      {
      	  resTransStack = StackNew(64);
      }
-     if (tile->ti_client == (ClientData) MINFINITY)
+     if (tile->ti_client == (ClientData) CLIENTDEFAULT)
      {
 #ifdef NONMANHATTAN
 	  if (IsSplit(tile))
@@ -352,7 +352,7 @@ ResAddPlumbing(tile, arg)
 		    for (tp2= RT(tp1); RIGHT(tp2) > LEFT(tp1); tp2 = BL(tp2))
 		    {
 		     	  if ((TiGetBottomType(tp2) == t1) &&
-			      (tp2->ti_client == (ClientData) MINFINITY))
+			      (tp2->ti_client == (ClientData) CLIENTDEFAULT))
 			       {
      	  			    Junk = resAddField(tp2);
 				    STACKPUSH((ClientData)(tp2),resTransStack);
@@ -374,7 +374,7 @@ ResAddPlumbing(tile, arg)
 		     for (tp2= LB(tp1); LEFT(tp2) < RIGHT(tp1); tp2 = TR(tp2))
 		     {
 		     	  if ((TiGetTopType(tp2) == t1) &&
-			      (tp2->ti_client == (ClientData) MINFINITY))
+			      (tp2->ti_client == (ClientData) CLIENTDEFAULT))
 			       {
      	  			    Junk = resAddField(tp2);
 				    STACKPUSH((ClientData)(tp2),resTransStack);
@@ -395,7 +395,7 @@ ResAddPlumbing(tile, arg)
 		     for (tp2= TR(tp1); TOP(tp2) > BOTTOM(tp1); tp2 = LB(tp2))
 		     {
 		     	  if ((TiGetLeftType(tp2) == t1) &&
-			      (tp2->ti_client == (ClientData) MINFINITY))
+			      (tp2->ti_client == (ClientData) CLIENTDEFAULT))
 			       {
      	  			    Junk = resAddField(tp2);
 				    STACKPUSH((ClientData)(tp2),resTransStack);
@@ -416,7 +416,7 @@ ResAddPlumbing(tile, arg)
 		     for (tp2= BL(tp1); BOTTOM(tp2) < TOP(tp1); tp2 = RT(tp2))
 		     {
 		     	  if ((TiGetRightType(tp2) == t1) &&
-			      (tp2->ti_client == (ClientData) MINFINITY))
+			      (tp2->ti_client == (ClientData) CLIENTDEFAULT))
 			       {
      	  			    Junk = resAddField(tp2);
 				    STACKPUSH((ClientData)(tp2),resTransStack);
@@ -522,7 +522,7 @@ ResAddPlumbing(tile, arg)
  *
  * Results: returns 0
  *
- * Side Effects: frees up memory; resets tile->ti_client fields to MINFINITY
+ * Side Effects: frees up memory; resets tile->ti_client fields to CLIENTDEFAULT
  *
  *-------------------------------------------------------------------------
  */
@@ -534,10 +534,10 @@ ResRemovePlumbing(tile, arg)
 
 {
      
-     if (tile->ti_client != (ClientData) MINFINITY)
+     if (tile->ti_client != (ClientData) CLIENTDEFAULT)
      {
           freeMagic(((char *)(tile->ti_client)));
-	  tile->ti_client = (ClientData) MINFINITY;
+	  tile->ti_client = (ClientData) CLIENTDEFAULT;
      }
      return(0);
 }
@@ -725,7 +725,7 @@ resAddField(tile)
 
 {
         tileJunk *Junk;
-	if ((Junk=(tileJunk *)tile->ti_client) == (tileJunk *) MINFINITY)
+	if ((Junk=(tileJunk *)tile->ti_client) == (tileJunk *) CLIENTDEFAULT)
 	{
      	      Junk = (tileJunk *) mallocMagic((unsigned) (sizeof(tileJunk)));
 	      ResJunkInit(Junk);
