@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/select/selCreate.c,v 1.6 2001/07/30 17:16:22 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.2 2006/04/15 03:36:49 anonymous Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -115,7 +115,7 @@ SelectInit()
     }
     SelectUse = DBCellNewUse(SelectDef, (char *) NULL);
     DBSetTrans(SelectUse, &GeoIdentityTransform);
-    SelectUse->cu_expandMask = -1;	/* This is always expanded. */
+    SelectUse->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
 
     Select2Def = DBCellLookDef("__SELECT2__");
     if (Select2Def == (CellDef *) NULL)
@@ -127,7 +127,7 @@ SelectInit()
     }
     Select2Use = DBCellNewUse(Select2Def, (char *) NULL);
     DBSetTrans(Select2Use, &GeoIdentityTransform);
-    Select2Use->cu_expandMask = -1;	/* This is always expanded. */
+    Select2Use->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
     UndoEnable();
 
     SelUndoInit();
@@ -162,16 +162,19 @@ void
 SelectClear()
 {
     SearchContext scx;
+    Rect r;
     extern int selClearFunc();		/* Forward declaration. */
 
     if (SelectRootDef == NULL) return;
 
     scx.scx_area = SelectDef->cd_bbox;
 
-    if (!strcmp(SelectDef->cd_name, "__SELECT_NET__"))
+    /* Special handling when the cell is named __SELECT_NET__ 	*/
+
+    if (!strncmp(SelectDef->cd_name + 9, "NET", 3))
     {
 	SelNetRememberForUndo((CellDef *)NULL, (Point *)NULL,
-		TT_SPACE, FALSE, FALSE);
+			TT_SPACE, FALSE, FALSE);
 	DBCellRenameDef(SelectDef, "__SELECT__");
 	DBCellClearDef(SelectDef);
     }
@@ -211,12 +214,17 @@ SelectClear()
 	    if (selNDelete < MAXUSES) break;
 	}
 	selectLastUse = NULL;
+
 	SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
     }
 
+    /* Reset the transform, if we have been moving the selection around */
+    GeoTransRect(&SelectUse->cu_transform, &scx.scx_area, &r);
+    SelectUse->cu_transform = GeoIdentityTransform;
+
     /* Erase the selection from the screen. */
+    DBWHLRedraw(SelectRootDef, &r, TRUE);
 
-    DBWHLRedraw(SelectRootDef, &scx.scx_area, TRUE);
     DBReComputeBbox(SelectDef);
     DBWAreaChanged(SelectDef, &scx.scx_area, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
@@ -235,6 +243,7 @@ selClearFunc(scx)
     if (selNDelete == MAXUSES) return 1;
     else return 2;
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -740,7 +749,7 @@ SelectRegion(scx, type, xMask, pArea, less)
 	scx2.scx_trans = GeoIdentityTransform;
 	DBCellCopyAllPaint(&scx2, &DBAllButSpaceAndDRCBits,
 			0, SelectUse);
-	DBCellCopyAllLabels(&scx2, &DBAllTypeBits, -1, SelectUse,
+	DBCellCopyAllLabels(&scx2, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
 			(Rect *) NULL);
       }
 
@@ -846,7 +855,7 @@ SelectNet(scx, type, xMask, pArea, less)
 	scx2.scx_trans = GeoIdentityTransform;
 	DBCellCopyAllPaint(&scx2, &DBAllButSpaceAndDRCBits,
 			   0, SelectUse);
-	DBCellCopyAllLabels(&scx2, &DBAllTypeBits, -1, SelectUse,
+	DBCellCopyAllLabels(&scx2, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
 			(Rect *) NULL);
     }
 
@@ -970,13 +979,61 @@ SelectCell(use, rootDef, trans, replace)
     DBWAreaChanged(SelectDef, &newUse->cu_bbox, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * SelectAndCopy1 --
+ *
+ *	This procedure takes the contents of SelectDef,  and makes a
+ *	copy of them in the edit cell.  Unlike SelectAndCopy2, the
+ *	original selection is unchanged, and SelectDef is not cleared.
+ *	This allows the implementation of "drag and drop" selections.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The selection is augmented with what's in Select2Def.  The caller
+ *	should normally have cleared the selection before calling us.
+ *	The edit cell is modified to include everything that was in
+ *	Select2Def.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+SelectAndCopy1()
+{
+    SearchContext scx;
+    Rect editArea;
+
+    /* Just copy the information in Select2Def twice, once into the
+     * edit cell and once into the main selection cell.
+     */
+    
+    scx.scx_use = SelectUse;
+    scx.scx_area = SelectUse->cu_bbox;
+    GeoTransTrans(&SelectUse->cu_transform, &RootToEditTransform, &scx.scx_trans);
+    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+		EditCellUse);
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, EditCellUse,
+		(Rect *) NULL);
+    (void) DBCellCopyAllCells(&scx, CU_DESCEND_SPECIAL, EditCellUse, (Rect *) NULL);
+    GeoTransRect(&scx.scx_trans, &scx.scx_area, &editArea);
+    DBAdjustLabels(EditCellUse->cu_def, &editArea);
+    DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS,
+	(TileTypeBitMask *) NULL);
+    DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
+    DBReComputeBbox(EditCellUse->cu_def);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
  * SelectAndCopy2 --
  *
- * 	This is procedure is intended for use only within the selection
+ * 	This procedure is intended for use only within the selection
  *	module.  It takes what's in Select2Def, makes a copy of it in the
  *	edit cell, and makes the copy the selection.  It's used, for
  *	example, by the transformation and copying routines.
@@ -1013,11 +1070,13 @@ SelectAndCopy2(newSourceDef)
     scx.scx_use = Select2Use;
     scx.scx_area = Select2Use->cu_bbox;
     scx.scx_trans = RootToEditTransform;
-    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, -1, EditCellUse);
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, -1, EditCellUse,
-	(Rect *) NULL);
-    (void) DBCellCopyAllCells(&scx, -1, EditCellUse, (Rect *) NULL);
+    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+		EditCellUse);
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, EditCellUse,
+		(Rect *) NULL);
+    (void) DBCellCopyAllCells(&scx, CU_DESCEND_SPECIAL, EditCellUse, (Rect *) NULL);
     GeoTransRect(&scx.scx_trans, &scx.scx_area, &editArea);
+
     DBAdjustLabels(EditCellUse->cu_def, &editArea);
     DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
@@ -1045,7 +1104,7 @@ SelectAndCopy2(newSourceDef)
 		(ClientData) plane);
     }
 
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, -1, SelectUse,
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
 	    &labelArea);
 
     /* We also have to be careful about copying subcells into the
