@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/select/selOps.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selOps.c,v 1.3 2006/09/24 23:35:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -349,11 +349,11 @@ selTransCellFunc(selUse, realUse, realTrans, transform)
 
     /* ARGSUSED */
 int
-selTransLabelFunc(label, cellDef, defTransform, transform)
+selTransLabelFunc(label, cellUse, defTransform, transform)
     Label *label;		/* Label to copy.  This points to label
 				 * in cellDef.
 				 */
-    CellDef *cellDef;		/* Definition containing label in layout. */
+    CellUse *cellUse;		/* (unused) */
     Transform *defTransform;	/* Transform from cellDef to root. */
     Transform *transform;	/* How to modify coords before copying to
 				 * Select2Def.
@@ -626,9 +626,9 @@ selArrayCFunc(selUse, use, transform, arrayInfo)
 
     /* ARGSUSED */
 int
-selArrayLFunc(label, def, transform, arrayInfo)
+selArrayLFunc(label, use, transform, arrayInfo)
     Label *label;		/* Label to be copied and replicated. */
-    CellDef *def;		/* Definition containing label. */
+    CellUse *use;		/* (unused) */
     Transform *transform;	/* Transform from coords of def to root. */
     ArrayInfo *arrayInfo;	/* How to replicate. */
 {
@@ -778,7 +778,7 @@ SelectStretch(x, y)
     
     SelectAndCopy2(EditRootDef);
     DBWAreaChanged(EditCellUse->cu_def, &editModified, DBW_ALLWINDOWS,
-	    (TileTypeBitMask *) NULL);
+	(TileTypeBitMask *) NULL);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editModified);
 }
 
