@@ -91,7 +91,7 @@ TileTypeBitMask	SimFetMask[TT_MAXTYPES];
     /*
      * Mask of planes for which SimFetMask is non-zero.
      */
-int		SimFetPlanes;
+PlaneMask	SimFetPlanes;
 
 
 /*
@@ -356,10 +356,34 @@ int SimTransTerms( bp, trans )
 {
     TransTerm	*term;
     Tile	*tile = bp->b_outside;
+    TileType	type;
     NodeRegion	*reg = (NodeRegion *) tile->ti_client;
-    int		pNum = DBPlane( TiGetType( tile ) );
+    int		pNum;
     int		i;
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile)) {
+	switch(bp->b_direction) {
+	    case BD_LEFT:
+		type = TiGetRightType(tile);
+		break;
+	    case BD_RIGHT:
+		type = TiGetLeftType(tile);
+		break;
+	    case BD_TOP:
+		type = TiGetBottomType(tile);
+		break;
+	    case BD_BOTTOM:
+		type = TiGetTopType(tile);
+		break;
+	}
+    }
+    else
+#endif
+	type = TiGetTypeExact(tile);
+
+    pNum = DBPlane(type);
+
     for( i = 0; i < trans->t_nterm; i++ )
     {
 	term = &trans->t_term[i];
@@ -509,6 +533,7 @@ SimFindOneNode( sx, tile )
     CellDef		*def = sx->scx_use->cu_def;
     NodeRegion		*reg;
     FindRegion		arg;
+    TileType		type, loctype;
     static NodeSpec	ret;
 
 	/* Allocate a new node */
@@ -524,7 +549,15 @@ SimFindOneNode( sx, tile )
 	/* Find all connected paint in this cell */
     arg.fra_connectsTo = ExtCurStyle->exts_nodeConn;
     arg.fra_def = def;
-    arg.fra_pNum = DBPlane( TiGetType( tile ) );
+
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	type = SplitSide(tile) ? TiGetRightType(tile) : TiGetLeftType(tile);
+    else
+#endif
+	type = TiGetTypeExact(tile);
+
+    arg.fra_pNum = DBPlane(type);
     arg.fra_uninit = (ClientData) extUnInit;
     arg.fra_region = (Region *) reg;
     arg.fra_each = SimFindTxtor;
@@ -538,7 +571,16 @@ SimFindOneNode( sx, tile )
 
 	gateTile->ti_client = (ClientData) extUnInit;
 	arg.fra_connectsTo = &SimTransMask;
-	arg.fra_pNum = DBPlane( TiGetType( gateTile ) );
+
+#ifdef NONMANHATTAN
+	if (IsSplit(tile))
+	    loctype = SplitSide(gateTile) ? TiGetRightType(gateTile)
+			: TiGetLeftType(gateTile);
+	else
+#endif
+	    loctype = TiGetTypeExact(gateTile);
+
+	arg.fra_pNum = DBPlane(loctype);
 	arg.fra_uninit = (ClientData) extUnInit;
 	arg.fra_region = (Region *) reg;
 	arg.fra_each = SimTransistorTile;
@@ -546,7 +588,7 @@ SimFindOneNode( sx, tile )
 
 	    /* Unmark current region since not all paint was traced */
 	arg.fra_connectsTo = ExtCurStyle->exts_nodeConn;
-	arg.fra_pNum = DBPlane( TiGetType( tile ) );
+	arg.fra_pNum = DBPlane(type);
 	arg.fra_uninit = (ClientData) reg;
 	arg.fra_region = (Region *) extUnInit;
 	arg.fra_each = (int (*)()) NULL;
@@ -570,7 +612,16 @@ SimFindOneNode( sx, tile )
 
 	/* collect the transistor position, and its terminals */
 	arg.fra_connectsTo = &SimTransMask;
-	arg.fra_pNum = DBPlane( TiGetType( sdTile ) );
+
+#ifdef NONMANHATTAN
+	if (IsSplit(tile))
+	    loctype = SplitSide(sdTile) ? TiGetRightType(sdTile)
+			: TiGetLeftType(sdTile);
+	else
+#endif
+	    loctype = TiGetTypeExact(sdTile);
+
+	arg.fra_pNum = DBPlane(loctype);
 	arg.fra_uninit = (ClientData) sdTile->ti_client;
 	arg.fra_region = (Region *) &ret;
 	arg.fra_each = SimTransistorTile;
