@@ -118,6 +118,8 @@ proc magic::popstack {} {
 
 proc magic::pushbox {{values {}}} {
    global boxstack
+   set snaptype [snap list]
+   snap internal
    if {[catch {set boxstack}]} {
       set boxstack {}
    }
@@ -126,6 +128,7 @@ proc magic::pushbox {{values {}}} {
    } else {
       lappend boxstack $values
    }
+   snap $snaptype
    return
 }
 
@@ -139,6 +142,8 @@ proc magic::pushbox {{values {}}} {
 
 proc magic::popbox {{type values}} {
    global boxstack
+   set snaptype [snap list]
+   snap internal
    if {[catch {set boxstack}]} {
       error "No stack"
    } elseif {$boxstack == {}} {
@@ -158,6 +163,7 @@ proc magic::popbox {{type values}} {
       }
    }
    set boxstack [lrange $boxstack 0 end-1]
+   snap $snaptype
    return $b
 }
 
