@@ -1115,11 +1115,14 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 	-background sienna4 -anchor w -padx 5
 
    # Menu buttons
+   frame ${framename}.titlebar.mbuttons
 
-   menubutton ${framename}.titlebar.filebutton -text File -relief raised \
-		-menu ${framename}.titlebar.filebutton.filemenu -borderwidth 2
-   menubutton ${framename}.titlebar.optsbutton -text Options -relief raised \
-		-menu ${framename}.titlebar.optsbutton.optsmenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.file -text File -relief raised \
+		-menu ${framename}.titlebar.mbuttons.file.filemenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.opts -text Options -relief raised \
+		-menu ${framename}.titlebar.mbuttons.opts.optsmenu -borderwidth 2
+   pack ${framename}.titlebar.mbuttons.file -side left
+   pack ${framename}.titlebar.mbuttons.opts -side left
 
    # DRC status button
    checkbutton ${framename}.titlebar.drcbutton -text "DRC" -anchor w \
@@ -1148,13 +1151,12 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    grid rowconfigure ${framename} 1 -weight 1
    grid columnconfigure ${framename} 1 -weight 1
 
-   grid ${framename}.titlebar.filebutton -row 0 -column 0 -sticky news
-   grid ${framename}.titlebar.optsbutton -row 0 -column 1 -sticky news
-   grid ${framename}.titlebar.drcbutton -row 0 -column 2 -sticky news
-   grid ${framename}.titlebar.caption -row 0 -column 3 -sticky news
-   grid ${framename}.titlebar.message -row 0 -column 4 -sticky news
-   grid ${framename}.titlebar.pos -row 0 -column 5 -sticky news
-   grid columnconfigure ${framename}.titlebar 3 -weight 1
+   grid ${framename}.titlebar.mbuttons -row 0 -column 0 -sticky news
+   grid ${framename}.titlebar.drcbutton -row 0 -column 1 -sticky news
+   grid ${framename}.titlebar.caption -row 0 -column 2 -sticky news
+   grid ${framename}.titlebar.message -row 0 -column 3 -sticky news
+   grid ${framename}.titlebar.pos -row 0 -column 4 -sticky news
+   grid columnconfigure ${framename}.titlebar 2 -weight 1
 
    bind $winname <Enter> "focus %W ; set Opts(focus) $framename"
 
@@ -1177,7 +1179,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    # Generate the file menu
 
-   set m [menu ${framename}.titlebar.filebutton.filemenu -tearoff 0]
+   set m [menu ${framename}.titlebar.mbuttons.file.filemenu -tearoff 0]
    $m add command -label "New window" -command "magic::openwrapper \
 		[${winname} cellname list window]"
    $m add command -label "Close" -command "magic::closewrapper ${framename}"
@@ -1198,7 +1200,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    # Generate the options menu
 
-   set m [menu ${framename}.titlebar.optsbutton.optsmenu -tearoff 0]
+   set m [menu ${framename}.titlebar.mbuttons.opts.optsmenu -tearoff 0]
    $m add check -label "Toolbar" -variable Winopts(${framename},toolbar) \
 	-command [subst {if { \$Winopts(${framename},toolbar) } { \
 		magic::maketoolbar ${framename} ; \
@@ -1287,10 +1289,10 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    wm protocol ${framename} WM_DELETE_WINDOW "closewrapper $framename"
 
    # If the variable $Opts(callback) is defined, then attempt to execute it.
-   catch {$Opts(callback)}
+   catch {eval $Opts(callback)}
 
    # If the variable $Winopts(callback) is defined, then attempt to execute it.
-   catch {$Winopts(${framename}, callback)}
+   catch {eval $Winopts(${framename}, callback)}
 
    return ${winname}
 }
