@@ -17,7 +17,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/tiles/tile.c,v 1.3 2006/11/14 10:40:03 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/tiles/tile.c,v 1.5 2007/10/09 19:00:37 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -249,7 +249,7 @@ TiSplitX(tile, x)
     ASSERT(x > LEFT(tile) && x < RIGHT(tile), "TiSplitX");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
 
     LEFT(newtile) = x;
@@ -321,7 +321,7 @@ TiSplitY(tile, y)
     ASSERT(y > BOTTOM(tile) && y < TOP(tile), "TiSplitY");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
@@ -394,7 +394,7 @@ TiSplitX_Left(tile, x)
     ASSERT(x > LEFT(tile) && x < RIGHT(tile), "TiSplitX");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
@@ -457,7 +457,7 @@ TiSplitY_Bottom(tile, y)
     ASSERT(y > BOTTOM(tile) && y < TOP(tile), "TiSplitY");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
@@ -665,7 +665,7 @@ static signed char
 mmapTileStore()
 {
     int prot = PROT_READ | PROT_WRITE;
-    int flags = MAP_ANONYMOUS | MAP_PRIVATE;
+    int flags = MAP_ANON | MAP_PRIVATE;
     unsigned long map_len = TILE_STORE_BLOCK_SIZE; 	
 
     _block_begin = mmap(NULL, map_len, prot, flags, -1, 0);
@@ -741,7 +741,7 @@ TiAlloc()
     Tile *newtile;
 
     newtile = getTileFromTileStore();
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
     return (newtile);
 }
@@ -774,7 +774,7 @@ TiAlloc()
     Tile *newtile;
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    TiSetClient(newtile, MINFINITY);
+    TiSetClient(newtile, CLIENTDEFAULT);
     TiSetBody(newtile, 0);
     return (newtile);
 }
