@@ -17,7 +17,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/tiles/tile.h,v 1.4 2006/11/14 10:40:03 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/tiles/tile.h,v 1.5 2007/04/20 17:57:00 tim Exp $"
  */
 
 #ifndef _TILES_H
@@ -60,7 +60,7 @@ typedef struct tile
     Point	 ti_ll;		/* Lower left coordinate */
     ClientData	 ti_client;	/* This space for hire.  Warning: the default
 				 * value for this field, to which all users
-				 * should return it when done, is MINFINITY
+				 * should return it when done, is CLNTDEFAULT
 				 * instead of NULL.
 				 */
 } Tile;
@@ -154,6 +154,13 @@ typedef struct
 #define	INFINITY	((1 << (8*sizeof (int) - 2)) - 4)
 #define	MINFINITY	(-INFINITY)
 
+/* CLIENTDEFAULT differs from MINFINITY on 64-bit systems, where it	*/
+/* prevents problems arising from MINFINITY being two different values	*/
+/* depending on whether it is cast into a 32 or a 64 bit word.		*/
+
+#define	CLIENTMAX	(((pointertype)1 << (8 * sizeof(pointertype) - 2)) - 4)
+#define	CLIENTDEFAULT	(-CLIENTMAX)
+
 /* ------------------------ Flags, etc -------------------------------- */
 
 #define	BADTILE		((Tile *) -1)	/* Invalid tile pointer */
